/* Poopmup - An urban flight sim
 *
 * Copyright (C) 2002 Zaroubi, Fink, Leclerc, Stanek szaroubi@hotmail.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/***************************************************************************
                          myBoundingBox.h  -  description
                             -------------------
    begin                : Tue Nov 5 2002
    copyright            : (C) 2002 by Sherif Zaroubi
    email                : szaroubi@users.sf.net
 ***************************************************************************/

#ifndef MYBOUNDINGBOX_H
#define MYBOUNDINGBOX_H

#include "../../common.h"
#include "myMatrix.h"
#include "myVector.h"
#include "my3dPoint.h"
#include "myLine.h"

#include <string>
using namespace std;

/**
 * IMPLEMENTS 2 TYPES OF BOUNDING BOXES
 * - Oriented Bounding Boxes (OBB)
 * - Spherical Bounding Boxes
 */

/*** The Idea where taken from the following paper
     --------------------------------------------
     ftp://ftp.cs.unc.edu/pub/users/manocha/PAPERS/COLLISION/sig96.pdf
***/

class myBoundingBox {
  
 private:
  
  
 public:

  //Do the 2 point arrays overlap on line l?
  //All points belong to line l .
  int overlap(my3dPoint *list1, my3dPoint *list2, myLine l);
  
  myBoundingBox();
  
  ~myBoundingBox() {
  };
  
  //Draw the BB
  void draw();

  //Used in Debuging.
  void print();
  
  
  void init();
  
  /*********** GETTER AND SETTER ***********/
  //get the 8 corners of the box;
  my3dPoint getCorner(int i); 

  //get the 15 axis of projection used in obb collision detection.
  myLine getProjAxis(myBoundingBox &bb2, int i); 

  //get width of bounding box
  float getWidth(){
    return - lowerCloseCorner.getX() + upperFarCorner.getX() ;
  }
  //Get Height of bounding box.
  float getDepth(){
    return - lowerCloseCorner.getY() + upperFarCorner.getY();
  }

  //Get height of boundingbox
  float getHeight(){
    return - lowerCloseCorner.getZ() + upperFarCorner.getZ();
  }
  
  //TODO: Check if we still need them.
  my3dPoint getLowerCloseCorner(){return lowerCloseCorner;}
  my3dPoint getUpperFarCorner(){return upperFarCorner;}
  // ______________________________________
  
  /*********** TRANSFORMATIONS ************/
  void rotateAroundXBy(GLfloat angle);
  void rotateAroundYBy(GLfloat angle);
  void rotateAroundZBy(GLfloat angle);
  void translateBy(GLfloat x, GLfloat y, GLfloat z);
  void scaleBy(GLfloat scaleX, GLfloat scaleY, GLfloat scaleZ);
  void applyTransphormation(myMatrix m);
  void applyTransphormation(myMatrix *m);

  // ______________________________________
  int SphericalCollide(myBoundingBox bb2);
  int SphericalContains(my3dPoint p);

  //OBB Collision detection.
  int collide(myBoundingBox  bb2);

  my3dPoint lowerCloseCorner;
  my3dPoint upperFarCorner;
  my3dPoint getCenter();
};

my3dPoint myBoundingBox::getCenter(){
  my3dPoint retVal;
  retVal.setXYZ(
		(lowerCloseCorner.getX() + upperFarCorner.getX())/2,
		(lowerCloseCorner.getY() + upperFarCorner.getY())/2,
		(lowerCloseCorner.getZ() + upperFarCorner.getZ())/2
		);
  return retVal;
}

int myBoundingBox::SphericalContains(my3dPoint p){
  int retVal = 0;
  float dist;
  float collideDist;
  float w1;
  my3dPoint center = getCenter();
  float x1,x2,y1,y2,z1,z2;

  //Coordinates of center of this bounding box
  x1 = center.getX();
  y1 = center.getY();
  z1 = center.getZ();

  //Coordinats of center of other bounding box
  x2 = p.getX();
  y2 = p.getY();
  z2 = p.getZ();

  //Compute the Square of the distances.
  dist = (x1-x2)*(x1-x2);
  dist += (y1-y2)*(y1-y2);
  dist += (z1-z2)*(z1-z2);
  
  //Compute the radius of the spherical bounding boxs
  w1 = (getWidth() + getDepth()+ getHeight())/6;
 
  //Compute the square of the distance between the 2 "collision distance"
  collideDist = w1*w1;

  //Compare the distance between the 2 points to the "collision distance"
  if (dist < collideDist ) 
    retVal = 1;

  return retVal;
  return retVal;
}

int myBoundingBox::SphericalCollide(myBoundingBox bb2){
  float dist;
  float collideDist;
  float w1, w2;
  int retVal = 0;

  float x1,x2,y1,y2,z1,z2;

  //Coordinates of center of this bounding box
  x1 = (lowerCloseCorner.getX() + upperFarCorner.getX())/2;
  y1 = (lowerCloseCorner.getY() + upperFarCorner.getY())/2;
  z1 =  (lowerCloseCorner.getZ() + upperFarCorner.getZ())/2;

  //Coordinats of center of other bounding box
  x2 = (bb2.lowerCloseCorner.getX() + bb2.upperFarCorner.getX())/2;
  y2 = (bb2.lowerCloseCorner.getY() + bb2.upperFarCorner.getY())/2;
  z2 = (bb2.lowerCloseCorner.getZ() + bb2.upperFarCorner.getZ())/2;

  //Compute the Square of the distances.
  dist = (x1-x2)*(x1-x2);
  dist += (y1-y2)*(y1-y2);
  dist += (z1-z2)*(z1-z2);
  
  //Compute the radius of the spherical bounding boxs
  w1 = (getWidth() + getDepth()+ getHeight())/6;
  w2 = (bb2.getWidth() + bb2.getDepth() + bb2.getHeight())/6;

  //Compute the square of the distance between the 2 "collision distance"
  collideDist = (w1 + w2)*(w1+w2);

  //Compare the distance between the 2 points to the "collision distance"
  if (dist < collideDist ) 
    retVal = 1;

  return retVal;
  
}

//Does This Box collide with the other one ?
int myBoundingBox::collide(myBoundingBox bb2){
  // algo: if we come across any gap, then there is no collision.
  // collision requires that all the projections have an overlap.
  /**
   * Get projectionAxis (the seperating Axis)
   * Project both bouning boxes onto that axis
   * Check if projections will overlap
   * if no overlap => we KNOW that their is no collision ...break loop;
   * if overlap pass to next projAxis
   * do all axis or until there is no overlap
   */
  int retval = 1;
  my3dPoint thisBoxProj[8], otherBoxProj[8];
  myLine projAxis;
  myPlane pl;  
  
  for (int i=0; i < 15 && retval; i++){
    projAxis = getProjAxis(bb2, i);
    for (int cornerIdx = 0; cornerIdx<8; cornerIdx++){
      //The arrays contains the projected points onto the projection axis
      thisBoxProj[cornerIdx] = projAxis.getProjectionOf(getCorner(cornerIdx));
      otherBoxProj[cornerIdx] = projAxis.getProjectionOf(bb2.getCorner(cornerIdx));
      //cout<<"Box 1"; thisBoxProj[cornerIdx].print();cout<<endl;
      //cout<<"Box 2"; otherBoxProj[cornerIdx].print();cout<<endl;
    }
    
    if(!overlap(thisBoxProj, otherBoxProj, projAxis))
      {
	// no overlap - stop the loop
	retval = 0;
      }
    
  }
  
  return retval;
}



int myBoundingBox::overlap(my3dPoint *list1, my3dPoint *list2, myLine l)
{
  const int list_max(8); // ONLY 8 things in each array
  
  // 
  // if any point in list2 is between any points of list1, then there is an overlap, return true
  // All points lie on line l
  
  int list1_min(0), list1_max(0);
  int list2_min(0), list2_max(0);
  
  int i;
  myVector vect1, vect2;
  
  myVector minVect, maxVect;
  myVector minVect2, maxVect2;

  /**** BOOTSTRAP THE EXTREMETIES OF LIST1 *****/
  MAKE_VECTOR(vect1, list1[0], list1[1]);
  MAKE_VECTOR(vect2, list1[0], list1[2]);
  
  if (vect1.sameDirection(vect2)){
    if (vect1.getLength() > vect2.getLength()){
      list1_min = 1;
      list1_max = 2;
    }
    else{
      list1_min = 2;
      list1_max = 1;
    }
  }
  else{
    list1_min = 1;
    list1_max = 2;
  }


  /**** BOOTSTRAP THE EXTREMETIES OF LIST2 *****/
  MAKE_VECTOR(vect1, list2[0], list2[1]);
  MAKE_VECTOR(vect2, list2[0], list2[2]);
  
  if (vect1.sameDirection(vect2)){
    if (vect1.getLength() > vect2.getLength()){
      list2_min = 1;
      list2_max = 2;
    }
    else{
      list2_min = 2;
      list2_max = 1;
    }
  }
  else{
    list2_min = 1;
    list2_max = 2;
  }
  

  for(i = 3; i < list_max; ++i)
    {   
      // LIST 1 PROCESSING
      MAKE_VECTOR(vect1, list1[0], list1[i]);
      MAKE_VECTOR(minVect, list1[0], list1[list1_min]);
      MAKE_VECTOR(maxVect, list1[0], list1[list1_max]);
      if (minVect.sameDirection(vect1)){
	if (minVect.getLength() < vect1.getLength())
	  list1_min = i;
      }else{
	//minVect and vect1 are in opposite directions
	//-=> maxVect and vect1 are in same direction
	if (maxVect.getLength()< vect1.getLength())
	  list1_max = i;
      }
      
      //LIST 2 PROCESSING
      MAKE_VECTOR(vect1, list2[0], list2[i]);
      MAKE_VECTOR(minVect, list2[0], list2[list2_min]);
      MAKE_VECTOR(maxVect, list2[0], list2[list2_max]);
      if (minVect.sameDirection(vect1)){
	if (minVect.getLength() < vect1.getLength())
	  list2_min = i;
      }else{
	//minVect and vect1 are in opposite directions
	//-=> maxVect and vect1 are in same direction
	if (maxVect.getLength()< vect1.getLength())
	  list2_max = i;
      }
    }

  // IF MIN AND MAX IN SAME DIRECTION THEN MAKE THE POINT 0 AN EXTREMETY
  MAKE_VECTOR(vect1, list1[0], list1[list1_min]);
  MAKE_VECTOR(vect2, list1[0], list1[list1_max]);
  if (vect1.sameDirection(vect2)){
    if(vect1.getLength() > vect2.getLength()){
      list1_max = 0;
    }
    else{
      list1_min = 0;
    }
  }
  MAKE_VECTOR(vect1, list2[0], list2[list2_min]);
  MAKE_VECTOR(vect2, list2[0], list2[list2_max]);
  if (vect1.sameDirection(vect2)){
    if(vect1.getLength() > vect2.getLength()){
      list2_max = 0;
    }
    else{
      list2_min = 0;
    }
  }
  //______________________________________________________


  //***** THE ACTUALL CHECKING FOR OVERLAP OF THE EXTREMESE ****/
  MAKE_VECTOR(minVect, list1[list1_min], list2[list2_min]);
  MAKE_VECTOR(maxVect, list1[list1_max], list2[list2_min]);
  MAKE_VECTOR(minVect2, list1[list1_min], list2[list2_max]);
  MAKE_VECTOR(maxVect2, list1[list1_max], list2[list2_max]);
  int retVal = 1; // assume overlap
  if(   minVect.sameDirection(maxVect)
	&& minVect.sameDirection(minVect2)
	&& minVect.sameDirection(maxVect2)
	){
    // all in the same direction - no overlap
    retVal = 0;
  }

  return retVal;
}

myBoundingBox::myBoundingBox()
{
  init();
}

void myBoundingBox::init(){
  upperFarCorner.setXYZW(1,1,1,1);
  lowerCloseCorner.setXYZW(-1,-1,-1,1);
}

void myBoundingBox::translateBy(GLfloat x, GLfloat y, GLfloat z)
{
  myMatrix m;
  m.setTranslation(x,y,z);
  lowerCloseCorner = m * lowerCloseCorner;
  upperFarCorner = m * upperFarCorner;
}

void myBoundingBox::scaleBy(GLfloat x, GLfloat y, GLfloat z)
{
  myMatrix m;
  m.setScale(x,y,z);
  lowerCloseCorner = m * lowerCloseCorner;
  upperFarCorner = m * upperFarCorner;
}

void myBoundingBox::rotateAroundXBy(GLfloat angle){
  myMatrix m;
  m.setRotationAroundX(angle);
  
  upperFarCorner = m * upperFarCorner;
  lowerCloseCorner = m * lowerCloseCorner;
}

void myBoundingBox::rotateAroundYBy(GLfloat angle)
{
  myMatrix m;
  m.setRotationAroundY(angle);
  
  upperFarCorner = m * upperFarCorner;
  lowerCloseCorner = m * lowerCloseCorner;
}

void myBoundingBox::rotateAroundZBy(GLfloat angle)
{
  myMatrix m;
  m.setRotationAroundZ(angle);
  
  upperFarCorner = m * upperFarCorner;
  lowerCloseCorner = m * lowerCloseCorner;
}

void myBoundingBox::applyTransphormation(myMatrix m){
  upperFarCorner = m * upperFarCorner;
  lowerCloseCorner = m * lowerCloseCorner;
}

void myBoundingBox::applyTransphormation(myMatrix *m){
  upperFarCorner = (*m) * upperFarCorner;
  lowerCloseCorner = (*m) * lowerCloseCorner;
}

void myBoundingBox::print(){
  cout<<"Bounding Box Debug"<<endl;
  
  cout<<"\t LowerCloseCorner : ";
  lowerCloseCorner.print();
  cout<<endl;
  
  cout<<"\t UpperFarCorner : ";
  upperFarCorner.print();
  cout<<endl;
  
  cout<<"\t Width  :"<<getWidth()<<" | ";
  cout<<"Height :"<<getHeight()<<" | ";
  cout<<"Depth  :"<<getDepth();
  
}

my3dPoint myBoundingBox::getCorner(int i){
  my3dPoint p;
  switch(i){
  case 0:
    p = lowerCloseCorner;
    break;
  case 1:
    p = lowerCloseCorner;
    p.setX(p.getX() + getWidth());
    break;
  case 2:
    p = lowerCloseCorner;
    p.setX(p.getX() + getWidth());
    p.setY(p.getY() + getDepth());
    break;
  case 3:
    p = lowerCloseCorner;
    p.setY(p.getY() + getDepth());
    break;
  case 4:
    p = upperFarCorner;
    p.setX(p.getX() - getWidth());
    p.setY(p.getY() - getDepth());
    break;
  case 5:
    p = upperFarCorner;
    p.setY(p.getY() - getDepth());
    break;
  case 6:
    p = upperFarCorner;
    break;
  case 7:
    p = upperFarCorner;
    p.setX(p.getX() - getWidth());
    break;
  }
  
  return p;
}


void myBoundingBox::draw(){
  double x0 = lowerCloseCorner.getX();
  double y0 = lowerCloseCorner.getY();
  double z0 = lowerCloseCorner.getZ();
  
  double x1 = upperFarCorner.getX();
  double y1 = upperFarCorner.getY();
  double z1 = upperFarCorner.getZ();
  
  static GLfloat v[8][3];
  
  /*  indices of front, top, left, bottom, right, back faces  */
  static GLubyte indices[6][4] = {
    {4, 5, 6, 7}, {2, 3, 7, 6}, {0, 4, 7, 3},
    {0, 1, 5, 4}, {1, 5, 6, 2}, {0, 3, 2, 1}
  };
  
  v[0][0] = v[3][0] = v[4][0] = v[7][0] = x0;
  v[1][0] = v[2][0] = v[5][0] = v[6][0] = x1;
  v[0][1] = v[1][1] = v[4][1] = v[5][1] = y0;
  v[2][1] = v[3][1] = v[6][1] = v[7][1] = y1;
  v[0][2] = v[1][2] = v[2][2] = v[3][2] = z0;
  v[4][2] = v[5][2] = v[6][2] = v[7][2] = z1;
  
  glEnableClientState (GL_VERTEX_ARRAY);
  glVertexPointer (3, GL_FLOAT, 0, v);
  glDrawElements(GL_QUADS, 6*4, GL_UNSIGNED_BYTE, indices);
  glDisableClientState (GL_VERTEX_ARRAY);
}

myLine myBoundingBox::getProjAxis(myBoundingBox &bb2, int i){
  //For OBB's there are 15 axis to test to check if the bb collide.
  //The 3 axis of "this bounding box"
  //The 3 of the other bouning box
  //The 9 combination of 3 axis of this * 3 axis of other.
  
  myLine l;
  myLine l1[3];
  myLine l2[3];
  
  // lines for this OBB
  l1[0].setLine(getCorner(0), getCorner(1));	
  l1[1].setLine(getCorner(0), getCorner(4));
  l1[2].setLine(getCorner(0), getCorner(3));
  
  // lines for other OBB
  l2[0].setLine(bb2.getCorner(0), bb2.getCorner(1));	
  l2[1].setLine(bb2.getCorner(0), bb2.getCorner(4));
  l2[2].setLine(bb2.getCorner(0), bb2.getCorner(3));
  
  switch(i){
    
  case 0:
    l = l1[0];
    break;
  case 1:
    l = l1[1];
    break;
  case 2:
    l = l1[2];
    break;
    
  case 3:
    l = l2[0];
    break;
  case 4:
    l = l2[1];
    break;
  case 5:
    l = l2[2];
    break;
    
    // these 9 cases need the NORMAL of the plane described by two edges, one from each bounding box
    
  case 6: // case 0 and 3
    l.setLine(getCorner(0), l1[0].getDirection() * l2[0].getDirection());
    break;
  case 7: // case 0 and 4
    l.setLine(getCorner(0), l1[0].getDirection() * l2[1].getDirection());
    break;
  case 8: // case 0 and 5
    l.setLine(getCorner(0), l1[0].getDirection() * l2[2].getDirection());
    break;
    
    
  case 9: // case 1 and 3
    l.setLine(getCorner(0), l1[1].getDirection() * l2[0].getDirection());
    break;
  case 10: // case 1 and 3
    l.setLine(getCorner(0), l1[1].getDirection() * l2[1].getDirection());
    break;
  case 11:
    l.setLine(getCorner(0), l1[1].getDirection() * l2[2].getDirection());
    break;
    
    
  case 12:
    l.setLine(getCorner(0), l1[2].getDirection() * l2[0].getDirection());
    break;
  case 13:
    l.setLine(getCorner(0), l1[2].getDirection() * l2[1].getDirection());
    break;
  case 14:
    l.setLine(getCorner(0), l1[2].getDirection() * l2[2].getDirection());
    break;
    
  }
  
  return l;
}
#endif
