/* Poopmup - An urban flight sim
 *
 * Copyright (C) 2002 Zaroubi, Fink, Leclerc, Stanek szaroubi@hotmail.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/***************************************************************************
 	myCam.h  -  description
  	-------------------
  	begin                : Sun Oct 6 2002
  	copyright            : (C) 2002 by Sherif Zaroubi
	email                : fogof@localhost.localdomain
 ***************************************************************************/

/************* HELPER FUNCTIONS **********/
void glprintf(char *string,int x,int y){
  int i=0;
  glRasterPos2f(x,y);
  while(string[i]!='\0'){
		glutBitmapCharacter(GLUT_BITMAP_HELVETICA_10,string[i++]);
	}
}

#ifndef MYCAM_H
#define MYCAM_H

#define CAM_DEBUG FALSE
#define VIEW_SIZE 60
#define TRUE 1
#define FALSE 0
#define MODE_ORTHO 0
#define MODE_PRESPECTIVE 1

#include "../../common.h"
//fink: #include <GL/glut.h> included in common

#include <math.h>
#include "my3dPoint.h"
#include "myVector.h"

class myCam {

 protected:
  /** @dia:route 0,2;h,7,7,-3,8,3 */
  my3dPoint eye;
  /** @dia:route 0,2;h,7,7,-3,8,3 */
  my3dPoint lookAt;
  /** @dia:route 0,2;h,7,7,-3,8,3 */
  myVector up;
  int mode;   	
  GLfloat rollAngle, rotationAngleAroundX, rotationAngleAroundY;
  int enabled;
  GLdouble left;
  GLdouble right; 
  GLdouble bottom;
  GLdouble top; 
  GLdouble nearPlane;
  GLdouble farPlane;
  GLdouble viewAngle;
  GLfloat viewRatio;
  float theta, phi;

 public:
  myCam(){
    rollAngle = 0;
    moveTo(0,0,10);
    setLookAt(0,0,0);
    setUpVector(0,0,1);
    enabled = true;
    setModePerspective();
	left = -300;
    right = 300;
    bottom = -400;
    top = 400;
    nearPlane = 10;
    farPlane = 10000;
    //viewAngle = 130; //bird view
    viewAngle = 70; //normal view
    viewRatio = 1;
    theta = phi = 0;
  }
  
  void disable(){enabled = FALSE;}
  void enable() {enabled = TRUE ;}
  
  //GETTER AND SETTER
  void setModeOrtho(){mode = MODE_ORTHO;}
  void setModePerspective(){mode = MODE_PRESPECTIVE;}
  
  GLfloat getLookAtX(){return lookAt.getX();}
  GLfloat getLookAtY(){return lookAt.getY();}
  GLfloat getLookAtZ(){return lookAt.getZ();}
  GLfloat getEyeX(){return eye.getX();}
  GLfloat getEyeY(){return eye.getY();}
  GLfloat getEyeZ(){return eye.getZ();}
  my3dPoint getEye(){return eye;}
  my3dPoint getLookAt(){return lookAt;}
  
  //Basic movement
  void moveTo(GLfloat x, GLfloat y, GLfloat z);
  void moveBy(GLfloat x, GLfloat y, GLfloat z);
  
  void setLookAt(GLfloat x, GLfloat y, GLfloat z);
  void setLookAt(myVector v);
  void moveLookAtBy(GLfloat x, GLfloat y, GLfloat z);
  
  void setUpVector(myVector v);
  void setUpVector(GLfloat x, GLfloat y, GLfloat z);
  myVector getUpVector(){return up;}
  myVector getViewVector();
  

  //less basic movement
  void panTo(GLfloat x, GLfloat y, GLfloat z);
  void panBy(GLfloat x, GLfloat y, GLfloat z);
  void zoomBy (GLfloat factor);
  void rollBy(GLfloat tAngle);
  void rollTo(GLfloat tAngle);
  void rotateAroundY(GLfloat angle);
  void rotateAroundX(GLfloat angle);
  void rotateAroundXY(GLfloat angleX, GLfloat angle);
  void rotateAroundY(GLfloat angle, GLfloat x, GLfloat y, GLfloat z);
  void rotateAroundX(GLfloat angle, GLfloat x, GLfloat y, GLfloat z);
  void rotateAroundXY(GLfloat angleX, GLfloat angleY, GLfloat x, GLfloat y, GLfloat z);
 
  void rotateLookAtBy(float theta, float phi);

  void draw();
  double getViewAngle(){return viewAngle;}
  void setViewAngle(double t){viewAngle = t;}
  void init();
  
};

void myCam::init(){
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  //gluPerspective(28, 1, 0.1, 1000);
  up.setXYZ(0,0,1);
  lookAt.setXYZ(0,0,0);
  eye.setXYZ(0,0,0);
  rotationAngleAroundX = 0;
  rotationAngleAroundY = 0;
  rollAngle = 0;

}


void myCam::rotateLookAtBy(float tphi, float ttheta){
  theta += ttheta/5;
  phi -= tphi/5;

  
  //while (theta > PI) theta -= PI;
  if(theta > PI) theta = PI - 0.01;
  while (phi > 2*PI) phi -= 2*PI;
  
  //while (theta < 0) theta += PI;
  if(theta < 0) theta = 0.01;
  while (phi < 0) phi += 2*PI;

  float R = getViewVector().getLength();

  // Bring the lookAtPoint into world coords
  setLookAt(
	    getLookAtX() - getEyeX(),
	    getLookAtY() - getEyeY(),
	    getLookAtZ() - getEyeZ()
	    );
 
  // Rotate it on a sphere
  setLookAt(
	    sin(theta)*cos(phi)*R,
	    sin(theta)*sin(phi)*R,
	    cos(theta)*R
	    );
  
  // Bring it back to Bird's Eye Coords.
  setLookAt(
	    getLookAtX() + getEyeX(),
	    getLookAtY() + getEyeY(),
	    getLookAtZ() + getEyeZ()
	    );
  
}


void myCam::rollBy(GLfloat tAngle){
  //tAngle is in Radians
  rollAngle += tAngle;
  while (rollAngle >= 2*PI){rollAngle -= 2*PI;}
  while (rollAngle <= 0){rollAngle += 2*PI;}
  up.setX(sin(rollAngle));
  up.setY(cos(rollAngle));
}

void myCam::rollTo(GLfloat tAngle){
  //tAngle is in Radians
  rollAngle = tAngle;
  while (rollAngle >= 2*PI){rollAngle -= 2*PI;}
  while (rollAngle <= 0){rollAngle += 2*PI;}
  up.setX(sin(rollAngle));
  up.setY(cos(rollAngle));
}

void myCam::panBy(GLfloat x, GLfloat y, GLfloat z){
  moveBy(x, y, z);
  moveLookAtBy(x,y,z);
}

void myCam::panTo(GLfloat x, GLfloat y, GLfloat z){
  GLfloat dX = x - eye.getX();
  GLfloat dY = y - eye.getY();
  GLfloat dZ = z - eye.getZ();
  moveTo(x, y, z);
  moveLookAtBy(dX,dY,dZ);
}

void myCam::zoomBy(GLfloat distance){
  /** Moves the eye in the lookAt direction **/
  if (distance != 0){
    myVector lookDirection;
    lookDirection.setXYZ(eye.getX() - lookAt.getX(), eye.getY() - lookAt.getY(), eye.getZ() - lookAt.getZ());
    lookDirection.normalize();
    eye.setXYZ( eye.getX() + lookDirection.getX() * distance,
		eye.getY() + lookDirection.getY() * distance,
		eye.getZ() + lookDirection.getZ() * distance
		);
  }
}

void myCam::rotateAroundY(GLfloat angle, GLfloat x, GLfloat y, GLfloat z){
  GLfloat radius;
  radius = (eye.getX() - x)*(eye.getX() - x) + (eye.getZ() - z)*(eye.getZ() - z);
  radius = sqrt(radius);
  rotationAngleAroundY += angle;
  moveTo (cos(rotationAngleAroundY/70)*radius, eye.getY() , sin(rotationAngleAroundY/70)*radius);
}
void myCam::rotateAroundY(GLfloat angle){
  rotateAroundY(angle, lookAt.getX(), lookAt.getY(), lookAt.getZ());
}

void myCam::rotateAroundX(GLfloat angle, GLfloat x, GLfloat y, GLfloat z){
  GLfloat radius;
  radius = (eye.getY() - y)*(eye.getY() - y) + (eye.getZ() - z)*(eye.getZ() - z);
  radius = sqrt(radius);
  rotationAngleAroundX += angle;
  moveTo (eye.getX(), sin(rotationAngleAroundX/70)*radius  , -cos(rotationAngleAroundX/70)*radius);
  
}
void myCam::rotateAroundX(GLfloat angle){
  rotateAroundX(angle, lookAt.getX(), lookAt.getY(), lookAt.getZ());
}

void myCam::rotateAroundXY(GLfloat angleX, GLfloat angleY, GLfloat x, GLfloat y, GLfloat z){
  GLfloat radius;
  radius = (eye.getX() - x)*(eye.getX() - x) + (eye.getY() - y)*(eye.getY() - y) + (eye.getZ() - z)*(eye.getZ() - z);
  radius = sqrt(radius);
  rotationAngleAroundX += angleX;
  rotationAngleAroundY += angleY;
  
  GLfloat theta = rotationAngleAroundX / 100;
  GLfloat phi = rotationAngleAroundY   / 100;
  
  moveTo (cos(theta) * cos(phi) * radius, cos(theta) * sin(phi) * radius  , sin(theta) * sin(phi)*radius);
  
}

void myCam::rotateAroundXY(GLfloat angleX, GLfloat angleY){
  rotateAroundXY(angleX, angleY, lookAt.getX(), lookAt.getY(), lookAt.getZ());
}

void myCam::moveTo(GLfloat x, GLfloat y, GLfloat z){
  eye.setXYZ( x,  y , z) ;
}

void myCam::moveBy(GLfloat x, GLfloat y, GLfloat z){
  moveTo( x + eye.getX()  ,  y + eye.getY() , z + eye.getZ());
}

void myCam::setLookAt(GLfloat x, GLfloat y, GLfloat z){
  lookAt.setXYZ(x,y,z);
}

void myCam::setLookAt(myVector v){
  setLookAt(v.getX(), v.getY(), v.getZ());
}

void myCam::moveLookAtBy(GLfloat x, GLfloat y, GLfloat z){
  setLookAt(x + lookAt.getX(),  y + lookAt.getY(), z + lookAt.getZ());
}

void myCam::setUpVector(GLfloat x, GLfloat y, GLfloat z){
  up.setXYZ(x,y,z);
}

void myCam::setUpVector(myVector v){
  setUpVector(v.getX(), v.getY(), v.getZ());
}


void myCam::draw(){  
  if (enabled == TRUE){
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

    if (mode == MODE_ORTHO)
      glOrtho(left,right,bottom,top,nearPlane,farPlane);
    else
      gluPerspective(viewAngle, viewRatio, nearPlane, farPlane);

    gluLookAt(  eye.getX(), eye.getY(), eye.getZ(),
		lookAt.getX() , lookAt.getY(), lookAt.getZ() ,
		up.getX(), up.getY(), up.getZ());
    glMatrixMode(GL_MODELVIEW); 
  }
}

myVector myCam::getViewVector(){
  myVector returnVal;
  returnVal.setXYZ(
		   lookAt.getX() - eye.getX(), 
		   lookAt.getY() - eye.getY(), 
		   lookAt.getZ() - eye.getZ()
		   );
  return returnVal;
}

#endif
