/* Poopmup - An urban flight sim
 *
 * Copyright (C) 2002 Zaroubi, Fink, Leclerc, Stanek szaroubi@hotmail.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef MYLINE_H
#define MYLINE_H

#include "my3dPoint.h"
#include "myVector.h"
#include "myPlane.h"


/**
 * A line has a starting point and an end point.
 * The line doesn't have to stop at these 2 points.
 * They are there to define the line in 3d space.
 */
class myLine{

 private:
  my3dPoint start;
  my3dPoint end;

 public:
  myLine(){setStart(0,0,0); setEnd(1,0,0);}
  ~myLine(){}
  
  my3dPoint getStart(){ return start;}
  my3dPoint getEnd(){ return end;}
  
  void setLine(my3dPoint p, myVector v){
    start = p;
    end = p + v;
  }
  
  void setLine(my3dPoint p1, my3dPoint p2)
    {
      setStart(p1);
      setEnd(p2);
    }
  
  void setStart(float x, float y, float z){start.setXYZ(x,y,z);}
  void setStart(my3dPoint p){setStart(p.getX(), p.getY(), p.getZ());}
  void setEnd(float x, float y, float z){end.setXYZ(x,y,z);}
  void setEnd(my3dPoint p){setEnd(p.getX(), p.getY(), p.getZ());}
  
  void setDirection(myVector v){
    end = start+ v;
  }
  
  void setDirection(float x, float y, float z){
    myVector v;
    v.setXYZ(x,y,z);
    v = v.normalize();
    setDirection(v);
  }
  
  myVector getDirection(){
    myVector v;
    v.setX(end.getX() - start.getX());
    v.setY(end.getY() - start.getY());
    v.setZ(end.getZ() - start.getZ());
    return v.normalize();
  }
  
  void print(){
    cout<<"Start "; start.print();
    cout<<"End "; end.print();
  }
  
  my3dPoint projectOnto(myPlane pl);
  my3dPoint getProjectionOf(my3dPoint p);
};


my3dPoint myLine::projectOnto(myPlane pl){
  //Then you projet the line on a plane you get the intersection Point.
  //TODO: Check if line and plane are parallel
  my3dPoint p;
  float u;
  myVector v;

  v = getDirection();
  u =  start.getX() * pl.getA() + 
    start.getY() * pl.getB() +  
    start.getZ() * pl.getC() + pl.getD();

  u /= -(pl.getA()*v.getX() + pl.getB()*v.getY() + pl.getC()*v.getZ() );
  
  p = start + v * u;

  return p;
}

my3dPoint myLine::getProjectionOf(my3dPoint p){
  //RETURNS THE ORTHO PROJECTION OF P ONTO THIS LINE
  myPlane pl;
  pl.setPerpendicularTo(p, getDirection());
  return projectOnto(pl);
}

#endif
