/* Poopmup - An urban flight sim
 *
 * Copyright (C) 2002 Zaroubi, Fink, Leclerc, Stanek szaroubi@hotmail.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**************************************************************************
                          myMatrix.h  -  description
                             -------------------
    begin                : Wed Oct 16 2002
    copyright            : (C) 2002 by Sherif Zaroubi
    email                : fogof@localhost.localdomain
 *************************************************************************/


/*****************
!!!!!!!!!!!!!!!!!!
MATRISES ARE STORED IN COLUMN MAJOR FORM
!!!!!!!!!!!!!!!!!!
*****************/

/** How to use ?
 * myMatrix m1,m2,m3;
 * m1.setRotationAroundX(30);
 * m2.setTranslation(10,10,10);
 * m3 = m2 * m1;
 * m3.print();
 */

/** myMatrix m1, invM1, mResult;
 * m1.set<whatEverMatrix>(PARAMS);
 * invM1.set<whatEverMatrix>Inv(SAME_PARAMS);
 * mResult = m1 * invM1;
 * RESULT WILL ALWAYS BE the identity matrix
 */


#ifndef MYMATRIX_H
#define MYMATRIX_H

#define DEBUG 0
#include "../../common.h"
// #include <GL/glut.h>
#include "my3dPoint.h"
#include "myVector.h"

#include <iostream.h>
#include <math.h>

/**
 *@author Sherif Zaroubi
 */

class myMatrix {

  private:
    GLfloat matrix[16];


  public:
    myMatrix();

  /***** Basic (Get/Set)er Functions ******/
    void setMatrix(GLfloat * tMatrix);
    void setElement(int row, int col, GLfloat value);
    GLfloat getElement(int row, int col) {
	return matrix[col * 4 + row];
    }
    GLfloat *getMatrix() {
	return matrix;
    }


  /**** Operator Overloading *******/
    myMatrix operator *(myMatrix & RHS);
    my3dPoint operator *(my3dPoint & p);
    myVector operator *(myVector & v);

  /**** Special Matrices ***/
    void setIdentity();
    void setTranslation(GLfloat x, GLfloat y, GLfloat z);
    void setScale(GLfloat x, GLfloat y, GLfloat z);
    void setRotationAroundX(GLfloat angle);
    void setRotationAroundY(GLfloat angle);
    void setRotationAroundZ(GLfloat angle);

    void setTranslationInv(GLfloat x, GLfloat y, GLfloat z);
    void setScaleInv(GLfloat x, GLfloat y, GLfloat z);
    void setRotationAroundXInv(GLfloat angle);
    void setRotationAroundYInv(GLfloat angle);
    void setRotationAroundZInv(GLfloat angle);
    
    void print();

    ~myMatrix();
};


void
 myMatrix::setMatrix(GLfloat * tMatrix)
{
    for (int i = 0; i < 16; i++)
	matrix[i] = tMatrix[i];

}

void myMatrix::setElement(int row, int col, GLfloat value)
{
    matrix[col * 4 + row] = value;
}


myMatrix myMatrix::operator * (myMatrix & RHS) {
    myMatrix result;
    GLfloat value = 0;

    for (int row = 0; row < 4; row++) {
	for (int col = 0; col < 4; col++) {
	    for (int i = 0; i < 4; i++) {
		value += getElement(row, i) * RHS.getElement(i, col);
	    }
	    if ((value < 0.00001) && (value > -0.00001))
		value = 0;
	    result.setElement(row, col, value);
	    value = 0;
	}
    }
    return result;
}

my3dPoint myMatrix::operator * (my3dPoint & p) {
    my3dPoint result;
    GLfloat value = 0;

    for (int row = 0; row < 4; row++) {
	if (DEBUG)
	    cout << "Row : " << row << endl;
	for (int i = 0; i < 4; i++) {
	    value += getElement(row, i) * p.getElement(i);
	    // if(DEBUG) cout<<"\tVal : "<<getElement(row,i) <<", "<<
	    // RHS.getElement(i)<< " = "<<value<<endl;
	}

	if ((value < 0.00001) && (value > -0.00001))
	    value = 0;
	result.setElement(row, value);
	value = 0;
    }
    return result;
}

myVector myMatrix::operator * (myVector & v) {
  myVector result;
  GLfloat value = 0;
  
  for (int row = 0; row < 4; row++) {
    if (DEBUG)
      cout << "Row : " << row << endl;
  for (int i = 0; i < 4; i++) {
    value += getElement(row, i) * v.getElement(i);
  }
  
  if ((value < 0.00001) && (value > -0.00001))
    value = 0;
  result.setElement(row, value);
  value = 0;
  }
  return result;
}


void myMatrix::print()
{
    for (int row = 0; row < 4; row++) {
	cout << endl;
	for (int col = 0; col < 4; col++) {
	    cout << getElement(row, col) << " | ";
	}
    }
    cout << endl;
}

void myMatrix::setIdentity()
{
    GLfloat temp[16] = {
	1, 0, 0, 0, 
	0, 1, 0, 0, 
	0, 0, 1, 0, 
	0, 0, 0, 1
    };

    setMatrix(temp);

}

void myMatrix::setTranslation(GLfloat x, GLfloat y, GLfloat z)
{
    GLfloat temp[16] = {
	1, 0, 0, 0, 
	0, 1, 0, 0, 
	0, 0, 1, 0, 
	x, y, z, 1
    };

    setMatrix(temp);

}

void myMatrix::setScale(GLfloat x, GLfloat y, GLfloat z)
{
    GLfloat temp[16] = {
	x, 0, 0, 0,
	0, y, 0, 0,
	0, 0, z, 0, 
	0, 0, 0, 1
    };

    setMatrix(temp);

}

void myMatrix::setRotationAroundX(GLfloat angle)
{
  angle = angle * (PI/180); //Convert To Radian
  GLfloat temp[16] = {
    1, 0, 0, 0,
    0, cos(angle), sin(angle), 0, 
    0, -sin(angle), cos(angle), 0, 
    0, 0, 0, 1
  };
  
  setMatrix(temp);
}

void myMatrix::setRotationAroundY(GLfloat angle)
{
  angle = angle * (PI/180); //Convert To Radian
  GLfloat temp[16] = {
    cos(angle), 0, -sin(angle), 0, 
    0, 1, 0, 0, 
    sin(angle), 0, cos(angle), 0, 
    0, 0, 0, 1
  };

    setMatrix(temp);
}

void myMatrix::setRotationAroundZ(GLfloat angle)
{
  angle = angle * (PI/180); //Convert To Radian
  GLfloat temp[16] = {
    cos(angle), sin(angle),	0, 0, 
    -sin(angle), cos(angle), 0, 0, 
    0, 0, 1, 0, 
    0, 0, 0, 1
  };
  
  setMatrix(temp);
}


void myMatrix::setTranslationInv(GLfloat x, GLfloat y, GLfloat z)
{
  setTranslation(-x,-y,-z);
}

void myMatrix::setScaleInv(GLfloat x, GLfloat y, GLfloat z)
{
  setScale(1/x,1/y,1/z);
}

void myMatrix::setRotationAroundXInv(GLfloat angle)
{
  setRotationAroundX(-angle);
}

void myMatrix::setRotationAroundYInv(GLfloat angle)
{
  setRotationAroundY(-angle);
}

void myMatrix::setRotationAroundZInv(GLfloat angle)
{
  setRotationAroundZ(-angle);
}


myMatrix::myMatrix()
{
}

myMatrix::~myMatrix()
{
}

#endif
