/* Poopmup - An urban flight sim
 *
 * Copyright (C) 2002 Zaroubi, Fink, Leclerc, Stanek szaroubi@hotmail.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/***************************************************************************
                          myObject2.h  -  description
                             -------------------
    begin                : Wed Oct 16 2002
    copyright            : (C) 2002 by Sherif Zaroubi
    email                : fogof@localhost.localdomain
 ***************************************************************************/
/**
 * BASE CLASS FOR ALL VISUAL OBJECS IN THE WORLD
 * They enven have bouning boxes.
 * They should ONLY be transphormed with the transphormations define in THIS clas.
 * There should NOT be any gl<transphormation> transphormation the objects outside of this class.
 * If you do so, the bounding boxes will not apear where you expect them.
 */



/** IMPORTANT **
 * all that "To" transphormations (rotateTo, translateTo)....
 * will reset the state => ALL SCALING/Rotation will be lost.
 */


#ifndef MYOBJECT2_H
#define MYOBJECT2_H
#include "../../common.h"
    // #include <GL/glut.h>
#include <string>
#include "myMatrix.h"
#include "my3dPoint.h"

#include "myBoundingBox.h"

/**
  *@author Sherif Zaroubi
  */

using namespace std;

class myObject2 {
  private:
    /*
     * State will hold the transphormation And invState will hold the
     * transphormation To bring the object Back to the origin 
     */
  //At this point invState is not used anywhere.
  myMatrix state, invState;
  //my3dPoint center;

  //Draw mode is "Solid" or "Wire" 
  //TODO: This should be macrofied
  string drawMode;

  //Color informations.
  //TODO: should be myColor class
  GLfloat r, g, b;

  //Heheheheheh 30 hours of work.
  myBoundingBox boundingBox;

  //used when a translation is done after a scale.
  //TODO: Check the w element of my vector.
  myVector scaleFactor;
  
 public:
  //Set to 1 if you want the object draw it's bounding box.
  int debugBB;
  
  myMatrix * getState() {
    return &state;
  }
  myMatrix * getInvState(){
      return &invState;
    }
  
  void setDrawMode(string t) {
    drawMode = t;
  };
  string getDrawMode() {
    return drawMode;
  };
  
  
  //Returns the Center of the Bounding box.
  my3dPoint getCenter(){return getBoundingBox().getCenter();}
  
  void setState(myMatrix *tState, myMatrix *tInvState){
    state = *tState; 
    invState = *tInvState;
  }
  
  myBoundingBox getBoundingBox(){return boundingBox;}
  
  void resetState() {
    state.setIdentity();
    scaleFactor.setXYZ(1,1,1);
  }
  
  /********** All transphormations ***********/
  void rotateAroundXBy(GLfloat angle);
  void rotateAroundYBy(GLfloat angle);
  void rotateAroundZBy(GLfloat angle);
  void translateBy(GLfloat x, GLfloat y, GLfloat z);
  void translateBy(myVector v)
    {translateBy(v.getX(), v.getY(),v.getZ());}
  void translateTo(GLfloat x, GLfloat y, GLfloat z)
    {resetState();translateBy(x,y,z);}
  void translateTo(myVector v){resetState();translateBy(v);}
  void scaleBy(GLfloat scaleX, GLfloat scaleY, GLfloat scaleZ);
  void syncTransphormations(); //Sync to all atached objects

  void setColor(GLfloat tr, GLfloat tg, GLfloat tb) 
    { r = tr; g = tg; b = tb; }

  // Will call drawGeometry.
  void draw();

  //Should be define in subObjs
  virtual void drawGeometry() = 0;
  
  // does this obj collide with o2?
  int collide(myObject2 &o2){
    //return getBoundingBox().collide(o2.getBoundingBox());
    return getBoundingBox().SphericalCollide(o2.getBoundingBox());
  }

  // Is point P inside the "Spherical BB" Of this ? 
  int collide(my3dPoint p){
    return boundingBox.SphericalContains(p);
  }

  myObject2();
  virtual ~ myObject2();
};


/************************ BY FUNCTIONS **************************/
void myObject2::translateBy(GLfloat x, GLfloat y, GLfloat z)
{
    myMatrix translateBy, invMat;
    x /= scaleFactor.getX();
    y /= scaleFactor.getY();
    z /= scaleFactor.getZ();

    translateBy.setTranslation(x, y, z);
    invMat.setTranslationInv(x, y, z);

    state = state * translateBy;
    invState = invMat * invState;
    syncTransphormations();
    //boundingBox.translateBy(x,y,z);
}

void myObject2::scaleBy(GLfloat scaleX, GLfloat scaleY, GLfloat scaleZ)
{
    myMatrix scaleBy, invMat;
    scaleFactor.setXYZ(scaleX, scaleY, scaleZ);

    invMat.setScaleInv(scaleX, scaleY, scaleZ);
    scaleBy.setScale(scaleX, scaleY, scaleZ);

    state = state * scaleBy;
    invState = invMat * invState;
    //boundingBox.scaleBy(scaleX, scaleY, scaleZ);
    syncTransphormations();
}

void myObject2::rotateAroundXBy(GLfloat angle)
{
    myMatrix rotateBy, invMat;

    rotateBy.setRotationAroundX(angle);
    invMat.setRotationAroundXInv(angle);

    state = state * rotateBy;
    invState = invMat * invState;
    //boundingBox.rotateAroundXBy(angle);
    syncTransphormations();

}

void myObject2::rotateAroundYBy(GLfloat angle)
{
    myMatrix rotateBy, invMat;

    rotateBy.setRotationAroundY(angle);
    invMat.setRotationAroundYInv(angle);

    state = state * rotateBy;
    invState = invMat * invState;
    //boundingBox.rotateAroundYBy(angle);
    syncTransphormations();
}

void myObject2::rotateAroundZBy(GLfloat angle)
{
    myMatrix rotateBy, invMat;

    rotateBy.setRotationAroundZ(angle);
    invMat.setRotationAroundZInv(angle);

    state = state * rotateBy;
    invState = invMat * invState;
    //boundingBox.rotateAroundZBy(angle);
    syncTransphormations();
}



void myObject2::draw()
{
  // Set to model View
  glMatrixMode(GL_MODELVIEW);
  glPushMatrix();
  { //All transphormations from this point will only apply to this obj
    //Set the state
    glLoadMatrixf(state.getMatrix());
    //Set the color of this object
    glColor3f(r, g, b);
    //Draw the geometry.
    //This func is defined in derived objects
    drawGeometry();
    //Set the color back to grey
    glColor3f(0.5, 0.5, 0.5);
  }//All transphormations from this point are no longer apply to obj
  glPopMatrix();

  //For debuging the Boundingbox.
  //The BB is in World coordinats
  //And that is why it doesn't apear in the "Push() Pop()" block
  if (debugBB)
    boundingBox.draw();
  
}

myObject2::myObject2()
{
  debugBB = 0;
  r = g = b = 0.5;
  resetState();
}

myObject2::~myObject2()
{
}

void myObject2::syncTransphormations(){
  // When ever this object is transphormed ....
  // THe bounding box should be transphormed TO the state of the object.
  // and NOT "BY the state"
  boundingBox.init();
  boundingBox.applyTransphormation(state);
}

#endif







