/* Poopmup - An urban flight sim
 *
 * Copyright (C) 2002 Zaroubi, Fink, Leclerc, Stanek szaroubi@hotmail.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef MYPLANE_H
#define MYPLANE_H

#include "my3dPoint.h"
#include "myVector.h"
#include <math.h>


/**
 * Defines a 3d plane in a 4d space.
 */

class myPlane{

 private:
  float coords[4];

 public:
  float getA(){return coords[0];}
  float getB(){return coords[1];}
  float getC(){return coords[2];}
  float getD(){return coords[3];}

  void setA(float t){coords[0] = t;}
  void setB(float t){coords[1] = t;}
  void setC(float t){coords[2] = t;}
  void setD(float t){coords[3] = t;}

  myPlane(){setA(1);setB(1);setC(1);setD(1);}
  ~myPlane(){};

  void print();
  void setPerpendicularTo(my3dPoint p, myVector v);
  int pointIsOnPlane(my3dPoint p){
    int returnVal = 0;
    float d;
    d  = getA() * p.getX();
    d += getB() * p.getY();
    d += getC() * p.getZ();
    d += getD();

    if ( (d < 0.001) && (d > -0.001))
      returnVal = 1;

    return returnVal;
  }

};


void myPlane::setPerpendicularTo(my3dPoint p, myVector v){
  /** myVector v describs the euqation of a line in 3d space.
   *
   * Algo:
   * -----
   * we know that if vector has function:
   *                ax + by + c = 0;
   * the perpendicular plane has equation;
   *       ax + by + cz + d = 0
   * but what is d ?
   * d will be computed from the point p
   */
  v = v.normalize();
  setA(v.getX());
  setB(v.getY());
  setC(v.getZ());
  float tD = - (getA() * p.getX());
  tD -= getB() * p.getY();
  tD -= getC() * p.getZ();
  setD(tD);
}

void myPlane::print(){
  cout << "A : "    << getA();
  cout << " | B : " << getB();
  cout << " | C : " << getC();
  cout << " | D : " << getD();
  cout << endl;
}
#endif




