/* Poopmup - An urban flight sim
 *
 * Copyright (C) 2002 Zaroubi, Fink, Leclerc, Stanek szaroubi@hotmail.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/***************************************************************************
                          myVector.h  -  description
                             -------------------
    begin                : Tue Oct 15 2002
    copyright            : (C) 2002 by Sherif Zaroubi
    email                : fogof@localhost.localdomain
 ***************************************************************************/

#ifndef MYVECTOR_H
#define MYVECTOR_H

/**
 *@author Sherif Zaroubi
 */

#include "my3dPoint.h"
#include <math.h>
#include <stdlib.h>


/** This is a Fink workaround.
 * How to make a vector from 2 points?
 * use the bellow Macro
 */
#define MAKE_VECTOR(v,p1,p2) v.setXYZ(p2.getX() - p1.getX(),p2.getY() - p1.getY(),p2.getZ() - p1.getZ())


//How close to 0 is a point to be considered 0
#define PRESISION 0.00001

class myVector {

  private:
    float coords[4];

  public:
     myVector();
     myVector(float tx, float ty, float tz);
    ~myVector();

    float getX() {
	return coords[0];
    }
    float getY() {
	return coords[1];
    }
    float getZ() {
	return coords[2];
    }
    float getW() {
	return coords[3];
    }

    myVector crossProduct(myVector  v2);
    float dotProduct(myVector v2);
    
    int sameDirection(myVector v2);

    myVector operator + (myVector & v2);
    myVector operator *(float i);

    //Cross Product of 2 vectors
    myVector operator *(myVector v2);
    void setXYZ(float tx, float ty, float tz) {
	setX(tx);
	setY(ty);
	setZ(tz);
    }

    void setXYZW(float tx, float ty, float tz, float tw) {
	setXYZ(tx, ty, tz);
	setW(tw);
    }

    void setX(float t) {
      if ((t < PRESISION) && (t>-PRESISION))
	t = 0;
	coords[0] = t;
    }
    void setY(float t) {
      if ((t < PRESISION) && (t>-PRESISION))
	t = 0;
	coords[1] = t;
    }
    void setZ(float t) {
      if ((t < PRESISION) && (t>-PRESISION))
	t = 0;
	coords[2] = t;
    }
    void setW(float t) {
      if ((t < PRESISION) && (t>-PRESISION))
	t = 0;
	coords[3] = t;
    }

    float getElement(int i) {
	if ((i < 4) && (i >= 0))
	    return coords[i];
	else
	    return -99999;	// 5 - 9's
    }

    void setElement(int i, float value) {
      if ((value < PRESISION) && (value>-PRESISION))
	value = 0;
	if ((i < 4) && (i >= 0))
	    coords[i] = value;
    }
    
	myVector getPerpendicular(){
      myVector t;
      t.setXYZW(getX(), getY(), getZ(), getW());
      return crossProduct(t);
    }

    float getLength();
    myVector normalize();
    void operator= (myVector v2){
      setXYZW(v2.getX(),
	      v2.getY(),
	      v2.getZ(),
	      v2.getW()
	      );
    }
    void print();
};

myVector myVector::operator + (myVector & v2) {
    myVector result;

    result.setXYZ(getX() + v2.getX(),
		  getY() + v2.getY(),
		  getZ() + v2.getZ()
		  );

    return result;
}



myVector myVector::operator * (float i) {
    myVector result;

    result.setXYZ(getX() * i, getY() * i, getZ() * i);
    return result;
}

myVector myVector::crossProduct(myVector  v2)
{
    myVector result;

    result.setX(getY() * v2.getZ() - getZ() * v2.getY());
    result.setY(getZ() * v2.getX() - getX() * v2.getZ());
    result.setZ(getX() * v2.getY() - getY() * v2.getX());
    return result;
}

myVector myVector::operator *(myVector  v2){
  return crossProduct(v2);
}

float myVector::getLength(){
  return sqrt(getX() * getX() + getY() * getY() + getZ() * getZ());
}

float myVector::dotProduct(myVector  v2){
  return (  getX() * v2.getX() 
	    + getY() * v2.getY() 
	    + getZ() * v2.getZ());
}

myVector myVector::normalize(){
  
  myVector returnVal;
  float length = getLength();
  returnVal.setXYZ(getX() / length, getY() / length, getZ() / length);
  return returnVal;
}

myVector::myVector(float tx, float ty, float tz){
  setXYZ(tx, ty, tz);
}

myVector::myVector(){
  setXYZW(0, 0, 0, 1);
}

myVector::~myVector(){
}

int myVector::sameDirection(myVector v2){
  //Returns 1 if in same Direction
  //And 0 if in oposite direction
  
  float dp = dotProduct(v2);
  int retVal = 0;
  if (dp > 0){
    retVal = 1; 
  }
  return retVal;
}

void myVector::print(){
  cout << "X:" << getX() << " |Y: " << getY() << "|Z: " << getZ() <<
    "| W:" << getW() << " |length: "<<getLength() << endl;
  
}

#endif
