/* Poopmup - An urban flight sim
 *
 * Copyright (C) 2002 Zaroubi, Fink, Leclerc, Stanek szaroubi@hotmail.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef MYBIRD_H
#define MYBIRD_H

#include "../../common.h"
#include "birdGeometry.h"
#include "../base/myVector.h"
#include "myPoop.h"
#include "../base/myLight.h"
#include "../derivedGeom/myCrossAir.h"

#define MAX_POOP 30
#define STARTING_NUM_POOP 15
#define BIRD_STARTING_HEALTH 100
#define BIRD_MAX_HEALTH 100
#define BIRD_MAX_SPEED 100
#define BIRD_MAX_HEIGHT 2200
#define BIRD_MIN_HEIGHT 35
#define BIRD_WORLD_PADDING 175

//Duration is in seconds
//TODO: Get value of duration for config file
#define BIRD_ACID_TRIP_DURATION  30

class myBird: public myCam{
  
 private:
  //Poop management variables
  int storedPoop, activePoop;  POOP_LIST m_poopList;

  //The birdGeometry
  birdGeometry birdGeom;

  //Movement vector
  myVector displacement;

  //the headlight TODO:Tweak the headlight
  myLight headLight;
  
  //The bird's health
  float health;
  
  //The time at which the trip ends 
  //(in seconds relative to GET_CURRENT_SECOND )
  float acidTripEnd;

  //Bird's points
  int points;

  //old field of view used in acid trip
  double oldFov; 
  
  //Is the bird on Acid?
  int isOnAcid;
  
  //Bird's speed
  float speed;

  //TODO: Check if variable is still needed
  myVector initUpVector;

  //the world's size
  //Used to check if bird is still in boundry
  float worldWidth, worldDepth;

  //The frame number in birds animation
  //Used to slow down bird in Move function
  int frameNum; 

  //List of poop
  POOP_LIST::iterator m_poop_it;

  //The bird's crossHair
  myCrossAir crossAir;

  //The position of bird's crossHair
  my3dPoint crossAirLocation;
  
  //At what angle is the bird on
  myVector turnVector;

 public:
  myBird();
  ~myBird(){};

  //Is the poopOCam enabled?
  int poopOCamEnabled;
  

  /** Movement **/
  myVector getDisplacement(){return displacement;}
  void setDisplacement(myVector v){ displacement = v;}
  void Move();
  void turnBy(float hVect, float vVect);

  /** Points **/
  int getPoints() {return points;}
  void setPoints(int t){ points = t;}
  void incPoints(int t){ setPoints( points + t);}
  
  /** SPEED **/
  void setSpeed(float t){if ((t < BIRD_MAX_SPEED) && (t>0)) speed = t;}
  void incSpeed(float t){setSpeed(speed + t);}
  int getMaxSpeed(){return BIRD_MAX_SPEED;}
  float getSpeed(){return speed;}

  
  /**** GEOMETRY ******/
  void animate();
  void drawGeometry();
  void reInitUpVector();
  void drawCrossair();
  void drawPoopoCam();
  
  /** HEALTH **/
  float getMaxHealth(){return BIRD_MAX_HEALTH;}
  void decHealthBy(float t);//{health -= t;}
  void incHealthBy(float t);//{health += t;}
  float getHealth(){return health;}
  int isAlive(){ return (health > 0);}
  void setHealth(float t){health = t;}

  /** POOP **/
  void incNumPoop(int t);
  void dropPoop();
  myPoop *getCurrentPoop();
  myPoop *getFirstPoop();
  myPoop *getNextPoop();
  void DeletePoop(myPoop *pPoop);
  int getNumPoop() {return storedPoop;}
  int getMaxPoop() {return MAX_POOP;}

  /** Acid Trip **/  
  void onAcid();
  void offAcid();
 
  /* MISC */
  void reset();
  void setWorldSize(float tw, float td){ worldWidth = tw; worldDepth = td;}
  //Get the poop's Gravity
  myVector getPoopGravity(){myPoop m; return m.getGravity();}

};

void myBird::drawCrossair(){
  crossAir.translateTo(eye.getX(), eye.getY() , 10);
  crossAir.draw();
}

myPoop *myBird::getCurrentPoop(){
  if(m_poop_it == m_poopList.end())
    return NULL;
  else
    return *m_poop_it;    
}
myPoop *myBird::getFirstPoop(){
  m_poop_it = m_poopList.begin();
  return getCurrentPoop();
}

myPoop *myBird::getNextPoop(){
  ++m_poop_it;
  return getCurrentPoop();
}

void myBird::decHealthBy(float t){
  if (t<0) t = -t;
  if (health -t < 0) health = 0;
  else health -= t;
}

void myBird::incHealthBy(float t){
  if (t<0) t = -t;
  if (health +t > BIRD_MAX_HEALTH) health =  BIRD_MAX_HEALTH;
  else health += t;
}

void myBird::incNumPoop(int t){ 
  if(storedPoop+t<MAX_POOP)
    storedPoop += t;
  else
    storedPoop = MAX_POOP;
}

myBird::myBird(){
  isOnAcid = 0;
  //onAcid(); //DEBUG
  initUpVector = getUpVector();
  headLight.init(GL_LIGHT0);
  headLight.enable();
  storedPoop = STARTING_NUM_POOP;
  activePoop = 0;
  health = BIRD_STARTING_HEALTH;
  points = 0;
  frameNum = 0;
  oldFov = getViewAngle();
  turnVector.setXYZ(0,0,0);
}

void myBird::reInitUpVector(){
  setUpVector(initUpVector);
}

void myBird::onAcid(){
  //The bird ate a bad fry ..
  //So he is trippin'
  oldFov = getViewAngle();
  setViewAngle(170);
  isOnAcid = 1;
  acidTripEnd = GET_CURRENT_SECOND + BIRD_ACID_TRIP_DURATION;
}

void myBird::offAcid(){
  //The Acid trip wore off, what to do ?
  isOnAcid = 0;
  setViewAngle(oldFov);
}
void myBird::dropPoop(){
  if (storedPoop > 0){
    //Still have poop


    myPoop *pPoop = new myPoop;
    my3dPoint poopDestination = getEye();
    poopDestination.setZ(0);
    pPoop->drop(getEye(),poopDestination);

    m_poopList.push_front(pPoop);

    activePoop++;
    storedPoop--; 

  }
}

/************** ANIMATE *****************/
void myBird::Move(){
  // This is where you Move the bird.
  // And therefore check if the bird is still in the world ...
  // If not in world force back into world.

  frameNum++;
  //If looking down then speed up.
  // If looking up then slow down.
  incSpeed(-getViewVector().getZ()/3000);
  
  // Move the bird's view
  //tilt the view downwards
  rotateLookAtBy(0, PI/8 - (speed/BIRD_MAX_SPEED)*PI/8);
  
  //Cure the bird from the acid trip if duration ran out.
  if (acidTripEnd < GET_CURRENT_SECOND) offAcid();
 
  //If too high/low bring back
  if (getEye().getZ() > BIRD_MAX_HEIGHT)
    panBy(0,0,  BIRD_MAX_HEIGHT - getEyeZ());
  
  if (getEye().getZ() < BIRD_MIN_HEIGHT)
    panBy(0,0,BIRD_MIN_HEIGHT -getEyeZ()); 
  

  if (getEye().getX() < BIRD_WORLD_PADDING)
    panBy(BIRD_WORLD_PADDING - getEyeX(),0,0);

  if (getEye().getY() < BIRD_WORLD_PADDING)
    panBy(0,BIRD_WORLD_PADDING - getEyeY(),0);

  if (getEye().getX() > worldWidth -  BIRD_WORLD_PADDING)
    panBy(worldWidth -  BIRD_WORLD_PADDING - getEyeX(),0,0);

  if (getEye().getY() > worldDepth - BIRD_WORLD_PADDING)
    panBy(0,worldWidth -  BIRD_WORLD_PADDING - getEyeY(),0);



  setDisplacement(getViewVector().normalize() * speed);
  //Move the Camera
  panBy(displacement.getX(), displacement.getY(), displacement.getZ());

  // Move the light
  headLight.moveTo(
		   getEye().getX(), 
		   getEye().getY(), 
		   getEye().getZ()
		   ); 
  headLight.lookAt(
		   getLookAt().getX(), 
		   getLookAt().getY(), 
		   getLookAt().getZ()
		   );


  birdGeom.resetState();
  birdGeom.translateTo(
		       getEye().getX(), 
		       getEye().getY(), 
		       getEye().getZ() - 2
		       ); 


}
void myBird::animate(){
  //Move the bird.
  Move();
  
  // move/animate the poop
  POOP_LIST::iterator it = m_poopList.begin();
  myPoop *pPoop;
  while(it != m_poopList.end())
    {
      pPoop = (*it);
	  it++;
	  if(pPoop->getCenter().getZ() < 0)
	{
	  DeletePoop(pPoop);
	}
      else  
	{    
	  pPoop->animate();
	}
      
  }
  
  //Animate the birds movement
  if (isOnAcid){
    //Animation modification so that the bird looks like he is trippin
  }
  
}

void myBird::drawGeometry(){
  headLight.draw();

  drawCrossair();

  birdGeom.draw();
  POOP_LIST::iterator it = m_poopList.begin();
  while(it != m_poopList.end())
    {
      (*it)->draw();
      it++;
    }
}

void myBird::DeletePoop(myPoop * pPoop)
{
  m_poop_it--;
  
  m_poopList.remove(pPoop);
  delete pPoop;
  activePoop--;
}

void myBird::drawPoopoCam()
{
  // from either the first poop, or the bird's current pos
  // draw the camera looking down.
  if (poopOCamEnabled){
  my3dPoint oldeye = eye;
  my3dPoint oldlookat = lookAt;


  myPoop *pPoop = getFirstPoop();
  if(pPoop)
    {
      my3dPoint c = pPoop->getCenter();
      eye.setXYZ(c.getX(), c.getY(), c.getZ() + 100);
      lookAt.setXYZ(c.getX()-0.001, c.getY(), c.getZ()-1); 
    }
  else
    {
      
      float dx = (lookAt.getX() - eye.getX()) / 10000;
      float dy = (lookAt.getY() - eye.getY()) / 10000;
      
      // look down
      lookAt.setXYZ(eye.getX()+dx, eye.getY() + dy, 0);
    }
  // draw
  draw();
  drawGeometry();

  // restore old state
  eye = oldeye;
  lookAt = oldlookat;

  }


}

void myBird::turnBy(float hVect, float vVect){
  
  turnVector.setXYZ(
		    turnVector.getX() + hVect,
		    turnVector.getY() + vVect,
		    turnVector.getZ()
		    );

  if (turnVector.getX()>2*PI)turnVector.setX(turnVector.getX() - 2*PI);
  if (turnVector.getY()>2*PI)turnVector.setY(turnVector.getY() - 2*PI);
  if (turnVector.getX()<0)turnVector.setX(turnVector.getX() + 2*PI);
  if (turnVector.getY()<0)turnVector.setY(turnVector.getY() + 2*PI);

  myVector tU, tV;
  myMatrix r1,r2, r3;
  myMatrix invR1, invR2;

  tV = getViewVector();
  tU = myVector(0,0,1);
  
  /* Rotation around arbitrary axis */
  //TODO: implement as 1 matrix in myMatrix.h

  //Rotate Around X to place on YZ Plane
  
  float a =  acos(
		  tV.getZ() 
		  /
		  (
		   sqrt(
			tV.getY()*tV.getY() 
			+ 
			tV.getZ()*tV.getZ()
			)
		   )
		  );
  r1.setRotationAroundX(a*180/PI);
  invR1.setRotationAroundXInv(a*180/PI);

  //Rotate around Y onto the XZ plane
  //Therefore will be aligned with Z axis
  float b = acos(
		  tV.getZ() 
		  /
		  (
		   sqrt(
			tV.getX()*tV.getX() 
			+ 
			tV.getZ()*tV.getZ()
			)
		   )
		  );

  r2.setRotationAroundY(b*180/PI);
  invR2.setRotationAroundYInv(b*180/PI);

  //Rotate Around Z
  r3.setRotationAroundZ(hVect*180/PI);
  //cout<<"A="<<a<<" |B="<<b<<endl;
  //tV.print();
  //tU.print();
  tU = (r1 * r2 * r3 * invR2 * invR1) * tU;
  //setUpVector(tU);
  //tU.print();
  
  rotateLookAtBy( hVect, vVect );
}

void myBird::reset(){
  health = BIRD_STARTING_HEALTH;
  points = 0;
  m_poopList.clear();
  storedPoop = STARTING_NUM_POOP;
  activePoop = 0;
  speed = 0;
  offAcid();
  turnVector.setXYZ(0,0,0);
}

#endif





