/* Poopmup - An urban flight sim
 *
 * Copyright (C) 2002 Zaroubi, Fink, Leclerc, Stanek szaroubi@hotmail.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef MYCOULD_H
#define MYCLOUD_H

#include "../../common.h"
#include "../movableObjects/MovableObject.h"
#include "../base/myColor.h"
#include "../base/my3dPoint.h"


class myCloud:public MovableObject{
 private:
  my3dPoint vertex[4];
  float width, depth, height;
  void DrawCurvedSurfaceCloud();
  void DrawSphericalCloud();
 public:
  myCloud();
  ~myCloud(){}
  void Move();
  void drawGeometry();

};


myCloud::myCloud(){
  m_speed = 10 ;
  m_facingDir.setXYZ(0,0,0); //Change this to make clouds move on start.
  width = 1200 + rand() % 2400;
  depth = 1200 + rand() % 2400;
  height = 300 + rand()%200;

  vertex[0].setX(m_location.getX() - width/2);
  vertex[0].setY(m_location.getY() + depth/2);
  vertex[0].setZ(m_location.getZ() + height/2);

  vertex[1].setX(m_location.getX() + width/2);
  vertex[1].setY(m_location.getY() + depth/2);
  vertex[1].setZ(m_location.getZ() + height/2);

  vertex[2].setX(m_location.getX() + width/2);
  vertex[2].setY(m_location.getY() - depth/2);
  vertex[2].setZ(m_location.getZ() + height/2);

  vertex[3].setX(m_location.getX() - width/2);
  vertex[3].setY(m_location.getY() - depth/2);
  vertex[3].setZ(m_location.getZ() + height/2);
}


void myCloud::Move()
{
  // just use default movement
  MovableObject::Move();
}


void myCloud::drawGeometry(){ 

  
  
  glEnable (GL_BLEND);
  glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

  float colorOffsetR = rand()%45/100;
  float colorOffsetG = rand()%45/100;
  float colorOffsetB = rand()%45/100;
  glColor4f(1-colorOffsetR,1-colorOffsetG,1-colorOffsetB,0.5);
  
  DrawSphericalCloud();

/* 

   **** Draw Triangles going from center to next 2 vertecies *****
	 
	 glBegin(GL_TRIANGLE_FAN);
	 {
	 for (int i = 0; i < NUM_VERTEX; i++){
	 glVertex3f(
	 vertex[i].getX(),
	 vertex[i].getY(),
	 vertex[i].getZ()
	 );
	 }
	 }
	 glEnd();
	
*/

 glDisable(GL_BLEND);
}

void myCloud::DrawSphericalCloud(){
  
  glPushMatrix();{
    glScalef(width,depth, height);
    glutSolidSphere(1, 5,5);
    //glutSolidTorus(0.2, .4, 10, 14);
  }glPopMatrix();

}

void myCloud::DrawCurvedSurfaceCloud(){
  /*COPIED PASTED AND MODIFIED SLIGHTLY FROM
    http://www.gamedev.net/reference/programming/features/curvessurfaces/page2.asp
   */

// Sixteen control points (substitute these values with your own if you like)
  
  float offsetHeight = rand()%50;
  /***************VERTEX 1*******************/
  double Ax = vertex[0].getX(); 
  double Ay = vertex[0].getY(); 
  double Az = vertex[0].getZ();

          //VECTOR 1 of vertex 2
  double Bx =  depth/2;
  double By =  0.1; 
  double Bz =  0.1;
  //VECTOR 2 of vertex 1
  double Ex =  0.1; 
  double Ey =  -width/2;
  double Ez =  0.1;
  //VECTOR 3 of vertex 1
  double Fx =  0.1; 
  double Fy =  0.1; 
  double Fz =  offsetHeight;

 /***************VERTEX 2*******************/
  double Dx = vertex[1].getX(); 
  double Dy = vertex[1].getY(); 
  double Dz = vertex[1].getZ();

         //VECTOR 1 of vertex 1
  double Cx =  depth/2;
  double Cy =  0.1; 
  double Cz =  0.1;
          //VECTOR 2 of vertex 2
  double Gx =  0.1; 
  double Gy =  0.1; 
  double Gz =  offsetHeight;
          //VECTOR 3 of vertex 2
  double Hx =  0.1; 
  double Hy =  width/2;
  double Hz =  0.1;


  /***************VERTEX 4*******************/
  //Vertex 3
  double Px = vertex[2].getX(); 
  double Py = vertex[2].getY(); 
  double Pz = vertex[2].getZ();
  
  //VECTOR 1 of vertex 3
  double Kx =  0.1; 
  double Ky =  0.1;
  double Kz =   offsetHeight;
  //VECTOR 2 of vertex 3
  double Lx =  0.1; 
  double Ly =  width/2;   
  double Lz =  0.1;
  //VECTOR 3 of vertex 3
  double Ox =  -depth/2;
  double Oy =  0.1; 
  double Oz =  0.1;
  
  /***************VERTEX 4*******************/
  double Mx = vertex[3].getX(); 
  double My = vertex[3].getY();
  double Mz = vertex[3].getZ();
              //VECTOR 1 of vertex 4
  double Ix =  0.1; 
  double Iy =  -width/2;
  double Iz =  0.1;
  //VECTOR 2 of vertex 4
  double Jx =  0.1; 
  double Jy =  0.1; 
  double Jz =  offsetHeight ;
  //VECTOR 3 of vertex 4
  double Nx =  -depth/2;  
  double Ny =  0.1; 
  double Nz =  0.1;




/* We will not actually draw a curved surface, but we will divide the
surface into small quads and draw them. If the quads are small enough,
it will appear as a curved surface. We will use a variable, detail, to
define how many quads to use. Since the variables goes from 1.0 to 0.0
we must change them by 1/detail from vertex to vertex. We will also
store the vertices and the normal vectors in arrays and draw them in a
separate loop */

// Detail of 10 mean that we will calculate 1111 vertices
int    detail = 3;
double change = 1.0 / (double)detail;

// Vertices (maximum detail will now be 2020 quads)
double Xv[21][21];
double Yv[21][21];
double Zv[21][21];

// Normal vectors
double Xn[21][21];
double Yn[21][21];
double Zn[21][21];

// Just making sure that the detail level is not set too high
if(detail > 20)
{
  detail = 20;
}

// Variables
double a = 1.0;
double b = 1.0 - a;
double c = 1.0;
double d = 1.0 - c;

// Tangent vectors
double Xta;
double Yta;
double Zta;

double Xtc;
double Ytc;
double Ztc;

/* Since we have two variables, we need two loops, we will change the
a-variable from 1.0 to 0.0 by steps of 1/detail ( = change), and for each
step we loop the c-variable from 1.0 to 0.0, thus creating a grid of
points covering the surface. Note that we could have had separate detail
 levels for the a-variable and the c-variable if we wanted to */
for(int i = 0; i <= detail; i++)
{
  for(int j = 0; j <= detail; j++)
  {
    // First get the vertices
    Xv[i][j] = Ax*a*a*a*c*c*c   + Bx*3*a*a*a*c*c*d
             + Cx*3*a*a*a*c*d*d + Dx*a*a*a*d*d*d
             + Ex*3*a*a*b*c*c*c + Fx*9*a*a*b*c*c*d
             + Gx*9*a*a*b*c*d*d + Hx*3*a*a*b*d*d*d
             + Ix*3*a*b*b*c*c*c + Jx*9*a*b*b*c*c*d
             + Kx*9*a*b*b*c*d*d + Lx*3*a*b*b*d*d*d
             + Mx*b*b*b*c*c*c   + Nx*3*b*b*b*c*c*d
             + Ox*3*b*b*b*c*d*d + Px*b*b*b*d*d*d;

    Yv[i][j] = Ay*a*a*a*c*c*c   + By*3*a*a*a*c*c*d
             + Cy*3*a*a*a*c*d*d + Dy*a*a*a*d*d*d
             + Ey*3*a*a*b*c*c*c + Fy*9*a*a*b*c*c*d
             + Gy*9*a*a*b*c*d*d + Hy*3*a*a*b*d*d*d
             + Iy*3*a*b*b*c*c*c + Jy*9*a*b*b*c*c*d
             + Ky*9*a*b*b*c*d*d + Ly*3*a*b*b*d*d*d
             + My*b*b*b*c*c*c   + Ny*3*b*b*b*c*c*d
             + Oy*3*b*b*b*c*d*d + Py*b*b*b*d*d*d;

    Zv[i][j] = Az*a*a*a*c*c*c   + Bz*3*a*a*a*c*c*d
             + Cz*3*a*a*a*c*d*d + Dz*a*a*a*d*d*d
             + Ez*3*a*a*b*c*c*c + Fz*9*a*a*b*c*c*d
             + Gz*9*a*a*b*c*d*d + Hz*3*a*a*b*d*d*d
             + Iz*3*a*b*b*c*c*c + Jz*9*a*b*b*c*c*d
             + Kz*9*a*b*b*c*d*d + Lz*3*a*b*b*d*d*d
             + Mz*b*b*b*c*c*c   + Nz*3*b*b*b*c*c*d
             + Oz*3*b*b*b*c*d*d + Pz*b*b*b*d*d*d;

    // Then use the derived functions to get the tangent vectors
    Xta = Ax*3*a*a*c*c*c           + Bx*9*a*a*c*c*d
        + Cx*9*a*a*c*d*d           + Dx*3*a*a*d*d*d
        + Ex*3*(2*a-3*a*a)*c*c*c   + Fx*9*(2*a-3*a*a)*c*c*d
        + Gx*9*(2*a-3*a*a)*c*d*d   + Hx*3*(2*a-3*a*a)*d*d*d
        + Ix*3*(1-4*a+3*a*a)*c*c*c + Jx*9*(1-4*a+3*a*a)*c*c*d
        + Kx*9*(1-4*a+3*a*a)*c*d*d + Lx*3*(1-4*a+3*a*a)*d*d*d
        + Mx*3*(2*a-1-a*a)*c*c*c   + Nx*9*(2*a-1-a*a)*c*c*d
        + Ox*9*(2*a-1-a*a)*c*d*d   + Px*3*(2*a-1-a*a)*d*d*d;

    Yta = Ay*3*a*a*c*c*c           + By*9*a*a*c*c*d
        + Cy*9*a*a*c*d*d           + Dy*3*a*a*d*d*d
        + Ey*3*(2*a-3*a*a)*c*c*c   + Fy*9*(2*a-3*a*a)*c*c*d
        + Gy*9*(2*a-3*a*a)*c*d*d   + Hy*3*(2*a-3*a*a)*d*d*d
        + Iy*3*(1-4*a+3*a*a)*c*c*c + Jy*9*(1-4*a+3*a*a)*c*c*d
        + Ky*9*(1-4*a+3*a*a)*c*d*d + Ly*3*(1-4*a+3*a*a)*d*d*d
        + My*3*(2*a-1-a*a)*c*c*c   + Ny*9*(2*a-1-a*a)*c*c*d
        + Oy*9*(2*a-1-a*a)*c*d*d   + Py*3*(2*a-1-a*a)*d*d*d;

    Zta = Az*3*a*a*c*c*c           + Bz*9*a*a*c*c*d
        + Cz*9*a*a*c*d*d           + Dz*3*a*a*d*d*d
        + Ez*3*(2*a-3*a*a)*c*c*c   + Fz*9*(2*a-3*a*a)*c*c*d
        + Gz*9*(2*a-3*a*a)*c*d*d   + Hz*3*(2*a-3*a*a)*d*d*d
        + Iz*3*(1-4*a+3*a*a)*c*c*c + Jz*9*(1-4*a+3*a*a)*c*c*d
        + Kz*9*(1-4*a+3*a*a)*c*d*d + Lz*3*(1-4*a+3*a*a)*d*d*d
        + Mz*3*(2*a-1-a*a)*c*c*c   + Nz*9*(2*a-1-a*a)*c*c*d
        + Oz*9*(2*a-1-a*a)*c*d*d   + Pz*3*(2*a-1-a*a)*d*d*d;

    Xtc = Ax*3*a*a*a*c*c           + Bx*3*a*a*a*(2*c-3*c*c)
        + Cx*3*a*a*a*(1-4*c+3*c*c) + Dx*3*a*a*a*(-1+2*c-c*c)
        + Ex*9*a*a*b*c*c           + Fx*9*a*a*b*(2*c-3*c*c)
        + Gx*9*a*a*b*(1-4*c+3*c*c) + Hx*9*a*a*b*(-1+2*c-c*c)
        + Ix*9*a*b*b*c*c           + Jx*9*a*b*b*(2*c-3*c*c)
        + Kx*9*a*b*b*(1-4*c+3*c*c) + Lx*9*a*b*b*(-1+2*c-c*c)
        + Mx*3*b*b*b*c*c           + Nx*3*b*b*b*(2*c-3*c*c)
        + Ox*3*b*b*b*(1-4*c+3*c*c) + Px*3*b*b*b*(-1+2*c-c*c);

    Ytc = Ay*3*a*a*a*c*c           + By*3*a*a*a*(2*c-3*c*c)
        + Cy*3*a*a*a*(1-4*c+3*c*c) + Dy*3*a*a*a*(-1+2*c-c*c)
        + Ey*9*a*a*b*c*c           + Fy*9*a*a*b*(2*c-3*c*c)
        + Gy*9*a*a*b*(1-4*c+3*c*c) + Hy*9*a*a*b*(-1+2*c-c*c)
        + Iy*9*a*b*b*c*c           + Jy*9*a*b*b*(2*c-3*c*c)
        + Ky*9*a*b*b*(1-4*c+3*c*c) + Ly*9*a*b*b*(-1+2*c-c*c)
        + My*3*b*b*b*c*c           + Ny*3*b*b*b*(2*c-3*c*c)
        + Oy*3*b*b*b*(1-4*c+3*c*c) + Py*3*b*b*b*(-1+2*c-c*c);

    Ztc = Az*3*a*a*a*c*c           + Bz*3*a*a*a*(2*c-3*c*c)
        + Cz*3*a*a*a*(1-4*c+3*c*c) + Dz*3*a*a*a*(-1+2*c-c*c)
        + Ez*9*a*a*b*c*c           + Fz*9*a*a*b*(2*c-3*c*c)
        + Gz*9*a*a*b*(1-4*c+3*c*c) + Hz*9*a*a*b*(-1+2*c-c*c)
        + Iz*9*a*b*b*c*c           + Jz*9*a*b*b*(2*c-3*c*c)
        + Kz*9*a*b*b*(1-4*c+3*c*c) + Lz*9*a*b*b*(-1+2*c-c*c)
        + Mz*3*b*b*b*c*c           + Nz*3*b*b*b*(2*c-3*c*c)
        + Oz*3*b*b*b*(1-4*c+3*c*c) + Pz*3*b*b*b*(-1+2*c-c*c);

    // Cross the tangent vectors, put the result to the normal vector array
    // Note: I simplified -((Xta*Ztc)-(Xtc*Zta)) to (Xtc*Zta) - (Xta*Ztc)
    Xn[i][j] = (Yta*Ztc) - (Ytc*Zta);
    Yn[i][j] = (Xtc*Zta) - (Xta*Ztc);
    Zn[i][j] = (Xta*Ytc) - (Xtc*Yta);

    // Find length of normal vector
    double length = sqrt((Xn[i][j]*Xn[i][j])+(Yn[i][j]
                         *Yn[i][j])+(Zn[i][j]*Zn[i][j]));

    // Normalize (and prevent divide by zero error)
    if(length > 0)
    {
      length = 1.0/length;
      Xn[i][j] *= length;
      Yn[i][j] *= length;
      Zn[i][j] *= length;
    }

    //change the c-variable within the inner loop
    c -= change;
    d  = 1.0 - c;
  }
  //change the a-variable outside the inner loop
  a -= change;
  b  = 1.0 - a;

  // Reset the c-variable to make it ready for the inner loop again
  c = 1.0;
  d = 1.0 - c;
}


/* Now we have two arrays, one with vertices, and one with normal vectors,
drawing them is straightforward if you know how to use a graphics API.
Following is one way to do it using openGL and triangle strips. (assuming
GL_LIGHTING etc.. has been properly set up) */
for(int m = 0; m < detail; m++)
{
  glBegin(GL_TRIANGLE_STRIP);
  for(int n = 0; n <= detail; n++)
  {
    glNormal3d(Xn[m][n],Yn[m][n],Zn[m][n]);
    glVertex3d(Xv[m][n],Yv[m][n],Zv[m][n]);

    // Note that I used real-less-than in the first loop, since I want to
    // access the m+1 entry in the array to properly draw the triangle strip
    glNormal3d(Xn[m+1][n],Yn[m+1][n],Zn[m+1][n]);
    glVertex3d(Xv[m+1][n],Yv[m+1][n],Zv[m+1][n]);
  }
  glEnd();
}
}

#endif
