/* Poopmup - An urban flight sim
 *
 * Copyright (C) 2002 Zaroubi, Fink, Leclerc, Stanek szaroubi@hotmail.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* Map Object
   We can use this as a base class for Buildings, intersections, etc
   
*/

#ifndef POOP_EM_UP_MYPARK_H
#define POOP_EM_UP_MYPARK_H

#include "../../common.h"
#include "../mapObjects/MapObject.h"


class myPark : public MapObject
{
 public:
  myPark()  {setType(MAPOBJ_TYPE_PARK);}
  ~myPark() {}
  
  virtual void SetupDisplayList();
  float W,D,H;
  void MakeTree(float x, float y, float height);
  
};

void myPark::SetupDisplayList()
{
  // draw a park

  W = (m_East - m_West);
  D = (m_North - m_South);
  H = (m_Top - m_Bottom);
  

  if(!IsInPlace)
    {	
      // use the translation and scaling instead of the 
      translateBy(m_West, m_South, m_Bottom);
      scaleBy(W,D,H);
      IsInPlace = 1;
    }
  //scaleBy(10,10,10);
  
  
  glNewList(displayLists.getList(m_DisplayList), GL_COMPILE);
  {
    
    // Here is where the drawing should begin
    
    if(FAST_COMPUTER)
      glEnable(GL_CULL_FACE);
    
    glPolygonMode(GL_FRONT, GL_SMOOTH);
    
    // Top Face (the TOP plane)
    
    glEnable(POOP_TEXTURE_2D);
    glBindTexture( GL_TEXTURE_2D, m_texTop);	
    glNormal3f(0,0,10);
    glBegin(GL_QUADS);
    {
      glTexCoord2f(1, 1);			 glVertex3f(1,1,0);
      glTexCoord2f(1, 0.0f);		 glVertex3f(0,1,0);
      glTexCoord2f(0.0f, 0.0f);		 glVertex3f(0,0,0);
      glTexCoord2f(0.0f, 1);		 glVertex3f(1,0,0);
    }
    glEnd();
    glDisable( GL_TEXTURE_2D );
    
    // add a sphere just for fun
    // big blotch
    //glTranslatef(0.3, 0.3, 0.5);
    //glutSolidSphere(.25, 10, 10);
    
    if ( POOP_SHOW_TREES){
      // add some pretty trees
      int i = 3 + rand() % 15;
      while(i)
	{
	  MakeTree((rand()%100)/100.0,
		   (rand()%100)/100.0,
		   20 + (rand()%80));
	  i--;
	}
    }
  }
  glEndList(); 
  
}

void myPark::MakeTree(float x, float y, float height)
{
  
    // "tree"
  glPushMatrix();
  {
    glTranslatef(x, y, height/200);
    glPushMatrix();
    {
      glColor4f(.4,.1,.1, 1);
      glScalef(3/W,3/D,height/H);
      glutSolidCube(1); // trunk
    }
    glPopMatrix();

    // glutSolidSphere doesn't use textures
    glEnable(POOP_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D, m_texTop);

    glTranslatef(0,0,height/200*2);
    glPushMatrix();
    {
      glColor4f(0,0.75 - rand()%50/100.0,0, 0.6);
      float randVal = rand()%20;
      glScalef((15+randVal)/W, (15 + randVal)/D, height/H);

      glutSolidSphere(1,3,3);
    }
    glPopMatrix();
    glDisable(GL_TEXTURE_2D);
    //glutSolidSphere(.2, 10, 10); // top
  } 
  glPopMatrix();
  
} 

#endif
