/* Poopmup - An urban flight sim
 *
 * Copyright (C) 2002 Zaroubi, Fink, Leclerc, Stanek szaroubi@hotmail.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* Car class
   Stephen Fink
   Nov 5, 2002
   
*/

#ifndef POOP_EM_UP_CAR_H
#define POOP_EM_UP_CAR_H

#include "../../common.h"
#include "MovableObject.h"
#include "../textureLoader.h"

#define CAR_STARTING_VALUE 20
#define CAR_MAX_SPEED 20

class Car : public MovableObject
{
 private:
  int numPoopedOn;
  float value;

 public:
  int getValue(){return (int)value;}
  void poopedOn();
  int getNumPooped(){return numPoopedOn;}
  int isSoiled(){return numPoopedOn;}

  Car(); 
  ~Car() {}
  void drawGeometry();
  void Move();
  //  void ChangeDirection();
  void initTransphorm();

  //float carAngle;
  GLuint car_front, car_back, car_top, car_windshield, car_wheel, car_doorLeft, car_doorRight,car_sideWindow;
};

void Car::poopedOn(){
  setColor(0, .7, 0);
  numPoopedOn++;
  value /= numPoopedOn;
}

Car::Car()
{
  
  //	debugBB =1;
  numPoopedOn = 0;
  rotateObj = 0;
  value = CAR_STARTING_VALUE;
  m_TilesCanWalkOn = TILE_ROAD;
  m_facingDir.setXYZ(1,0,0);
  m_speed = CAR_MAX_SPEED - 10 + (rand() % 10);
  float r = 1-rand()%50/100.0;
  float g = 1-rand()%50/100.0;
  float b = 1-rand()%50/100.0;
  
  setColor(r,g,b);
  m_LocalRotation.setXYZ(0,0,90);
  initTransphorm();
  //rotateAroundZBy(m_LocalRotation.getZ());
}

void Car::initTransphorm(){
    //rotateAroundZBy(m_LocalRotation.getZ());    
    scaleBy(80,60,60);
}

void Car::drawGeometry()
{
 /* // draw the car. OOOH PRETTY!
    glutSolidCube(1);
    glPushMatrix();{
        glScalef(1,1,3);
        glColor3f(1, 0, 0);
        glutSolidCube(0.5);
    };  glPopMatrix();
	
*/

	glPushMatrix();
	
	glEnable( POOP_TEXTURE_2D );
	glRotatef(rotateObj + 90,0,0,1);

	// Car Front Face
	glBindTexture( GL_TEXTURE_2D, car_front);
	glBegin(GL_QUADS);
		glNormal3f( 0.0f, 0.0f, 1.0f);			
		glTexCoord2f(1.0f, 1.0f); glVertex3f(-3.0/18, 0.0/18+1/9,  -1.0/18);	
		glTexCoord2f(0.0f, 1.0f); glVertex3f( 3.0/18, 0.0/18+1/9,  -1.0/18);	
		glTexCoord2f(0.0f, 0.0f); glVertex3f( 3.0/18, 0.0/18+1/9,   1.0/18);	
		glTexCoord2f(1.0f, 0.0f); glVertex3f(-3.0/18, 0.0/18+1/9,   1.0/18);	
	glEnd();								

	
	// Car Capot
	glBindTexture( GL_TEXTURE_2D, car_top);
	glBegin(GL_QUADS);
		glNormal3f( 0.0f, 0.0f, 1.0f);			
		glTexCoord2f(0.0f, 1.0f); glVertex3f(-3.0/18,  0.0/18+1/9,  1.0/18);	
		glTexCoord2f(1.0f, 1.0f); glVertex3f( 3.0/18,  0.0/18+1/9,  1.0/18);	
		glTexCoord2f(1.0f, 0.0f); glVertex3f( 3.0/18,  5.0/18+1/9,  2.0/18);	
		glTexCoord2f(0.0f, 1.0f); glVertex3f(-3.0/18,  5.0/18+1/9,  2.0/18);	
	glEnd();								


	// Car Front Wheel left
	glBindTexture( GL_TEXTURE_2D, car_wheel);
	glBegin(GL_QUADS);
		glNormal3f( 1.0f, 0.0f, 0.0f);			
		
		glTexCoord2f(0.0f, 0.0f); glVertex3f( 3.0/18,  5.0/18+1/9,  2.0/18);
		glTexCoord2f(1.0f, 0.0f); glVertex3f( 3.0/18,  0.0/18+1/9,  1.0/18);
		glTexCoord2f(1.0f, 1.0f); glVertex3f(  3.0/18,  0.0/18+1/9, -1.0/18);
		glTexCoord2f(0.0f, 1.0f); glVertex3f( 3.0/18,  5.0/18+1/9, -1.0/18);
	glEnd();								
	
	// Car Front Wheel right
	glBindTexture( GL_TEXTURE_2D, car_wheel);
	glBegin(GL_QUADS);
		glNormal3f( -1.0f, 0.0f, 0.0f);			
		glTexCoord2f(1.0f, 0.0f); glVertex3f( -3.0/18,  0.0/18+1/9,   1.0/18);
		glTexCoord2f(0.0f, 0.0f); glVertex3f( -3.0/18,  5.0/18+1/9,   2.0/18);
		glTexCoord2f(0.0f, 1.0f); glVertex3f( -3.0/18,  5.0/18+1/9,  -1.0/18);
		glTexCoord2f(1.0f, 1.0f); glVertex3f( -3.0/18,  0.0/18+1/9,  -1.0/18);
	glEnd();			
	



	// Car Windshield
	glBindTexture( GL_TEXTURE_2D, car_windshield);
	glBegin(GL_QUADS);
		glNormal3f( 1.0f, 0.0f, 1.0f);			
		glTexCoord2f(0.0f, 0.0f); glVertex3f( 3.0/18,  9.0/18+1/9,  4.0/18);	
		glTexCoord2f(1.0f, 0.0f); glVertex3f( -3.0/18, 9.0/18+1/9,  4.0/18);	
		glTexCoord2f(1.0f, 1.0f); glVertex3f( -3.0/18, 5.0/18+1/9,  2.0/18);	
		glTexCoord2f(0.0f, 1.0f); glVertex3f( 3.0/18,  5.0/18+1/9,  2.0/18);	
	glEnd();								


	
	// Car door Left
	glBindTexture( GL_TEXTURE_2D, car_doorLeft);
	glBegin(GL_QUADS);
		glNormal3f( 1.0f, 0.0f, 0.0f);			
		glTexCoord2f(1.0f, 0.0f); glVertex3f( 3.0/18, 15.0/18+1/9,  4.0/18);	
		glTexCoord2f(0.0f, 0.0f); glVertex3f( 3.0/18, 9.0/18+1/9,   4.0/18);	
		glTexCoord2f(0.0f, 1.0f); glVertex3f( 3.0/18, 9.0/18+1/9,  -1.0/18);	
		glTexCoord2f(1.0f, 1.0f); glVertex3f( 3.0/18, 15.0/18+1/9, -1.0/18);		
	glEnd();								


	// Car door Right
	glBindTexture( GL_TEXTURE_2D, car_doorRight);
	glBegin(GL_QUADS);
		glNormal3f( -1.0f, 0.0f, 0.0f);			
		glTexCoord2f(0.0f, 0.0f); glVertex3f( -3.0/18, 9.0/18+1/9,   4.0/18);	
		glTexCoord2f(1.0f, 0.0f); glVertex3f( -3.0/18, 15.0/18+1/9,  4.0/18);	
		glTexCoord2f(1.0f, 1.0f); glVertex3f( -3.0/18, 15.0/18+1/9, -1.0/18);	
		glTexCoord2f(0.0f, 1.0f); glVertex3f( -3.0/18, 9.0/18+1/9,  -1.0/18);			
	glEnd();								
		
	// Car TOP
	glBindTexture( GL_TEXTURE_2D, car_top);
	glBegin(GL_QUADS);
		glNormal3f( 0.0f, 0.0f, 1.0f);			
		glTexCoord2f(1.0f, 1.0f); glVertex3f( -3.0/18,  15.0/18+1/9, 4.0/18);	
		glTexCoord2f(0.0f, 1.0f); glVertex3f( -3.0/18,  9.0 /18+1/9,  4.0/18);	
		glTexCoord2f(0.0f, 0.0f); glVertex3f(  3.0/18,  9.0 /18+1/9,  4.0/18);	
		glTexCoord2f(1.0f, 0.0f); glVertex3f(  3.0/18,  15.0/18+1/9, 4.0/18);	
	glEnd();								
	
	
	// Car sideWindow front-Left
	glBindTexture( GL_TEXTURE_2D, car_sideWindow);
	glBegin(GL_TRIANGLES);
		glNormal3f( 1.0f, 0.0f, 0.0f);			
		glTexCoord2f(1.0f, 1.0f); glVertex3f( 3.0/18,  5.0/18+1/9,  2.0/18);	 
		glTexCoord2f(0.0f, 1.0f); glVertex3f( 3.0/18,  9.0/18+1/9,  2.0/18);	
		glTexCoord2f(1.0f, 0.0f); glVertex3f( 3.0/18,  9.0/18+1/9,  4.0/18);	
	glEnd();								

	// Car SideWindow front-bottom-Right
	glBindTexture( GL_TEXTURE_2D, car_top);
	glBegin(GL_QUADS);
		glNormal3f( -1.0f, 0.0f, 0.0f);			
		glTexCoord2f(0.0f, 0.0f); glVertex3f( -3.0/18, 5.0/18+1/9,  2.0/18);	
		glTexCoord2f(0.0f, 1.0f); glVertex3f( -3.0/18, 9.0/18+1/9,  2.0/18);	
		glTexCoord2f(1.0f, 1.0f); glVertex3f( -3.0/18, 9.0/18+1/9, -1.0/18);	
		glTexCoord2f(1.0f, 0.0f); glVertex3f( -3.0/18, 5.0/18+1/9, -1.0/18);	
	glEnd();								
	
	
	// Car sideWindow Front-Right
	glBindTexture( GL_TEXTURE_2D, car_sideWindow);
	glBegin(GL_TRIANGLES);
		glNormal3f( -1.0f, 0.0f, 0.0f);			
		glTexCoord2f(1.0f, 0.0f); glVertex3f( -3.0/18,  9.0/18+1/9,  4.0/18);
		glTexCoord2f(0.0f, 1.0f); glVertex3f( -3.0/18,  9.0/18+1/9,  2.0/18);
		glTexCoord2f(1.0f, 1.0f); glVertex3f( -3.0/18,  5.0/18+1/9,  2.0/18); 
	glEnd();								

	// Car SideWindow Front-bottom-Left
	glBindTexture( GL_TEXTURE_2D, car_top);
	glBegin(GL_QUADS);
		glNormal3f( 1.0f, 0.0f, 0.0f);			
		
		glTexCoord2f(0.0f, 1.0f); glVertex3f( 3.0/18, 9.0/18+1/9,  2.0/18);	
		glTexCoord2f(0.0f, 0.0f); glVertex3f( 3.0/18, 5.0/18+1/9,  2.0/18);	
		glTexCoord2f(1.0f, 0.0f); glVertex3f( 3.0/18, 5.0/18+1/9, -1.0/18 );	
		glTexCoord2f(1.0f, 1.0f); glVertex3f( 3.0/18, 9.0/18+1/9, -1.0/18);	
	glEnd();								
	
	// Car Back Window
	glBindTexture( GL_TEXTURE_2D, car_windshield);
	glBegin(GL_QUADS);
		glNormal3f( 1.0f, 1.0f, 0.0f);			
		glTexCoord2f(1.0f, 1.0f); glVertex3f( -3.0/18, 16.0/18+1/9,  2.0/18);	
		glTexCoord2f(1.0f, 0.0f); glVertex3f( -3.0/18, 15.0/18+1/9,  4.0/18);	
		glTexCoord2f(0.0f, 0.0f); glVertex3f( 3.0/18,  15.0/18+1/9,  4.0/18);	
		glTexCoord2f(0.0f, 1.0f); glVertex3f( 3.0/18,  16.0/18+1/9,  2.0/18);	
	glEnd();								
															
	// Car sideWindow back-Right
	glBindTexture( GL_TEXTURE_2D, car_sideWindow);
	glBegin(GL_TRIANGLES);
		glNormal3f( -1.0f, 0.0f, 0.0f);			
		glTexCoord2f(1.0f, 1.0f); glVertex3f( -3.0/18,  15.0/18+1/9, 2.0/18);	 
		glTexCoord2f(1.0f, 0.0f); glVertex3f( -3.0/18,  15.0/18+1/9, 4.0/18);	
		glTexCoord2f(0.0f, 1.0f); glVertex3f( -3.0/18,  16.0/18+1/9, 2.0/18);	
	glEnd();																	
											
	// Car sideWindow back-Left
	glBindTexture( GL_TEXTURE_2D, car_sideWindow);
	glBegin(GL_TRIANGLES);
		glNormal3f( 1.0f, 0.0f, 0.0f);			
		glTexCoord2f(0.0f, 1.0f); glVertex3f( 3.0/18,  16.0/18+1/9, 2.0/18);	
		glTexCoord2f(1.0f, 0.0f); glVertex3f( 3.0/18,  15.0/18+1/9, 4.0/18);	
		glTexCoord2f(1.0f, 1.0f); glVertex3f( 3.0/18,  15.0/18+1/9, 2.0/18);	
	glEnd();																												
											
											
	// Car sideWindow botton-back-right
	glBindTexture( GL_TEXTURE_2D, car_top);
	glBegin(GL_QUADS);
		glNormal3f( -1.0f, 0.0f, 0.0f);			
		glTexCoord2f(0.0f, 1.0f); glVertex3f( -3.0/18,  15.0/18+1/9,  2.0/18);	
		glTexCoord2f(0.0f, 0.0f); glVertex3f( -3.0/18,  16.0/18+1/9,  2.0/18);	
		glTexCoord2f(1.0f, 0.0f); glVertex3f( -3.0/18,  16.0/18+1/9, -1.0/18);
		glTexCoord2f(1.0f, 1.0f); glVertex3f( -3.0/18,  15.0/18+1/9, -1.0/18);
	glEnd();																		

	// Car sideWindow botton-back-Left
    glBindTexture( GL_TEXTURE_2D, car_top);
	glBegin(GL_QUADS);
		glNormal3f( 1.0f, 0.0f, 0.0f);			
		
		glTexCoord2f(0.0f, 0.0f); glVertex3f( 3.0/18,  16.0/18+1/9,   2.0/18);
		glTexCoord2f(0.0f, 1.0f); glVertex3f( 3.0/18,  15.0/18+1/9,   2.0/18);		
		glTexCoord2f(1.0f, 1.0f); glVertex3f( 3.0/18,  15.0/18+1/9,  -1.0/18);
		glTexCoord2f(1.0f, 0.0f); glVertex3f( 3.0/18,  16.0/18+1/9,  -1.0/18);
	glEnd();																	

	// Car sideWindow back wheel Left
    glBindTexture( GL_TEXTURE_2D, car_wheel);
	glBegin(GL_QUADS);
		glNormal3f( 1.0f, 0.0f, 0.0f);			
		glTexCoord2f(1.0f, 0.0f); glVertex3f( 3.0/18,  18.0/18+1/9,  1.0/18);	
		glTexCoord2f(0.0f, 0.0f); glVertex3f( 3.0/18,  16.0/18+1/9,  2.0/18);	
		glTexCoord2f(0.0f, 1.0f); glVertex3f( 3.0/18,  16.0/18+1/9, -1.0/18);
		glTexCoord2f(1.0f, 0.0f); glVertex3f( 3.0/18,  18.0/18+1/9, -1.0/18);	
	glEnd();								

	
	// Car sideWindow back wheel Right
    glBindTexture( GL_TEXTURE_2D, car_wheel);
	glBegin(GL_QUADS);
		glNormal3f( -1.0f, 0.0f, 0.0f);			
		glTexCoord2f(0.0f, 0.0f); glVertex3f( -3.0/18,  16.0/18+1/9,  2.0/18);		
		glTexCoord2f(1.0f, 0.0f); glVertex3f( -3.0/18,  18.0/18+1/9,  1.0/18);		
		glTexCoord2f(1.0f, 1.0f); glVertex3f( -3.0/18,  18.0/18+1/9, -1.0/18);		
		glTexCoord2f(0.0f, 1.0f); glVertex3f( -3.0/18,  16.0/18+1/9, -1.0/18);		
	glEnd();								

	// Car BACK capot
	glBindTexture( GL_TEXTURE_2D, car_top);
	glBegin(GL_QUADS);
		glNormal3f( 0.0f, 0.0f, 1.0f);			
		glTexCoord2f(1.0f, 0.0f); glVertex3f( 3.0/18,  16.0/18+1/9, 2.0/18);	
		glTexCoord2f(1.0f, 1.0f); glVertex3f( 3.0/18,  18.0/18+1/9, 1.0/18);	
		glTexCoord2f(0.0f, 1.0f); glVertex3f(-3.0/18,  18.0/18+1/9, 1.0/18);	
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-3.0/18,  16.0/18+1/9, 2.0/18);	
	glEnd();						
	
	// Car Back
	glBindTexture( GL_TEXTURE_2D, car_back);
	glBegin(GL_QUADS);
		glNormal3f( 0.0f, 0.0f, 1.0f);			
		glTexCoord2f(1.0f, 0.0f); glVertex3f(-3.0/18, 18.0/18+1/9,   1.0/18);	
		glTexCoord2f(0.0f, 0.0f); glVertex3f( 3.0/18, 18.0/18+1/9,   1.0/18);		
		glTexCoord2f(0.0f, 1.0f); glVertex3f( 3.0/18, 18.0/18+1/9,  -1.0/18);	
		glTexCoord2f(1.0f, 1.0f); glVertex3f(-3.0/18, 18.0/18+1/9,  -1.0/18);	
	glEnd();							

	glDisable( GL_TEXTURE_2D );
	glPopMatrix();

}

void Car::Move()
{
    // just use default movement
    MovableObject::Move();
}


#endif
