/* Poopmup - An urban flight sim
 *
 * Copyright (C) 2002 Zaroubi, Fink, Leclerc, Stanek szaroubi@hotmail.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* Pedestrian class
   Stephen Fink
   Oct 26, 2002
   
*/

#ifndef POOP_EM_UP_PEDESTRIAN_H
#define POOP_EM_UP_PEDESTRIAN_H

#include "../../common.h"
#include "MovableObject.h"

#define PED_MAX_SPEED 5
#define PED_STARTING_VALUE 10
class Pedestrian : public MovableObject
{
 private:
  int numPoopedOn;
  float value;
  int frameNumber;

 public:
  int getValue(){return (int)value;}
  void poopedOn();
  int getNumPooped(){return numPoopedOn;}
  int isSoiled(){return numPoopedOn;}
  
  Pedestrian(); 
  ~Pedestrian() {}
  void drawPed(float x, float y, float z);
  void drawPieceOfClothing();
  void draw_WC_Body();
  void draw_WC_Head();
  void drawArm();
  void drawLeg();
  void drawRightArm();
  void drawLeftArm();
  void drawLeftLeg();
  void drawRightLeg();
  void drawGeometry();
  void Move();
  //  void ChangeDirection();
  
  
  GLuint ped_wc_headface, ped_wc_headsideLeft,ped_wc_headsideRight,ped_wc_headback,ped_wc_headtop,ped_wc_bodyfront,ped_wc_bodyback, ped_wc_bodymisc;
  int rotateLeg;
  //  int rotatePed;
};

void Pedestrian::poopedOn(){
  setColor(0, .7, 0);
  numPoopedOn++;
  value /=numPoopedOn;
}

Pedestrian::Pedestrian()
{
  rotateObj = 0;
  rotateLeg = 0;
  frameNumber = 0;
  numPoopedOn = 0;

  value = PED_STARTING_VALUE;
  m_TilesCanWalkOn = TILE_SIDEWALK;
  m_speed = PED_MAX_SPEED - 3 + rand()%3;
  //setColor(0.5,0,0);
  scaleBy(5,5,5);
}

void Pedestrian::drawPieceOfClothing(){
  glPushMatrix();
  glEnable( GL_TEXTURE_2D );
  glBindTexture( GL_TEXTURE_2D, ped_wc_bodymisc);
  glBegin(GL_QUADS);
  
  // Front Face
  glNormal3f( 0.0f, 0.0f, 1.0f);			// Normal Pointing Towards Viewer
  glTexCoord2f(0.0f, 0.0f); glVertex3f(-1.0f, 1.0f,  -1.0f);	
  glTexCoord2f(1.0f, 0.0f); glVertex3f( 1.0f, 1.0f,  -1.0f);	
  glTexCoord2f(1.0f, 1.0f); glVertex3f( 1.0f,  1.0f,  1.0f);	
  glTexCoord2f(0.0f, 1.0f); glVertex3f(-1.0f,  1.0f,  1.0f);	
  
  // Back Face
  glNormal3f( 0.0f, 0.0f,-1.0f);					// Normal Pointing Away From Viewer
  glTexCoord2f(1.0f, 0.0f); glVertex3f(-1.0f, -1.0f,  -1.0f);	
  glTexCoord2f(1.0f, 1.0f); glVertex3f(-1.0f,  -1.0f,  1.0f);	
  glTexCoord2f(0.0f, 1.0f); glVertex3f( 1.0f,  -1.0f,  1.0f);	
  glTexCoord2f(0.0f, 0.0f); glVertex3f( 1.0f, -1.0f, -1.0f);	
  // Top Face
  glNormal3f( 0.0f, 1.0f, 0.0f);					// Normal Pointing Up
  glTexCoord2f(0.0f, 1.0f); glVertex3f(-1.0f,  -1.0f, 1.0f);	
  glTexCoord2f(0.0f, 0.0f); glVertex3f(-1.0f,  1.0f,  1.0f);	
  glTexCoord2f(1.0f, 0.0f); glVertex3f( 1.0f,  1.0f,  1.0f);	
  glTexCoord2f(1.0f, 1.0f); glVertex3f( 1.0f,  -1.0f, 1.0f);	
  // Bottom Face
  glNormal3f( 0.0f,-1.0f, 0.0f);					// Normal Pointing Down
  glTexCoord2f(1.0f, 1.0f); glVertex3f(-1.0f, -1.0f, -1.0f);	
  glTexCoord2f(0.0f, 1.0f); glVertex3f( 1.0f, -1.0f, -1.0f);	
  glTexCoord2f(0.0f, 0.0f); glVertex3f( 1.0f, 1.0f,  -1.0f);	
  glTexCoord2f(1.0f, 0.0f); glVertex3f(-1.0f, 1.0f,  -1.0f);	
  // Right face
  glNormal3f( 1.0f, 0.0f, 0.0f);					// Normal Pointing Right
  glTexCoord2f(1.0f, 0.0f); glVertex3f( 1.0f, -1.0f, -1.0f);	
  glTexCoord2f(1.0f, 1.0f); glVertex3f( 1.0f,  -1.0f, 1.0f);	
  glTexCoord2f(0.0f, 1.0f); glVertex3f( 1.0f,  1.0f,  1.0f);	
  glTexCoord2f(0.0f, 0.0f); glVertex3f( 1.0f, 1.0f,  -1.0f);	
  // Left Face
  glNormal3f(-1.0f, 0.0f, 0.0f);					// Normal Pointing Left
  glTexCoord2f(0.0, 0.0f);  glVertex3f(-1.0f, -1.0f, -1.0f);	
  glTexCoord2f(1.0, 0.0f);  glVertex3f(-1.0f, 1.0f,  -1.0f);	
  glTexCoord2f(1.0f, 1.0f); glVertex3f(-1.0f,  1.0f,  1.0f);	
  glTexCoord2f(0.0f, 1.0f); glVertex3f(-1.0f,  -1.0f, 1.0f);	
  glEnd();								
  
  glDisable(GL_TEXTURE_2D);
  glPopMatrix();
}

void Pedestrian::draw_WC_Body() {
  
  
  glPushMatrix();
  glEnable( GL_TEXTURE_2D );
  
  //----------------------- body -------------------------------
  
  
  // Body Front 
  glBindTexture( GL_TEXTURE_2D, ped_wc_bodyfront);
  glBegin(GL_QUADS);
  glNormal3f( 0.0f, 0.0f, 1.0f);			// Normal Pointing Towards Viewer
  glTexCoord2f(0.0f, 1.0f); glVertex3f( 0.5f, 0.5f, -0.5f);	
  glTexCoord2f(1.0f, 1.0f); glVertex3f(-0.5,  0.5,  -0.5f);	
  glTexCoord2f(1.0f, 0.0f); glVertex3f(-0.5f, 0.5f,  0.5f);	
  glTexCoord2f(0.0f, 0.0f); glVertex3f( 0.5f, 0.5f,  0.5f);	
  glEnd();	
  
  
  //  body Right Side
  glBindTexture( GL_TEXTURE_2D, ped_wc_bodymisc);
  glBegin(GL_QUADS);
  glNormal3f( -1.0f, 0.0f, 0.0f);			// Normal Pointing Towards Viewer
  
  glTexCoord2f(0.0f, 1.0f); glVertex3f(-0.5f,  0.5,  -0.5f);	
  glTexCoord2f(1.0f, 1.0f); glVertex3f(-0.5f, -0.5f, -0.5f);	
  
  glTexCoord2f(1.0f, 0.0f); glVertex3f(-0.5f, -0.5f,  0.5f);	
  glTexCoord2f(0.0f, 0.0f); glVertex3f(-0.5f,  0.5f,  0.5f);	
  glEnd();	
  
  
  //  Body Left Side
  glBindTexture( GL_TEXTURE_2D, ped_wc_bodymisc);
  glBegin(GL_QUADS);
  glNormal3f( 1.0f, 0.0f, 0.0f);			// Normal Pointing Towards Viewer
  glTexCoord2f(1.0f, 1.0f); glVertex3f(0.5f, -0.5f, -0.5f);	
  glTexCoord2f(0.0f, 1.0f); glVertex3f(0.5f,  0.5f, -0.5f);	
  glTexCoord2f(0.0f, 0.0f); glVertex3f(0.5f,  0.5f,  0.5f);	
  glTexCoord2f(1.0f, 0.0f); glVertex3f(0.5f, -0.5f,  0.5f);	
  glEnd();	
  
  
  // Body Top
  glBindTexture( GL_TEXTURE_2D, ped_wc_bodymisc);
  glBegin(GL_QUADS);
  glNormal3f( 0.0f, 1.0f, 0.0f);			
  glTexCoord2f(0.0f, 1.0f); glVertex3f( 0.5f, -0.5f, 0.5f);	
  glTexCoord2f(1.0f, 1.0f); glVertex3f( 0.5f,  0.5f, 0.5f);	
  glTexCoord2f(1.0f, 0.0f); glVertex3f(-0.5f,  0.5f, 0.5f);	
  glTexCoord2f(0.0f, 1.0f); glVertex3f(-0.5f, -0.5f, 0.5f);	
  glEnd();
  
  
  // Body back
  glBindTexture( GL_TEXTURE_2D, ped_wc_bodyback);
  glBegin(GL_QUADS);
  glNormal3f( 0.0f, 0.0f, -1.0f);			// Normal Pointing Towards Viewer
  glTexCoord2f(1.0f, 1.0f); glVertex3f(-0.5,  -0.5,  -0.5f);	
  glTexCoord2f(0.0f, 1.0f); glVertex3f( 0.5f, -0.5f, -0.5f);	
  glTexCoord2f(0.0f, 0.0f); glVertex3f( 0.5f, -0.5f,  0.5f);	
  glTexCoord2f(1.0f, 0.0f); glVertex3f(-0.5f, -0.5f,  0.5f);	
  glEnd();	
  
  
  glDisable(GL_TEXTURE_2D);
  glPopMatrix();	
  
  
}

void Pedestrian::draw_WC_Head() {
  
  
  glPushMatrix();
  glEnable( GL_TEXTURE_2D );
  
  //----------------------- head -------------------------------
  
  // Front Face
  glBindTexture( GL_TEXTURE_2D, ped_wc_headface);
  glBegin(GL_QUADS);
  glNormal3f( 0.0f, 0.0f, -1.0f);			// Normal Pointing Towards Viewer
  glTexCoord2f(0.0f, 1.0f); glVertex3f( 1, 1, -1);	
  glTexCoord2f(1.0f, 1.0f); glVertex3f(-1, 1, -1);	
  glTexCoord2f(1.0f, 0.0f); glVertex3f(-1, 1,  1);	
  glTexCoord2f(0.0f, 0.0f); glVertex3f( 1, 1,  1);	
  glEnd();	
  
  
  //  Face side right
  glBindTexture( GL_TEXTURE_2D, ped_wc_headsideRight);
  glBegin(GL_QUADS);
  glNormal3f( 0.0f, 0.0f, 1.0f);			// Normal Pointing Towards Viewer
  glTexCoord2f(0.0f, 1.0f); glVertex3f(-1,  1, -1);	
  glTexCoord2f(1.0f, 1.0f); glVertex3f(-1, -1, -1);	
  glTexCoord2f(1.0f, 0.0f); glVertex3f(-1, -1,  1);	
  glTexCoord2f(0.0f, 0.0f); glVertex3f(-1,  1,  1);	
  glEnd();	
  
  
  //  Face side Left
  glBindTexture( GL_TEXTURE_2D, ped_wc_headsideRight);
  glBegin(GL_QUADS);
  glNormal3f( 0.0f, 0.0f, 1.0f);			// Normal Pointing Towards Viewer
  glTexCoord2f(1.0f, 1.0f); glVertex3f(1, -1, -1);	
  glTexCoord2f(0.0f, 1.0f); glVertex3f(1,  1, -1);	
  glTexCoord2f(0.0f, 0.0f); glVertex3f(1,  1,  1);	
  glTexCoord2f(1.0f, 0.0f); glVertex3f(1, -1,  1);	
  glEnd();	
  
  
  // head top
  glBindTexture( GL_TEXTURE_2D, ped_wc_headtop);
  glBegin(GL_QUADS);
  glNormal3f( 0.0f, 1.0f, 0.0f);			
  glTexCoord2f(0.0f, 1.0f); glVertex3f( 1, -1,  1);	
  glTexCoord2f(1.0f, 1.0f); glVertex3f( 1,  1,  1);	
  glTexCoord2f(1.0f, 0.0f); glVertex3f(-1,  1,  1);	
  glTexCoord2f(0.0f, 1.0f); glVertex3f(-1, -1,  1);	
  glEnd();
  
  
  // back head
  glBindTexture( GL_TEXTURE_2D, ped_wc_headback);
  glBegin(GL_QUADS);
  glNormal3f( 0.0f, 0.0f, 1.0f);			// Normal Pointing Towards Viewer
  glTexCoord2f(1.0f, 1.0f); glVertex3f(-1,  -1, -1);	
  glTexCoord2f(0.0f, 1.0f); glVertex3f( 1,  -1, -1);	
  glTexCoord2f(0.0f, 0.0f); glVertex3f( 1,  -1,  1);	
  glTexCoord2f(1.0f, 0.0f); glVertex3f(-1,  -1,  1);	
  
  
  glEnd();	
  
  
  glDisable(GL_TEXTURE_2D);
  glPopMatrix();	
  
}


void Pedestrian::drawArm(){
  
  glPushMatrix();
  glScalef(0.3,0.3,2);
  glEnable( GL_TEXTURE_2D );
  glBindTexture( GL_TEXTURE_2D, ped_wc_bodymisc);
  drawPieceOfClothing();
  glDisable(GL_TEXTURE_2D);
  glPopMatrix();
  
  
}


void Pedestrian::drawLeg(){
  
  glPushMatrix();
  glScalef(0.5,0.5,2);
  glEnable( GL_TEXTURE_2D );
  glBindTexture( GL_TEXTURE_2D, ped_wc_bodymisc);
  drawPieceOfClothing();
  glDisable(GL_TEXTURE_2D);
  glPopMatrix();
  
}


void Pedestrian::drawRightArm() {
  
  glPushMatrix();
  glTranslatef(-1.3,0,-1.5);
  glRotatef(rotateLeg,1,0,0);
  glTranslatef(0,0,-2);
  drawArm();
  glPopMatrix();
  
}

void Pedestrian::drawLeftArm() {
  
  glPushMatrix();
  glTranslatef(1.3,0,-1.5);
  glRotatef(-rotateLeg,1,0,0);
  glTranslatef(0,0,-2);
  drawArm();
  glPopMatrix();
  
}


void Pedestrian::drawLeftLeg() {
  
  glPushMatrix();
  glTranslatef(0.75,0,-5);
  glRotatef(rotateLeg,1,0,0);
  glTranslatef(0,0,-3);
  drawLeg();
  glPopMatrix();
  
}

void Pedestrian::drawRightLeg() {
  
  glPushMatrix();
  glTranslatef(-0.75,0,-5);
  glRotatef(-rotateLeg,1,0,0);
  glTranslatef(0,0,-3);
  drawLeg();
  glPopMatrix();
  
}



void Pedestrian::drawPed(float x, float z,float y) {
  
  if(rotateLeg % 30 == 0){rotateLeg*=-1;}
  
  
  glTranslatef(x,z,y);
  
  glPushMatrix();
  glRotatef(rotateLeg+180,0,0,1);
  draw_WC_Head();
  glPopMatrix();
  
  drawLeftLeg();
  drawLeftArm();
  
  glPushMatrix();
  glRotatef(180,0,0,1);	
  glScalef(2,2,5);
  glTranslatef(0,0,-0.75);
  draw_WC_Body();
  glPopMatrix();
  
  drawRightArm();
  drawRightLeg();
  
  
}


void Pedestrian::drawGeometry()
{
  // just draw a sphere or something
  //glutSolidSphere(1,10,10);
  

  if (isSoiled()){
    rotateLeg += 20;
    m_speed = 10;
  }
  else
    rotateLeg += 10;

  if ( (!(frameNumber++ % 30)) && isSoiled()){
    ChangeDirection();
  }

  glPushMatrix();
  glRotatef(rotateObj + 90,0,0,1);
  drawPed(0,0,10);
  glPopMatrix();
  
}

void Pedestrian::Move()
{
  // just use default movement
  MovableObject::Move();
}


#endif
