/* Poopmup - An urban flight sim
 *
 * Copyright (C) 2002 Zaroubi, Fink, Leclerc, Stanek szaroubi@hotmail.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* Poop'm'Up Objectives for the player to achieve 


 */

#ifndef POOPMUP_OBJECTIVES_H
#define POOPMUP_OBJECTIVES_H

#include "../common.h"

// flags for the different kinds of objectives we want
#define OBJECTIVE_PEDS  1
#define OBJECTIVE_CARS  2
#define OBJECTIVE_FRIES 4
#define OBJECTIVE_TIME  8

// All bits are one
#define ALL_OBJECTIVES  0xFF

#define MAX_LEVELS 40

class myObjectives 
{
 public:

  // Constructors
  myObjectives();

  // Destructors
  ~myObjectives() {}

  struct _levels
  {
    int fries;
    int peds;
    int cars;
    float timelimit;
  } levels[MAX_LEVELS];
  

  
  void setLevel(int level = 1);
  void setFriesToEat(int num) { m_friesToEat = num; }
  void setPedsToPoop(int num) { m_pedsToPoop = num; }
  void setCarsToPoop(int num) { m_carsToPoop = num; }
  void setTimeLeft(float time) 
    { 
      m_time = currTime + (time * CLOCKS_PER_SEC);  
    }
  int timeout(){return getTimeLeft()<=0;}

  int getFriesLeft(void) { return m_friesToEat; }
  int getPedsLeft(void)  { return m_pedsToPoop; }
  int getCarsLeft(void)  { return m_carsToPoop; }
  float getTimeLeft(void) 
    { 
      float t = (m_time - currTime) / float(CLOCKS_PER_SEC); 
      if(t < 0)
	return 0;
      else
	return t;
    }
  void getTimeLeftString(char* buf);


  bool IsObjectiveMet(int flags = ALL_OBJECTIVES);

  void InitializeLevels();
  void InitializeLevel(int level, int fries, int peds, int cars, float timel);

  /* I;m leaving place to pass in the ID for future when we may want
     to eat/poop a specific item*/
  void AteFry(int id = 0) { if(m_friesToEat) m_friesToEat--; }
  void PoopedPed(int id = 0) {if(m_pedsToPoop) m_pedsToPoop--; }
  void PoopedCar(int id = 0) { if(m_carsToPoop) m_carsToPoop--; }
  char * toString();

 private:
    int m_friesToEat;
    int m_pedsToPoop;
    int m_carsToPoop;
    float m_time;
};


myObjectives::myObjectives()
{
  m_friesToEat = -1;
  m_pedsToPoop = -1;
  m_carsToPoop = -1;
  m_time = -1; // time in seconds
  InitializeLevels();
}

bool myObjectives::IsObjectiveMet(int flags /* = ALL_OBJECTIVES */)
{
  bool rc = true;
  // check the requested objective counters
  if(flags & OBJECTIVE_PEDS)
    {
      if(getPedsLeft() > 0)
	rc = false;
    }

  if(flags & OBJECTIVE_CARS)
    {
      if(getCarsLeft() > 0)
	rc = false;
    }

  if(flags & OBJECTIVE_FRIES)
    {
      if(getFriesLeft() > 0)
	rc = false;
    }

  

  
  return rc;

}

void myObjectives::getTimeLeftString(char *buf)
{
  float s = getTimeLeft(); // s is seconds left in the game
  float mins, secs, milli;

  mins = floor(s/60);
  s -= (mins * 60);
  secs = floor(s);
  s -= secs;
  milli = floor(s * 100);

  sprintf(buf, "%02.0f:%02.0f.%02.0f", mins, secs, milli);
		  
}

void myObjectives::setLevel(int level /* = 1 */)
{
  if(level > 0 && level < MAX_LEVELS)
    {
      setFriesToEat(levels[level].fries);
      setPedsToPoop(levels[level].peds);
      setCarsToPoop(levels[level].cars);
      setTimeLeft(levels[level].timelimit);

    }
}

void myObjectives::InitializeLevels()
{
  // set up the demansd for each of the MAX_LEVELS levels
  for(int i = 0; i < MAX_LEVELS; ++i)
    {
      InitializeLevel(i, i, i, i, 180);
    }

  // CUSTOM LEVELS
  // level, fries, peds, cars, time);
  InitializeLevel(1, 0, 1, 0, 30 * 60);
  InitializeLevel(2, 1, 1, 1, 10 * 60);
  InitializeLevel(3, 3, 5, 10, 5 * 60);

}

void myObjectives::InitializeLevel(int level, int fries, int peds, int cars, float timel)
{
      levels[level].fries = fries;
      levels[level].peds = peds;
      levels[level].cars = cars;
      levels[level].timelimit = timel; 
}

#endif

