/* Poopmup - An urban flight sim
 *
 * Copyright (C) 2002 Zaroubi, Fink, Leclerc, Stanek szaroubi@hotmail.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef BUTTON_H
#define BUTTON_H

#include <stdio.h>
#include "../../base/myColor.h"
#include "../../../common.h"
#include "formElem.h"


class Button: public formElem{
 private:
  
  void (*pActionFunc)(void);
  
  void drawOutBackground();
  void drawOverBackground();

  
 public:
  void runAction(float mouseX, float mouseY) {if(pActionFunc) pActionFunc(); }
  void setActionFunc(void (*pFunc)(void)){pActionFunc = pFunc;}
  void drawGeometryOut();
  void drawGeometryOver();

  Button();
  ~Button(){}
  
    
};

Button::Button(){
  bgColor.r = bgColor.g = bgColor.b = 1;
  bgColor.a = 0.5;
  setOverCursorType(GLUT_CURSOR_CROSSHAIR);
  pActionFunc = NULL;
  //setDescription(label);
}


void Button::drawOutBackground(){	   
  float x = getX();
  float y = getY();
  float w = getWidth();
  float h = getHeight();

  glPushMatrix();
  
  //glTranslatef(0,0,-0.1); 
  glColor4f(bgColor.r,bgColor.g,bgColor.b,bgColor.a);
  
  glBegin( GL_POLYGON );
  glVertex2f( x,	y);
  glVertex2f( x,	y+h);
  glVertex2f( x+w,	y+h);
  glVertex2f( x+w,	y);  
  glEnd();
  
  glTranslatef(0,0,0.1); 
  glColor4f(borderColor.r,borderColor.g,borderColor.b,borderColor.a);
  glBegin( GL_LINE_LOOP );
  glVertex3f( x,	y,		0);
  glVertex3f( x+w,	y,		0);     
  glVertex3f( x+w,	y+h,	0);
  glVertex3f( x,	y+h,	0);
  glEnd();
  
  
  glPopMatrix();
  
  
}


void Button::drawOverBackground(){	   
  float x = getX();
  float y = getY();
  float w = getWidth();
  float h = getHeight();

  glPushMatrix();
  
  //glTranslatef(0,0,-0.1); 
  glColor4f(overBgColor.r,overBgColor.g,overBgColor.b,overBgColor.a);
  
  glBegin( GL_POLYGON );
  glVertex2f( x,	y);
  glVertex2f( x,	y+h);
  glVertex2f( x+w,	y+h);
  glVertex2f( x+w,	y);  
  glEnd();
  
  glTranslatef(0,0,0.1); 
  glColor4f(overBorderColor.r,overBorderColor.g,overBorderColor.b,overBorderColor.a);
  glBegin( GL_LINE_LOOP );
  glVertex2f( x,	y);
  glVertex2f( x+w,	y);     
  glVertex2f( x+w,	y+h);
  glVertex2f( x,	y+h);
  glEnd();
  
  
  glPopMatrix();
  
  
}

void Button::drawGeometryOut(){
  float x = getX();
  float y = getY();
  float padding = getPadding();

  drawOutBackground();
  glColor4f(fgColor.r,fgColor.g,fgColor.b,fgColor.a);
  printString(x+padding,y+padding+BUTTON_FONT_HEIGHT,getLabel());
}
void Button::drawGeometryOver(){
  float x = getX();
  float y = getY();
  float padding = getPadding();

  drawOverBackground();
  glColor4f(overFgColor.r,overFgColor.g,overFgColor.b,overFgColor.a);
  printString(x+padding,y+padding+BUTTON_FONT_HEIGHT,getLabel());
}
#endif

