/* Poopmup - An urban flight sim
 *
 * Copyright (C) 2002 Zaroubi, Fink, Leclerc, Stanek szaroubi@hotmail.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef CHECKBOX_H
#define CHECKBOX_H


#include "formElem.h"

class checkBox: public formElem{

 private:
  /* A Pointer to the value that will be modified */
  int * checked;
  char title[BUTTON_DESC_MAX_SIZE];
  int checkedVal, uncheckedVal;

 public:
  void drawGeometryOver();
  void drawGeometryOut();
  void runAction(float mouseX, float mouseY);
  
  void setTitle(char *t){strncpy(title, t, BUTTON_DESC_MAX_SIZE);}

  /* State management */
  void setChecked(int *t){ checked = t;}
  int isChecked(){return (*checked == checkedVal);}
  void check(){*checked = checkedVal;}
  void unCheck(){*checked = uncheckedVal;}
  void toggleCheck(){if(isChecked())unCheck();else check();}
  
  void setCheckedVal(int t){checkedVal = t;}
  void setUncheckedVal(int t){uncheckedVal = t;}

  checkBox();
  ~checkBox(){}
};

void checkBox::runAction(float mouseX, float mouseY){
  toggleCheck();
}

void checkBox::drawGeometryOver(){
  float x = getX();
  float y = getY();
  float padding = getPadding();
   char str[ BUTTON_DESC_MAX_SIZE]="";

  fgColor.r = fgColor.g = fgColor.b = 1;
  fgColor.a = 1;

  if (isChecked()) sprintf(str, "%s | X |",title);
  else  sprintf(str, "%s |   |",title);


  setLabel(str);
  glColor4f(fgColor.r,fgColor.g,fgColor.b,fgColor.a);
  printString(x+padding,y+padding+BUTTON_FONT_HEIGHT,getLabel());
}

void checkBox::drawGeometryOut(){
  float x = getX();
  float y = getY();
  float padding = getPadding();
  char str[ BUTTON_DESC_MAX_SIZE]="";

  fgColor.r = fgColor.g = fgColor.b = 0.75;
  fgColor.a = 1;
  
  if (isChecked()) sprintf(str, "%s | X |",title);
  else  sprintf(str, "%s |   |",title);
  

  setLabel(str);
  glColor4f(fgColor.r,fgColor.g,fgColor.b,fgColor.a);
  printString(x+padding,y+padding+BUTTON_FONT_HEIGHT,getLabel());

}


checkBox::checkBox(){
  bgColor.r = bgColor.g = bgColor.b = 1;
  bgColor.a = 0.5;

  fgColor.r = fgColor.g = fgColor.b = 0.5;
  fgColor.a = 0.5;

  checked = 0;

  checkedVal = 1;
  uncheckedVal = 0;

}

#endif



