/* Poopmup - An urban flight sim
 *
 * Copyright (C) 2002 Zaroubi, Fink, Leclerc, Stanek szaroubi@hotmail.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */



#ifndef FORMELEM_H
#define FORMELEM_H

#include <stdio.h>
#include "../../../common.h"

#define BUTTON_OVER 1
#define BUTTON_OUT 2

//Will not actually affect the size of the text.
//Used to get width of button.
#define BUTTON_FONT_WIDTH 8
#define BUTTON_FONT_HEIGHT 13

#define BUTTON_DESC_MAX_SIZE 80


class formElem{

 private:
  float x,y,width, height, id;
  float padding;
  int overCursorType;

  char label[LABEL_MAX_SIZE];
  char description[BUTTON_DESC_MAX_SIZE]; 
  int state;

 public:
  /**** Seters ****/
  void setX(float t){ x = t;}
  void setY(float t){ y = t;}
  void setWidth(float t){ width = t;}
  void setHeight(float t){ height = t;}
  void setId(float t){ id = t;}
  void setPadding(float t){padding = t;}
  void setOverCursorType(int t){ overCursorType = t;}

  /*** Getters ****/
  float getX(){return x;}
  float getY(){return y;}
  float getWidth(){return width;}
  float getHeight(){return height;}
  float getId(){return id;}
  float getPadding(){return padding;}
  int getOverCursorType(){return overCursorType;}

  /* State management */
  void over(){ state = BUTTON_OVER;}
  void out(){ state = BUTTON_OUT;}
  void setState(int t){state = t;}
  virtual void runAction(float mouseX, float mouseY) = 0;

  /* Label and description */
  void setLabel(const char * t);
  char *getLabel(){return label;}
  void setDescription(char*t){strncpy(description, t, BUTTON_DESC_MAX_SIZE);}
  char * getDescription(){return description;}
  
  /**** Drawing ****/
  void draw();
  virtual void drawGeometryOver(void) = 0;
  virtual void drawGeometryOut(void) = 0;

  /** Colours **/
  myColor bgColor, fgColor, borderColor;
  myColor overBgColor, overFgColor, overBorderColor;

  void printString(float x, float y, char *str);
 
  formElem();
  virtual ~formElem(){};
};

void formElem::setLabel(const char * t){
  strncpy(label, t, LABEL_MAX_SIZE);
  setWidth(BUTTON_FONT_WIDTH* strlen(label) + 2*padding);
  setHeight(BUTTON_FONT_HEIGHT * 1 + 4*padding);
}

void formElem::draw(){
  glDisable(GL_LIGHTING);
  glEnable (GL_BLEND);
  
  glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  switch (state){
  case BUTTON_OVER:	
    drawGeometryOver();
    break;
  default:
  case BUTTON_OUT:
    drawGeometryOut();
    break;
  }
  glEnable(GL_LIGHTING);
  glDisable (GL_BLEND);
}

void formElem::printString(float x,float y, char *str)
{
  glPushMatrix();
  glTranslatef(0,0,0.1);
  //	glColor3f(1,1,1);
  char *c;
  glRasterPos2f(x,y);
  for (c=str; *c != '\0'; c++) 
    {	
      glutBitmapCharacter(GLUT_BITMAP_8_BY_13, *c);
    }
  glPopMatrix();
}

formElem::formElem(){
  state = BUTTON_OUT;
  setX(0);
  setY(0);
  //These values should be the the size of the text :-)
  setWidth(120); 
  setHeight(20);
  setPadding(2); //Padding between the text and border ;)
  setDescription("");
  setLabel("");
}


#endif
