/* Poopmup - An urban flight sim
 *
 * Copyright (C) 2002 Zaroubi, Fink, Leclerc, Stanek szaroubi@hotmail.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef POOPMUP_MENU_H
#define POOPMUP_MENU_H

#include <stdio.h>
#include "../base/myColor.h"
#include "formElem/button.h"
#include "formElem/intSelector.h"
#include "formElem/checkBox.h"
#include "formElem/formElem.h"
#include "../base/my3dPoint.h"
#include "../../common.h"

/** :: TODO ::
 * -The button list should NOT be a fixed sized array.
 */

#define MENU_MAX_ELEMENT_NUM 10
#define MENU_DESC_MAX_SIZE 80

#define TYPE_BUTTON 1
#define TYPE_INTSELECTOR 2
#define TYPE_CHECKBOX 3

struct element 
{
  int elementId; 
  int type;
  formElem *pElem;
};

class Menu{
 private:
  int enabled;
  char label[LABEL_MAX_SIZE];
  float x, y;
  float w,h;
  myColor bgColor;
 
  int defaultCursorType;

  Button button[MENU_MAX_ELEMENT_NUM];
  intSelector iSelector[MENU_MAX_ELEMENT_NUM];
  checkBox cBox[MENU_MAX_ELEMENT_NUM];
  element elemArray[MENU_MAX_ELEMENT_NUM *3];

 public:
  int selectedElement;
  int numButtons, numIntSelectors,numCheckBoxes, numElem;

  float padding;
  my3dPoint cursorPos;
  int mouseDown, mouseUp;
  int mouseButton;
  char menuDesc[MENU_DESC_MAX_SIZE], description[MENU_DESC_MAX_SIZE];

  Menu(); 
  ~Menu() {};
  void update();  

  /******* THE VISUALS ***********/
  void setBgColor(float r, float g, float b, float a = 1);
  void setWidth(float t){w = t;update();}
  void setHeight(float t){h = t;update();}
  void setX(int t){x = t;update();}
  void setY(int t){y = t;update();}
  void setLabel(const char * t){strncpy(label, t, LABEL_MAX_SIZE);}
  void setDescription(char *t){strncpy(description, t, MENU_DESC_MAX_SIZE);}
  void setMenuDescription(char *t){strncpy(menuDesc, t, MENU_DESC_MAX_SIZE);}

  /******* THE DRAWING*********/
  void printString(float x, float y, char *str);
  void drawBackground();
  void draw();
  void animate();

  /**** STATUS******/
  void enable(){enabled=1;};
  void disable(){enabled=0;};
  int active(){return enabled;}
  void setState(int t){enabled = t;}

  /**** Element MANAGEMENT *****/
  element newElement(int t);
  element getElementUnderMouse(); 
  Button* CreateButton(char *t_label, void (*pFunc)(void));
  intSelector *CreateIntSelector(char *t_label, int * tVal);
  checkBox *CreateCheckBox(char* t_label, int * tVal);

  /**** THE HOOKS ****/
  //Hooking up elements with functions
  void elementPressed(element e);
  void elementOver(element e);
};


element Menu::newElement(int t){
  element returnVal;
  
  if (t == TYPE_BUTTON){
    returnVal.elementId = numButtons;
    if (numButtons <  MENU_MAX_ELEMENT_NUM){
      numButtons++;
    }
    else{
      returnVal.elementId = -1;
    }
  }
  if (t == TYPE_INTSELECTOR){
    returnVal.elementId = numIntSelectors;
    if (numIntSelectors <  MENU_MAX_ELEMENT_NUM){
      numIntSelectors++;
    }
    else{
      returnVal.elementId = -1;
    }
  }
  if (t == TYPE_CHECKBOX){
    returnVal.elementId = numCheckBoxes;
    if (numCheckBoxes <  MENU_MAX_ELEMENT_NUM){
      numCheckBoxes++;
    }
    else{
      returnVal.elementId = -1;
    }
  }
  
  if (numElem <  MENU_MAX_ELEMENT_NUM){
    returnVal.elementId = numElem;
    returnVal.type = t;
    numElem++;
    return returnVal;
  }

  returnVal.elementId = -1;
  returnVal.type = -1;
  
  return returnVal; //Too many elements already.
  
}
void Menu::setBgColor(float r, float g, float b, float a){
  bgColor.setRGBA(r,g,b,a);
  update();
}

Menu::Menu() 
{
  strcpy(label," :");
  enabled = 0;
  x=5;
  y=200;
  w=800;
  h=400;
  setBgColor(1,1,1,0.3);
  padding = 10;
  
  defaultCursorType = GLUT_CURSOR_FULL_CROSSHAIR;

  //Because it will be incremented to 0 in newElement()
  numElem = numButtons = numIntSelectors= numCheckBoxes= 0; 
  
  setMenuDescription("");
};

element Menu::getElementUnderMouse(){
  element returnVal;
  returnVal.elementId = -1;
  returnVal.type = -1;
 
  for (int i = 0; i<numElem; i++){
    float x = elemArray[i].pElem->getX();
    float y = elemArray[i].pElem->getY();
    float w = elemArray[i].pElem->getWidth();
    float h = elemArray[i].pElem->getHeight();
    if (
	(cursorPos.getX() > x )
	&&
	(cursorPos.getX() < x + w)
	&&
	(cursorPos.getY() > y)
	&&
	(cursorPos.getY() < y+h)
	){
      return elemArray[i];
    }
  }


  return returnVal;
}

void Menu::drawBackground(){	   
  
  glPushMatrix();
  glEnable (GL_BLEND);
  glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  
  //glTranslatef(0,0,-0.1); 
  glColor4f(bgColor.r,bgColor.g,bgColor.b,bgColor.a);
  
  glBegin( GL_POLYGON );
  glVertex2f( x,	y);
  glVertex2f( x,	y+h);
  glVertex2f( x+w,	y+h);
  glVertex2f( x+w,	y);  
  glEnd();
  
  glTranslatef(0,0,0.1); 
  glColor4f(0.3,0.3,0.3,1);
  glBegin( GL_LINE_LOOP );
  glVertex3f( x,	y,		0);
  glVertex3f( x+w,	y,		0);     
  glVertex3f( x+w,	y+h,	0);
  glVertex3f( x,	y+h,	0);
  glEnd();
  
  glDisable (GL_BLEND);
  glPopMatrix();
  
  
}

void Menu::update(){
  h = 0;

  for (int i = 0; i < numElem; i++){
    elemArray[i].pElem->setX(x + padding);
    elemArray[i].pElem->setY(y + padding + h);
    h += elemArray[i].pElem->getHeight() + padding;
  }

  h += padding;
}

void Menu::animate(){
  int i;
  //Reset the menu description
  setDescription("");
  
  for (i = 0; i < numElem; i++){
    elemArray[i].pElem->setState(BUTTON_OUT);
  }
  
  //reset the cursor
  glutSetCursor(defaultCursorType);
  
  //Get The element under the mouse
  element selectedElement=getElementUnderMouse();
  
  if (selectedElement.elementId >=0){
    //If infact there is a element under the mouse
    //Fire the elementOver trigger
    elementOver(selectedElement);
  }
  if ((selectedElement.elementId >=0 ) &&(mouseUp) ){
    //If there is a element under the mouse and mouseButton jusr released
    //the fire the elementPressed trigger.
    elementPressed(selectedElement);
  }
}

void Menu::draw()
{
  if(enabled)
    {
      int i;
      if ( POOP_LIGHTING ) glDisable(GL_LIGHTING);
      
      drawBackground();
      
      //Draw elements in Menu
      glPushMatrix();
      glTranslatef(0,0,0.1); 
      
      for (i = 0; i< numElem; i++){
	elemArray[i].pElem->draw();
      }

      printString(x+w/2,y+h/2,label);
      printString(x+w/2,y+h/2+15,menuDesc);
      printString(x+w/2,y+h/2+30,description);
      glPopMatrix();
      
      if ( POOP_LIGHTING ) glEnable(GL_LIGHTING);
      
    }
}

void Menu::printString(float x,float y, char *str)
{
  if ( POOP_LIGHTING ) glDisable(GL_LIGHTING);
  glColor3f(1,1,1);
  char *c;
  glRasterPos2f(x,y);
  for (c=str; *c != '\0'; c++) 
    {	
      glutBitmapCharacter(GLUT_BITMAP_8_BY_13, *c);
    }
  if ( POOP_LIGHTING )  glEnable(GL_LIGHTING);
}

void Menu::elementPressed(element e){
  e.pElem->runAction(cursorPos.getX(), cursorPos.getY());
}

void Menu::elementOver(element e){
  e.pElem->setState(BUTTON_OVER);
  glutSetCursor( e.pElem->getOverCursorType());
  setDescription( e.pElem-> getDescription());
}

Button *Menu::CreateButton(char *t_label, void (*pFunc)(void)){
  
  element e = newElement(TYPE_BUTTON);
  int id = e.elementId;

  if(id >= 0){
    button[id].setLabel(t_label);
    button[id].setActionFunc(pFunc);
    button[id].setId(id);
    button[id].setOverCursorType(GLUT_CURSOR_CROSSHAIR);
    button[id].overFgColor = myColor(1,0,0,1) ;
    button[id].bgColor = myColor(0.2, 0.2, 0.2 , 0.3);
    elemArray[id].pElem = &button[id];
    return  &button[id];
  }
  else{
    return NULL;
  }
}

intSelector *Menu::CreateIntSelector(char *t_label, int * tVal){
  element e = newElement(TYPE_INTSELECTOR);
  int id = e.elementId;

  if(id >= 0){
    iSelector[id].setLabel(t_label);
    iSelector[id].setTitle(t_label);
    iSelector[id].setId(id);
    iSelector[id].setInt(tVal);
    elemArray[id].pElem = &iSelector[id]; 
    return &iSelector[id];
  }
  else{
    return NULL;
  }
}

checkBox *Menu::CreateCheckBox(char* t_label, int * tVal){
  element e = newElement(TYPE_CHECKBOX);
  int id = e.elementId;
  if(id >= 0){
    cBox[id].setTitle(t_label);
    cBox[id].setId(id);
    cBox[id].setChecked(tVal);
    elemArray[id].pElem = &cBox[id]; 
    return &cBox[id];
  }
  else{
    return NULL;
  }
}
#endif

