/* Poopmup - An urban flight sim
 *
 * Copyright (C) 2002 Zaroubi, Fink, Leclerc, Stanek szaroubi@hotmail.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef ONSCREENDISPLAY_H
#define ONSCREENDISPLAY_H

#include <stdio.h>
#include "../base/my3dPoint.h"
#include "../base/myColor.h"


#define TYPE_PIE 1
#define TYPE_TEXT 2
#define TYPE_HBAR 3
#define TYPE_VBAR 4
#define TYPE_CURSOR 5
#define TYPE_MASK 6

#define LABEL_MAX_SIZE 100

/**
 * For a list of the type read the #define statments above
 *
 * For VBAR and HBAR:
 *    - Color1 = color at 0
 *    - Color2 = color away from 0.
 * 
 * All text is written in Label Color
 * For CURSOR:
 *    - Color1 = Color of "Triangulary area" around the mousePos
 *    - Color2 = Color of Points and Line connecting center of screen+mousePos

 * For Mask:
 * x,y coords of upper Right corner
 */



class onScreenDisplay{
 private:

  int type,enabled;
  float  maxVal, minVal, curVal;
  my3dPoint cursorPos, screenCenter;

  myColor color1, color2, labelColor;

  char label[LABEL_MAX_SIZE], atZero[LABEL_MAX_SIZE], suffix[LABEL_MAX_SIZE];
  void drawPIE();
  void drawHBAR();
  void drawVBAR();
  void drawTEXT();
  void drawCURSOR();
  void drawMASK();

  float x, y;	
  int flasher, flasherOn;
  int noValue;

 public:
  onScreenDisplay(); 
  ~onScreenDisplay() {};
  
  void setX(int t){x = t;}
  void setY(int t){y = t;}
  
  void setType (int t){ type = t;}
  int getType (){return type;}
  
  void setLabel(const char * t){strncpy(label, t, LABEL_MAX_SIZE);}
  void setAtZero(const char * t){strncpy(atZero, t, LABEL_MAX_SIZE);}
  void setSuffix(const char * t){strncpy(suffix, t, LABEL_MAX_SIZE);}

  void setCursorPos(my3dPoint mousePos);
  void setScreenCenter(float x, float y);
  void setCurVal(float t) {curVal = t;}
  void setMaxVal(float t) {maxVal = t;} 
  void setMinVal(float t) {minVal = t;} 
  void printString(float x, float y, char *str);
  void printWarning();

  void setNoValue(int v) {noValue = v;}
  
  void setColor1(float tr, float tg, float tb, float ta=0.5)
    {color1.r=tr;color1.g=tg;color1.b=tb;color1.a=ta;}
  void setColor2(float tr, float tg, float tb, float ta=0.5)
    {color2.r=tr;color2.g=tg;color2.b=tb;color2.a=ta;}
  void setLabelColor(float tr, float tg, float tb, float ta=0.5)
    {labelColor.r=tr;labelColor.g=tg;labelColor.b=tb;labelColor.a=ta;}
  
  void setFlasherOn(){flasherOn = 1;}
  void setFlasherOff(){flasherOn = 0;}
  void toggleFlasher(){flasherOn = 1-flasherOn;}

  void enable(){enabled = 1;}
  void disable(){enabled = 0;}

  void draw();
  GLuint mask,cross;
};

void onScreenDisplay::setScreenCenter(float x, float y){
  screenCenter.setXYZ(x,y,0);
}

void onScreenDisplay::setCursorPos(my3dPoint t){
  cursorPos = t;
}

onScreenDisplay::onScreenDisplay() 
{
  x = -2000;
  y = 0;
  maxVal = 0;
  minVal = 0;
  curVal = 0;
  setLabel("");
  setAtZero("");
  setSuffix(""); 
  enabled = 1;
  flasher = 0; flasherOn = 0;
  noValue = false;
};

void onScreenDisplay::draw()
{
  if ( POOP_LIGHTING ) glDisable(GL_LIGHTING);
  //if(FAST_COMPUTER){
  glEnable (GL_BLEND);
  glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  //}
  //glColor3f(color1.r,color1.g,color1.b);

  if (enabled)
    {
      if(flasherOn)flasher++;
      if (flasher%5<3){
	printWarning();
	switch (type)
	  {
	  case TYPE_PIE:
	    drawPIE();
	    break;
	  case TYPE_TEXT:
	    drawTEXT();
	    break;
	  case TYPE_HBAR:
	    drawHBAR();
	    break;
	  case TYPE_VBAR:
	    drawVBAR();
	    break;
	  case TYPE_CURSOR:
	    drawCURSOR();
	  break;
	  case TYPE_MASK:
	    drawMASK();
	    break;
	  default:
	    break;
	}
      }
    }

  glColor4f(1,1,1,1);
  //if (FAST_COMPUTER)
    glDisable(GL_BLEND);

  if ( POOP_LIGHTING ) glEnable(GL_LIGHTING);
}


void onScreenDisplay::drawPIE()
{
  //NEED TO IMPLEMENT
};

void onScreenDisplay::printWarning(){
  //if (curVal <= 0 && flasher%10==0){	
    glColor3f( 1, 0, 0 );
    printString(-2,y-10, atZero);
    //}	
    //flasher++;
}

void onScreenDisplay::drawVBAR()
{
  
  //printWarning();
  printString(x,y, label);

  if (curVal >=  minVal){

    float lineWidth = 4;
    int barWidth = 20;
    float spacing = 0;
    float yPos = y - 5;

    myColor blockColor;

    float curValPercent = curVal / maxVal * 100;
    for ( float i = 0; i < curValPercent; i++){
      
      glBegin( GL_POLYGON );
      yPos -= (lineWidth + spacing);
          
      if (maxVal != 0){
	blockColor = color1.interpolate(color2, i/maxVal*100);
      }
      else
	blockColor = color1;

      glColor4f(blockColor.r , blockColor.g, blockColor.b, blockColor.a);

      //glColor3f(1-i/maxVal , i/maxVal, 0);

      glVertex3f( x + 15, yPos, 0 );
      glVertex3f( x + 15 + barWidth , yPos, 0 );
      
      glVertex3f( x + 15 + barWidth , yPos - lineWidth, 0 );
      glVertex3f( x + 15, yPos - lineWidth, 0 );
      glEnd();
    }
  }
  
}

void onScreenDisplay::drawCURSOR()
{
  float x0,x,y0,y;
  //Screen Center X,Y
  x = screenCenter.getX();
  y = screenCenter.getY();
  
  //Cursor Pos Xo,Yo
  x0 = cursorPos.getX();
  y0 = cursorPos.getY();


  glColor3f(color2.r, color2.g, color2.b);
  glBegin(GL_LINES)   ;
  glVertex2f(x0,y0);
  glVertex2f(x,y);
  glEnd();
  
  glPointSize(5);
  glBegin(GL_POINTS);	
  glVertex2f(x, y);
  glVertex2f(x0, y0);
  glEnd();
  glPointSize(1);
  
  //Distance of mouse from center of screen;
  float d = (x0-x)*(x0-x);
  d += (y0-y)*(y0-y);
  d = sqrt(d);


  float dx = (x0-x);
  float dy = (y0-y);
  float angleAdjust = 0;
  if ( (dx < 0 ) && (dy < 0 ) ){
    dx = -dx;
    angleAdjust = PI;
  }
  else if( (dx >= 0 ) && (dy <= 0 ) ){
    dx = -dx;
    angleAdjust = -PI;
  }

  if(d == 0) d = 1;
  float angleOfMouse = asin(dx/d) + angleAdjust;
	
  while(angleOfMouse <0) angleOfMouse += 2*PI;

  float startAngle = angleOfMouse - PI/8;
  float endAngle =  angleOfMouse  + PI/8;
  
  if (startAngle > endAngle){
    float t = endAngle;
    endAngle = startAngle;
    startAngle = t;
  }
  
  float startRadius =  d;

  float vectorX = x + (startRadius*(float)sin((double)startAngle));
  float vectorY = y + (startRadius*(float)cos((double)startAngle));


  glPushMatrix();
  glTranslatef(0,0,-0.1);    
  glColor4f(color1.r, color1.g, color1.b, color1.a);
  /*********** DRAW THE RADAR OUTLIE **************/
  for(float angle=startAngle;angle<=endAngle;angle+=0.05)
    { 
      glBegin(GL_TRIANGLES ); 
      glVertex2d(vectorX,vectorY);
      vectorX = x + (startRadius*(float)sin((double)angle));
      vectorY = y + (startRadius*(float)cos((double)angle));		
      glVertex2d(vectorX,vectorY);
      glVertex2d(x,y);
      glEnd();
    }
  
  glColor3f(0,1,0);
  glPopMatrix();

}

void onScreenDisplay::drawHBAR()
{
  
  //Print the label
  printString(x,y, label);

  float lineWidth = 4;
  int barWidth = 20;
  float spacing = 0;
  float xPos = x ;

  myColor blockColor;
  
  if (curVal >= minVal){
    //If the curent value is not under minVal
    float curValPercent = curVal / maxVal * 100.0;
    for ( float i = 0; i < curValPercent; i++){
      glBegin( GL_POLYGON );
      xPos += (lineWidth + spacing);
      if (maxVal != 0)
	blockColor = color1.interpolate(color2, i/maxVal*100);
      
      glColor4f(blockColor.r , blockColor.g, blockColor.b, blockColor.a);

      glVertex3f(xPos, y - 10, 0 );
      glVertex3f( xPos , y - 10 - barWidth, 0 );
      
      glVertex3f( xPos - lineWidth , y - 10 - barWidth, 0 );
      glVertex3f( xPos - lineWidth, y - 10, 0 );
      glEnd();
    }
  }
  
  
}

void onScreenDisplay::printString(float x, float y, char *str){
  char *c;  
  
  glColor4f(labelColor.r,labelColor.g,labelColor.b,labelColor.a);
  glRasterPos2f(x,y);
  
  for (c=str; *c != '\0'; c++) 
    {	
      glutBitmapCharacter(GLUT_BITMAP_8_BY_13, *c);
    }
}
void onScreenDisplay::drawTEXT()
{
  
  //glColor3f( 1, 1, 1 );
  
  char value[50];
  float incX = x;
  
  printString(x, y, label);

  if(noValue == false)
    {
      incX = x + strlen(label) * 8+ 3 ;
      if (curVal >= minVal){
	if (maxVal == 0)
	  sprintf(value, "%.0f %s", curVal, suffix);
	else
	  sprintf(value, "%.0f / %.0f", curVal, maxVal);
	
	printString(incX, y, value);
      }
    }
}


void onScreenDisplay::drawMASK() {
  
  /*
    if ( POOP_LIGHTING ) glDisable(GL_LIGHTING);
    ///////////////////// MASKING ///////////////////////////////
    // Once the room is drawn, we need to start doing the masking.  In order for
    // this to work, we NEED TO TURN OFF DEPTH TESTING.  Otherwise it won't work.
    // Before we render the real image, we will render the mask image.
    // The blending method we will use is (GL_DST_COLOR, GL_ZERO).
    
    // Disable depth testing (MUST do this to make it work)
    glDisable(GL_DEPTH_TEST);
    
    // Select our desired depth testing and turn on blending
    glEnable(GL_BLEND);											
    glBlendFunc(GL_DST_COLOR,GL_ZERO);
    
    
    // Bind the mask texture to our new 2D quad
    
    glEnable( GL_TEXTURE_2D );	
    glBindTexture(GL_TEXTURE_2D,  mask);
    // Display a 2D quad with the scope/cross hair mask
    glBegin(GL_QUADS);
    
    // Notice that when we are in ortho mode, we use glVertex2f() to pass
    // in screen coordinates, not vertices.  This makes it incredibly easy
    // to put up 2D interface art.  It's just like doing 2D graphics.
    // The texture coordinate stay the same regardless.
    
    // Display the top left point of the 2D image
    glTexCoord2f(0.0f, 1.0f);	glVertex2f(0, 0);
    
    // Display the bottom left point of the 2D image
    glTexCoord2f(0.0f, 0.0f);	glVertex2f(0, 500);
    
    // Display the bottom right point of the 2D image
    glTexCoord2f(1.0f, 0.0f);	glVertex2f(500, 500);
    
    // Display the top right point of the 2D image
    glTexCoord2f(1.0f, 1.0f);	glVertex2f(500, 0);
    
    // Stop drawing 
    glEnd();	
    
    // Once the mask is draw to the screen, we want to set a new blend mode.
    // This mode will perform the masking effect that we desire.  Then we just
    // render the desired transparent image over top of the mask and it will
    // only draw the desired effect.
    
    // Turn our blending mode to one to one
    
    glBlendFunc(GL_ONE,GL_ONE);
    
    
    // Bind the scope texture to this next 2D quad
    glBindTexture(GL_TEXTURE_2D,  cross);
    
    // Display a 2D quad with our scope texture
    glBegin(GL_QUADS);
    
    // Display the top left point of the 2D image
    glTexCoord2f(0.0f, 1.0f);	glVertex2f(0, -y);
    
    // Display the bottom left point of the 2D image
    glTexCoord2f(0.0f, 0.0f);	glVertex2f(0, y);
    
    // Display the bottom right point of the 2D image
    glTexCoord2f(1.0f, 0.0f);	glVertex2f(x,y);
    
    // Display the top right point of the 2D image
    glTexCoord2f(1.0f, 1.0f);	glVertex2f(x, -y);
    
    // Stop drawing 
    glEnd();
    
    // Since we are done, we can now turn OFF blending and turn ON depth testing
    glDisable(GL_BLEND);
    glEnable(GL_DEPTH_TEST);
    if ( POOP_LIGHTING ) glEnable(GL_LIGHTING);
  */
  
  // Display a 2D quad with our scope texture
  glBegin(GL_QUADS);
  glColor4f(color1.r, color1.g, color1.b, color1.a);
  // Display the top left point of the 2D image
  glVertex2f(0, -y);
  
  // Display the bottom left point of the 2D image
  glVertex2f(0, y);
  
  // Display the bottom right point of the 2D image
  glVertex2f(x,y);
  
  // Display the top right point of the 2D image
  glVertex2f(x, -y);
  
  // Stop drawing    
  glEnd();
}

#endif

