/* Poopmup - An urban flight sim
 *
 * Copyright (C) 2002 Zaroubi, Fink, Leclerc, Stanek szaroubi@hotmail.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef RADAR_H
#define RADAR_H

//#include <String.h>
#include <stdio.h>
#include "../derivedGeom/myFries.h"

/**
 * IMPORTANT:
 *        - I assume that the Y(positive) vector is the North
 *        - And that the X(positive) vector is East
 */

class Radar{
private:
  int enabled;
  
  char label[LABEL_MAX_SIZE];
  float x, y;	
  my3dPoint radarEye;
  myVector radarDirection;
  float radarRadius;
  float viewFactor; //Zoom 
  float radarWidth, radarDepth;
  MOVING_OBJ_LIST radarPedList, radarCarList, radarFryList;
  MAP_OBJ_LIST radarMapObjList;

 public:
  Radar(); 
  ~Radar() {};
  
  void setX(int t){x = t;}
  void setY(int t){y = t;}
  void setLabel(const char * t){strncpy(label, t, LABEL_MAX_SIZE);}
  void printString(float x, float y, char *str);
  void updateRadar(
		   MOVING_OBJ_LIST pedList,
		   MOVING_OBJ_LIST carList,  
		   MOVING_OBJ_LIST fryList,  
		   MAP_OBJ_LIST mapObjList,
		   myBird bird){
    radarPedList = pedList;
    radarCarList = carList;
    radarFryList = fryList;

    radarEye.setX(bird.getEyeX());
    radarEye.setY(bird.getEyeY());
    radarDirection = bird.getViewVector();
  }
  
  void setWorldSize(float worldWidth, float worldDepth){
    radarWidth = worldWidth; 
    radarDepth = worldDepth;
  }

  void setRadius(float r){radarRadius=r;}
  void incRadius(float r){radarRadius+=r;}
  void decRadius(float r){if (radarRadius>r)radarRadius-=r;}

  void setViewDist(float t){if ((t<5) && (t>1))viewFactor = t;}
  void incViewDist(float t){setViewDist(viewFactor + t);}
  void decViewDist(float t){setViewDist(viewFactor - t);}
  void drawPoint(my3dPoint p, float r, float g, float b);
  void drawPedestrians();
  void drawCars();
  void drawFries();
  void drawMapObjects();
  void drawOutline();
  void drawBird();
  void draw();
};

Radar::Radar() 
{
  x = -2000;
  y = 0;
  strcpy(label," :");
  enabled = 1;	
  radarRadius = 300;
  viewFactor = 5;
};


void Radar::drawOutline(){
  float angle;
  float vectorX=x+radarRadius/viewFactor; 
  float vectorY=y+radarRadius/viewFactor;


  glPushMatrix();
  glTranslatef(0,0,-0.1);    
  /*********** DRAW THE RADAR OUTLIE **************/
  glEnable (GL_BLEND);
  glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  glColor4f(0,0.5,0,0.4);
  for(angle=0.0;angle<=(2.1f*3.14159);angle+=0.2)
    { 
      glBegin(GL_TRIANGLES); 
      glVertex2d(vectorX,vectorY);
      vectorX = x + (radarRadius/viewFactor*(float)sin((double)angle));
      vectorY = y + (radarRadius/viewFactor*(float)cos((double)angle));		
      glVertex2d(vectorX,vectorY);
      glVertex2d(x,y);
      glEnd();
    }
  glColor3f(0,1,0);
  /*for(angle=0.0;angle<=(2.1f*3.14159);angle+=0.2)
    { 
      glBegin(GL_LINES); 
      glVertex2d(vectorX,vectorY);
      vectorX = x + (radarRadius/viewFactor*(float)sin((double)angle));
      vectorY = y + (radarRadius/viewFactor*(float)cos((double)angle));		
      glVertex2d(vectorX,vectorY);
      glEnd();
    }*/
  glDisable (GL_BLEND);
  glPopMatrix();
  
  
}

void Radar::drawPoint(my3dPoint p, float r, float g, float b){
  //my3dPoint p is in world Coordinats...
  // the rest are colors.

  float tPointX = 0, tPointY=0;
  float distFromBird;
  myMatrix t,invT,rot;

  //Get the angle of rotation of the radar.
  float radarRot =  atan2(radarDirection.getY(), radarDirection.getX());
  radarRot = 90 - radarRot * (180/PI); //Convert to degrees;
  //Bring the bird to the origin
  t.setTranslation(-radarEye.getX(),-radarEye.getY(),0);
  //rotate it by the angle
  rot.setRotationAroundZ(radarRot); 
  //bring it back to radar coordinates system
  invT.setTranslation(radarEye.getX(),radarEye.getY(),0);
  //Do the transphormations
  p = invT*rot*t*p;
  
  tPointX = (p.getX() - radarEye.getX()) *  radarRadius*2 / radarWidth;
  tPointY = (p.getY() - radarEye.getY()) *  radarRadius*2 / radarDepth;
  distFromBird = sqrt(tPointX*tPointX + tPointY*tPointY);

  if (distFromBird < radarRadius/viewFactor-1){
    glColor3f(r,g,b);
    glBegin(GL_POINTS);	
    //glVertex2f(x, y);
    glVertex2f(x+tPointX, y-tPointY);
    glEnd();
    }

}

void Radar::drawPedestrians(){
  Pedestrian *pPed;
  MOVING_OBJ_LIST::iterator it = radarPedList.begin();
  /**************** DRAW THE PEDESTRIANS **********/
  while(it != radarPedList.end()){
    pPed = (Pedestrian *) *it;
    if (!pPed->isSoiled())
      drawPoint(pPed->m_location,.7,1,.7);
    else
      drawPoint(pPed->m_location,.5,0.5,.5);
    it++;   
  }
  
}

void Radar::drawCars(){
  MOVING_OBJ_LIST::iterator it = radarCarList.begin();
  Car *pCar;
  /*************** DRAW THE CARS ********************/
  while(it != radarCarList.end()){
    pCar = (Car *) *it;
    if (! pCar->isSoiled())
      drawPoint(pCar->m_location,.7,.7,1);
    else
      drawPoint(pCar->m_location,.5,.5,.5);
    it++;
  }
}

void Radar::drawFries(){
  MOVING_OBJ_LIST::iterator it = radarFryList.begin();
  myFries *pFry;
  /*************** DRAW THE CARS ********************/
  while(it != radarFryList.end()){
    pFry = (myFries *) *it;
    drawPoint(pFry->m_location,1,.2,.2);
    it++;
  }
}

void Radar::drawBird(){
  /********* Draw the center (the birds' position) *************/
  glColor3f(0,0,0);
  glBegin(GL_POINTS);
  //glVertex2f(x, y);
  glVertex2f(x, y);
  glEnd();
}

void Radar::drawMapObjects(){
  /********** DRAW THE MAP OBJECTS (sidewals roads parks ...) *****/
  /*
    MAP_OBJ_LIST::iterator it = radarMapObjList.begin();
    while(it != radarMapObjList.end()){
    //(*it)->draw();
    it++;
    }
  */
}

void Radar::draw()
{	
  if ( POOP_LIGHTING ) glDisable(GL_LIGHTING);
  
  glPointSize(5);

  //Draw the circle
  drawOutline();
  //Print the label
  glColor3f(1,1,1);
  printString(x-13, y+45, label);

  //Draw elements in Radar
  drawPedestrians();
  drawCars();
  drawFries();
  drawBird();
  drawMapObjects();

  //Draw the North
  my3dPoint tp;
  // A Hack should always pass the if in the drawPoint function.
  //TODO: FIX THE NORTH
  tp.setXYZ(
	    radarEye.getX(),
	    radarEye.getY() + (1.7 + viewFactor)*radarRadius - 2, 
	    0
	    );
  glPointSize(5);
  drawPoint(tp, 1,1,1);
  
  glColor3f(1,1,1);
  glLineWidth(1);
  if ( POOP_LIGHTING ) glEnable(GL_LIGHTING);
  
};




void Radar::printString(float x,float y, char *str){
  char *c;
  glRasterPos2f(x,y);
  for (c=str; *c != '\0'; c++) 
    {	
      glutBitmapCharacter(GLUT_BITMAP_HELVETICA_12, *c);
    }
}



#endif

