/* Poopmup - An urban flight sim
 *
 * Copyright (C) 2002 Zaroubi, Fink, Leclerc, Stanek szaroubi@hotmail.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* PoopWorld class
   Stephen Fink
   Oct 26, 2002
   
   This class will encapsulate the whole world (city and inhabitabnts).
   It will maintain the lists needed 
   
*/


#include "../../common.h"
#include <math.h>
#include <time.h>
#include "../base/myCam.h"
#include "../mapObjects/Tile.h"
#include "../movableObjects/MovableObject.h"
#include "../mapObjects/MapObject.h"
#include "PoopWorldGenerator.h"
#include "../movableObjects/Pedestrian.h"
#include "../movableObjects/Car.h"
#include "../derivedGeom/myAxis.h"

#include "../bird/myBird.h"
#include "../textureLoader.h"
#include "../base/myLight.h"
#include "../mapObjects/myBuilding.h"
#include "../osd/onScreenDisplay.h"
#include "../osd/radar.h"
#include "../derivedGeom/myFries.h"
#include "../derivedGeom/myCloud.h"
#include "../myObjectives.h"


//#define ORTHO_PROJ_X 800
//#define ORTHO_PROJ_Y 600


#define WORLD_WIDTH 6000
#define WORLD_HEIGHT 2000
#define world_DEPTH 6000
class PoopWorld
{
  /********* PUBLIC *******/
 public:
  
  /*** OSD: The Radar ***/
  Radar osd_Radar;

  //The Bird (The users)
  myBird bird;
  //TODO: Check if needed or usefull
  myLight moon;
  //The X,Y coords of the mouse on the screen.
  my3dPoint mousePos;
  //the poopOCam camera
  myCam poop_o_cam;
  
  //Are the cars and peds moving?
  int isPaused;
  //??????
  int currentblock;

  // Public Methods
  PoopWorld();
  ~PoopWorld();
  
  
  void Draw(void);
  void DrawPoopoCam(void);
  void Update(void);
  
  void SetTileArray(int x, int y, float size);
  void loadMasks();
  void loadTextures();
  void loadDisplayLists();
  
  void CreatePedestrian();
  void DeleteAllPedestrians();
  void UpdatePedestrians();
  void SoilPedestrian( MOVING_OBJ_LIST::iterator it);

  void CreateCar();
  void DeleteAllCars();
  void UpdateCars();
  void SoilCar( MOVING_OBJ_LIST::iterator it);

  void CreateFries();
  void DeleteAllFries();
  void UpdateFries();
  void ConsumeFry(MOVING_OBJ_LIST::iterator it);

  void CreateClouds();
  void DeleteAllClouds();
  void UpdateClouds();

  void initOnScreenDisplay();
  void UpdateOnScreenDisplay();
  
  void Atmospherizer();
  void CollisionDetection();

  void StartLevel(int level = 1);
  int StartNextLevel();

  void setOrthographicProjection();
  void resetPerspectiveProjection();

  float getWorldWidth(){return m_TileX*m_TileSize;};
  float getWorldDepth(){return m_TileY*m_TileSize;};
  float getM_TileSize(){return m_TileSize;};  

  /***** Collision handlers *******/
  void ped2ped(MOVING_OBJ_LIST::iterator it1,MOVING_OBJ_LIST::iterator it2);
  void car2car(MOVING_OBJ_LIST::iterator it1,MOVING_OBJ_LIST::iterator it2);
  void bird2fry(MOVING_OBJ_LIST::iterator it);
  void poop2ped(myPoop *pPoop, MOVING_OBJ_LIST::iterator it);
  void poop2car(myPoop *pPoop, MOVING_OBJ_LIST::iterator it);
  void bird2Building(Tile t);

  void setOrthProjSize(float w, float h);
  float getOrthProjWidth(){return  orthProjWidth;}
  float getOrthProjHeight(){return  orthProjHeight;}
  
 
  myObjectives m_objectives;

 private:	
 
  //The tiles that the world is based on
  Tile* m_pTileArray;

  //Number of tiles in each direction
  int   m_TileX, m_TileY;
  
  //Size of a tile
  float m_TileSize;
  
  //Current Level of the game
  int currLevel;
  //Speed of the bird TODO:Check if still in need or
  //exchange for: bird.getSpeed();
  float birdSpeed;
  
  //Position of the light.
  static float lightposX, lightposY, lightposZ;
  
  //Todo:Check is still in use
  unsigned long b2bTimeStamp;

  //the world generator
  PoopWorldGenerator m_WorldGen;
  
  //Lists of all moving objects
  MOVING_OBJ_LIST m_listPedestrians;
  MOVING_OBJ_LIST m_listCars;
  MOVING_OBJ_LIST m_listFries;
  MOVING_OBJ_LIST m_listClouds;

  //List of map objects.
  MAP_OBJ_LIST    m_listMapObjects;

  /**** Textures loader ****/
  textureLoader block[2];
  textureLoader msk;
  textureLoader carPainter;
  textureLoader pedPainter;

  /**** OSDs ****/
  onScreenDisplay osd_Height;
  onScreenDisplay osd_PoopOMeter;
  onScreenDisplay osd_Mask;
  onScreenDisplay osd_Health;
  onScreenDisplay osd_Points;
  onScreenDisplay osd_fps;
  onScreenDisplay osd_Speed;
  onScreenDisplay osd_Cursor;
  onScreenDisplay osd_Timer;
  onScreenDisplay osd_Mission;
  
  
  //Size of the "Layer" on the screen.
  //this is where all the OSD's will be drawn.
  float orthProjWidth;
  float orthProjHeight;
  
  // Setup for Drawing
  void SetupCamera();
  void SetupLights();

  // Drawing of Entities
  void DrawPedestrians();
  void DrawCars();
  void DrawClouds();
  void DrawMapObjects();
  void drawOnScreenDisplay();  
  void DrawFries();

};

/*************** COLLITION DETECTION *******************************/
void PoopWorld::CollisionDetection(){
  Pedestrian *pPed1, *pPed2;
 
  MOVING_OBJ_LIST::iterator it1 = m_listPedestrians.begin();
  MOVING_OBJ_LIST::iterator it2;
  
  myPoop *pPoop;

  /********* PED TO PED COLLISION DETECTION LOOP *******/
  /** Algo
   * Check collision detection between ped1 and ped2 if:
   * ped1 is "smaller" that ped2
   **/
  while(it1 != m_listPedestrians.end()){
      pPed1 = (Pedestrian *) *it1;
      it2 = it1;
      it2++;
      while(it2 != m_listPedestrians.end()){
          pPed2 = (Pedestrian *)*it2;
          if(pPed1->collide( *pPed2 ) ){
	    ped2ped(it1,it2);
          }
          it2++;
      }
    /********* PED TO POOP COLLISION DETECTION LOOP *****/
    // also check for collision with the poop
      pPoop = bird.getFirstPoop();
      while(pPoop)
	{
	  if(pPed1->collide(*pPoop)){
	    //cout << "Ped got pooped!" << endl;
	    poop2ped(pPoop, it1);
	  }
	  pPoop = bird.getNextPoop();
      }
      it1++;
  }
  
  /******** CAR TO CAR COLLISION DETECTION *******/
  
  Car *pCar1, *pCar2;
  it1 = m_listCars.begin();
  while(it1 != m_listCars.end()){
      pCar1 = (Car *) *it1;
      it2 = it1;
      it2++;
      if (FAST_COMPUTER){
	while(it2 != m_listCars.end()){
	  pCar2 = (Car *)*it2;
	  if(pCar1->collide( *pCar2 ) ){
	    car2car(it1,it2);
	  }
	  it2++;
	}
      }
      /********* CAR TO POOP COLLISION DETECTION LOOP *****/
      // also check for collision with the poop
      pPoop = bird.getFirstPoop();
      while(pPoop)
	{
	  if(pCar1->collide(*pPoop)){
	    poop2car(pPoop, it1);
	  }
	  pPoop = bird.getNextPoop();
	}
      it1++;
  }
  
  /*** BIRD TO BUILDING COLLISION DETECTION ****/
  osd_Mask.disable();
  Tile tileUnderBird = m_WorldGen.GetTileFromWorldCoord(
                    (int) bird.getEyeX()+1,
                    (int) bird.getEyeY()+1
                    );
  if (tileUnderBird.m_pMapObject > 0){
    if (tileUnderBird.m_pMapObject->m_Top > bird.getEye().getZ()){
      bird2Building(tileUnderBird);
    }
  }
 
  /*** BIRD TO FRY COLLISION DETECTION *****/
   myFries *pFry;
   my3dPoint birdsEye = bird.getEye();
   it1 = m_listFries.begin();
   while (it1 != m_listFries.end()){
     pFry = (myFries *) *it1;
     if (pFry->collide(birdsEye)){
       bird2fry(it1);
       break;
     }
     it1++;
   }
}

/**************** COLLISION HANDLERS **************/
void PoopWorld::ped2ped(MOVING_OBJ_LIST::iterator it1,MOVING_OBJ_LIST::iterator it2){

}

void PoopWorld::car2car(MOVING_OBJ_LIST::iterator it1,MOVING_OBJ_LIST::iterator it2){
  Car *pCar;
  pCar = (Car *) *it1;
  pCar->ChangeDirection();
}
void PoopWorld::bird2fry(MOVING_OBJ_LIST::iterator it){
  ConsumeFry(it);
}

void PoopWorld::poop2ped(myPoop * pPoop, MOVING_OBJ_LIST::iterator it){
  //iterator it is for ped
  SoilPedestrian(it);
  bird.DeletePoop(pPoop);
}

void PoopWorld::poop2car(myPoop *pPoop, MOVING_OBJ_LIST::iterator it){	  
  SoilCar(it);
  bird.DeletePoop(pPoop);
}

void PoopWorld::bird2Building(Tile t){
  //Make the Bird bouce back.
  //cout<<"_";
   /*
     if (GET_CURRENT_SECOND > b2bTimeStamp + 1){
     
     //bird.setSpeed((-1)*bird.getSpeed());
     myVector t = bird.getViewVector();
     t.setX( -t.getX());
     t.setY( -t.getY());
     t.setZ( -t.getZ());
     bird.setLookAt(t );
     //bird.decHealthBy(30);
     
     //cout <<".";
     //bird.setSpeed(0);
     //bird.setLookAt(0.01,0.1,1); //can't be = to upVector
     b2bTimeStamp = GET_CURRENT_SECOND;
     }
   */
  if (t.m_pMapObject->getType() == MAPOBJ_TYPE_BUILDING){
    osd_Mask.setColor1(0.7,0,0);
    osd_Mask.enable();
    bird.decHealthBy(2);
  }
  else if(t.m_pMapObject->getType() == MAPOBJ_TYPE_PARK){
    osd_Mask.setColor1(0,0.7,0);
    osd_Mask.enable();
    bird.incHealthBy(1);
  }
}

/****************************************************/


void PoopWorld::Atmospherizer(){
  //Builds the Box around the world.

  glPushMatrix();
  glScalef(getWorldWidth()/2, getWorldDepth()/2, WORLD_HEIGHT);
  glColor3f(.7,.7,1);
  glTranslatef(1,1,0.5);
  
  if ( POOP_LIGHTING ) glDisable(GL_LIGHTING);	
  glEnable( GL_TEXTURE_2D );
  
  glDisable(GL_CULL_FACE);
  //top Face		
  glBindTexture( GL_TEXTURE_2D, block[currentblock].texture[10]);
  glBegin(GL_QUADS);
  glNormal3f( 0.0f, 0.0f, -1.0f);			// Normal Pointing Towards Viewer
  glTexCoord2f(0.0f, 0.0f); glVertex3f(-1.0f, -1.0f,  1.0f);	// Point 1 (Front)
  glTexCoord2f(1.0f, 0.0f); glVertex3f( 1.0f, -1.0f,  1.0f);	// Point 2 (Front)
  glTexCoord2f(1.0f, 1.0f); glVertex3f( 1.0f,  1.0f,  1.0f);	// Point 3 (Front)
  glTexCoord2f(0.0f, 1.0f); glVertex3f(-1.0f,  1.0f,  1.0f);	// Point 4 (Front)
  glEnd();			
  
  
  // front Face
  glBindTexture( GL_TEXTURE_2D, block[currentblock].texture[10]);
  glBegin(GL_POLYGON);
  glNormal3f( 0.0f, -1.0f, 0.0f);					// Normal Pointing Up
  glTexCoord2f(0.0f, 1.0f); glVertex3f(-1.0f,  1.0f, -1.0f);	// Point 1 (Top)
  glTexCoord2f(0.0f, 0.0f); glVertex3f(-1.0f,  1.0f,  1.0f);	// Point 2 (Top)
  glTexCoord2f(1.0f, 0.0f); glVertex3f( 1.0f,  1.0f,  1.0f);	// Point 3 (Top)
  glTexCoord2f(1.0f, 1.0f); glVertex3f( 1.0f,  1.0f, -1.0f);	// Point 4 (Top)
  glEnd();			
  
  
  
  // back Face
  glBindTexture( GL_TEXTURE_2D, block[currentblock].texture[10]);
  glBegin(GL_QUADS);
  glNormal3f( 0.0f,-1.0f, 0.0f);					// Normal Pointing Down
  glTexCoord2f(1.0f, 1.0f); glVertex3f(-1.0f, -1.0f, -1.0f);	// Point 1 (Bottom)
  glTexCoord2f(0.0f, 1.0f); glVertex3f( 1.0f, -1.0f, -1.0f);	// Point 2 (Bottom)
  glTexCoord2f(0.0f, 0.0f); glVertex3f( 1.0f, -1.0f,  1.0f);	// Point 3 (Bottom)
  glTexCoord2f(1.0f, 0.0f); glVertex3f(-1.0f, -1.0f,  1.0f);	// Point 4 (Bottom)
  glEnd();			
  
  
  // Right face
  glBindTexture( GL_TEXTURE_2D, block[currentblock].texture[10]);
  glBegin(GL_QUADS);
  glNormal3f( -1.0f, 0.0f, 0.0f);					// Normal Pointing Right
  glTexCoord2f(1.0f, 0.0f); glVertex3f( 1.0f, -1.0f, -1.0f);	// Point 1 (Right)
  glTexCoord2f(1.0f, 1.0f); glVertex3f( 1.0f,  1.0f, -1.0f);	// Point 2 (Right)
  glTexCoord2f(0.0f, 1.0f); glVertex3f( 1.0f,  1.0f,  1.0f);	// Point 3 (Right)
  glTexCoord2f(0.0f, 0.0f); glVertex3f( 1.0f, -1.0f,  1.0f);	// Point 4 (Right)
  glEnd();			
  
  // Left Face
  glBindTexture( GL_TEXTURE_2D, block[currentblock].texture[10]);
  glBegin(GL_QUADS);
  glNormal3f(1.0f, 0.0f, 0.0f);					// Normal Pointing Left
  glTexCoord2f(0.0f, 0.0f); glVertex3f(-1.0f, -1.0f, -1.0f);	// Point 1 (Left)
  glTexCoord2f(1.0f, 0.0f); glVertex3f(-1.0f, -1.0f,  1.0f);	// Point 2 (Left)
  glTexCoord2f(1.0f, 1.0f); glVertex3f(-1.0f,  1.0f,  1.0f);	// Point 3 (Left)
  glTexCoord2f(0.0f, 1.0f); glVertex3f(-1.0f,  1.0f, -1.0f);	// Point 4 (Left)
  glEnd();								// Done Drawing Quads
  glDisable(GL_TEXTURE_2D);
  glPolygonMode(GL_FRONT,GL_FILL);
  glEnable(GL_DEPTH_TEST);
  if ( POOP_LIGHTING ) glEnable(GL_LIGHTING);	
  glEnable(GL_CULL_FACE);
  glPopMatrix();		
}



/********************** CONSTRUCTORS *******************************/
PoopWorld::PoopWorld() : m_pTileArray(NULL)
{
  bird.reset();
  isPaused = 0;
  m_TileX = m_TileY = 0;
  //m_Camera.moveLookAtBy(0,20,0);
  
  initOnScreenDisplay();
  currentblock = 0;

  // set up the world generator
  
}

void PoopWorld::loadMasks(){
  msk.loadTextureMask();
  osd_Mask.mask = msk.textmsk[0];
  osd_Mask.cross = msk.textmsk[1];
}

/********************** DESTRUCTOR *******************************/
PoopWorld::~PoopWorld() 
{
  delete [] m_pTileArray;
  DeleteAllPedestrians();
  DeleteAllCars();
  DeleteAllFries();
  DeleteAllClouds();
}

/********************** DRAW *******************************/
void PoopWorld::Draw(void)
{
  // Draw the current scene 

BEGIN_TIMESTAT(TIMESTAT_World_Atmozpherizer);
  Atmospherizer(); 
END_TIMESTAT(TIMESTAT_World_Atmozpherizer);

BEGIN_TIMESTAT(TIMESTAT_World_SetupCamera);
  SetupCamera();
END_TIMESTAT(TIMESTAT_World_SetupCamera);

BEGIN_TIMESTAT(TIMESTAT_World_DrawMapObjects);
  DrawMapObjects();
END_TIMESTAT(TIMESTAT_World_DrawMapObjects);

BEGIN_TIMESTAT(TIMESTAT_World_DrawPedestrians);
  DrawPedestrians();
END_TIMESTAT(TIMESTAT_World_DrawPedestrians);

BEGIN_TIMESTAT(TIMESTAT_World_DrawCars);
  DrawCars();
END_TIMESTAT(TIMESTAT_World_DrawCars);

BEGIN_TIMESTAT(TIMESTAT_World_DrawFries);
  DrawFries();
END_TIMESTAT(TIMESTAT_World_DrawFries);

BEGIN_TIMESTAT(TIMESTAT_World_DrawClouds);
  DrawClouds();
END_TIMESTAT(TIMESTAT_World_DrawClouds);

BEGIN_TIMESTAT(TIMESTAT_World_BirdDrawGeometry);
  bird.drawGeometry();//ADDED BY SHERIF    	
END_TIMESTAT(TIMESTAT_World_BirdDrawGeometry);
  
BEGIN_TIMESTAT(TIMESTAT_World_DrawOSD);
  drawOnScreenDisplay();
END_TIMESTAT(TIMESTAT_World_DrawOSD);


}

void PoopWorld::DrawPoopoCam()
{
  // set up a new camera for the poop-o-cam
  Atmospherizer();
  bird.drawPoopoCam();

  // draw only the relevant objects
  DrawMapObjects();
  DrawPedestrians();
  DrawCars();
  
}

/******************** ON SCREEN DISPLAY **********************/
void PoopWorld::drawOnScreenDisplay(){
  setOrthographicProjection();
  
  osd_Height.draw();
  osd_PoopOMeter.draw();
  if (FAST_COMPUTER) osd_Radar.draw();
  osd_Health.draw();
  osd_Points.draw();
  osd_fps.draw();
  osd_Speed.draw();
  osd_Cursor.draw();
  osd_Timer.draw();
  osd_Mission.draw();
  
  osd_Mask.draw();
  
  resetPerspectiveProjection();
}

void PoopWorld::UpdateOnScreenDisplay(){
  
  osd_Height.setX((int)(orthProjWidth*(1.0/100)));
  osd_Height.setY((int)(orthProjHeight*(-39.0/100)));
  //Devide by 20 so that it looks right.
  osd_Height.setCurVal(bird.getEye().getZ());
  
  osd_PoopOMeter.setX( (int)(orthProjWidth*(1.0/100)));
  osd_PoopOMeter.setY((int)(orthProjHeight*(-31.0/100)));
  osd_PoopOMeter.setCurVal(bird.getNumPoop()*3);
  
  if (FAST_COMPUTER){
    osd_Radar.setRadius(orthProjHeight/3);
    osd_Radar.setX((int) (orthProjWidth *8.0/9));
    osd_Radar.setY((int) (orthProjHeight * 3.0/9) );
    osd_Radar.updateRadar(m_listPedestrians, m_listCars,m_listFries,m_listMapObjects, bird);
  }
  
  osd_Health.setX( (int)(orthProjWidth*(1.0/100)));
  osd_Health.setY((int)(orthProjHeight*(-35.0/100)));
  osd_Health.setCurVal(bird.getHealth());
  
  osd_Points.setCurVal(bird.getPoints());
  
  
  osd_Speed.setX((int)(orthProjWidth*(1.0/100)));
  osd_Speed.setY((int)(orthProjHeight*(-43.0/100)));
  osd_Speed.setCurVal(bird.getSpeed());
  
  osd_Cursor.setCursorPos(mousePos);
  osd_Cursor.setScreenCenter(orthProjWidth/2, 0);
  
  char buf[50];
  m_objectives.getTimeLeftString(buf);
  osd_Timer.setLabel(buf);
  osd_Timer.setX((int)(orthProjWidth * 35.0/100));
  osd_Timer.setY((int)(orthProjHeight * (-48.0/100)));
  
  sprintf(buf, "Level %d -- Peds: %d Cars: %d Fries: %d", 
	  currLevel,
	  m_objectives.getPedsLeft(),
	  m_objectives.getCarsLeft(),
	  m_objectives.getFriesLeft()
	  );
  
  osd_Mission.setX((int)(orthProjWidth* 1.0/100));
  osd_Mission.setY((int)(orthProjHeight* (-48.0/100)));
  
  osd_Mission.setLabel(buf);
  
  osd_fps.setX((int)(orthProjWidth * 60.0/100));
  osd_fps.setY((int)(orthProjHeight * (-48.0/100)));
  osd_fps.setCurVal((frameCount*CLOCKS_PER_SEC)/currTime);
  osd_fps.setMaxVal(GET_CURRENT_SECOND);
  
  osd_Points.setX((int) (orthProjWidth/2));
  osd_Points.setY((int) (orthProjHeight * 48.0/100));
  
  osd_Mask.setX((int) orthProjWidth);
  osd_Mask.setY((int) orthProjHeight);
  
}

void PoopWorld::initOnScreenDisplay(){
  osd_Height.setMaxVal(BIRD_MAX_HEIGHT);
  osd_Height.setColor1(0, 0, .3, .7);
  osd_Height.setColor2(.7, .7, 1,0.7);
  osd_Height.setLabel("HeightOMeter");
  osd_Height.setType(TYPE_HBAR);
  osd_Height.setSuffix("m");
  osd_Height.setAtZero("Under ground");
  
  osd_PoopOMeter.setColor1(0.3,0.3,0,.7);
  osd_PoopOMeter.setColor2(.7,.7,0,.7);
  osd_PoopOMeter.setLabel("PoopOMeter");
  osd_PoopOMeter.setType(TYPE_HBAR);
  
  osd_PoopOMeter.setMaxVal(bird.getMaxPoop()*3);
  osd_PoopOMeter.setSuffix("poops");
  osd_PoopOMeter.setAtZero("OUT OF POOP");
  
  
  osd_Radar.setLabel("");
  osd_Radar.setViewDist(3);
  osd_Radar.setWorldSize(getWorldWidth(), getWorldDepth());
  
  osd_Health.setColor1(.45,0,0);
  osd_Health.setColor2(1,1,1);
  osd_Health.setType(TYPE_HBAR);
  osd_Health.setMaxVal(bird.getMaxHealth());
  osd_Health.setLabel("HealthOmeter");
  osd_Health.setAtZero("You are dead");
  
  osd_Points.setLabelColor(1,0,0,1);
  osd_Points.setType(TYPE_TEXT);
  osd_Points.setLabel("SCORE");
  
  osd_fps.setType(TYPE_TEXT);
  osd_fps.setLabel("fps/runtime");
  osd_fps.setLabelColor(1,0,0,1);
  osd_fps.setMaxVal(60);
  
  osd_Speed.setMaxVal(bird.getMaxSpeed());
  osd_Speed.setColor1(1,1,0,1);
  osd_Speed.setColor2(1,1,.5,0.7);
  osd_Speed.setLabel("SpeedOMeter");
  osd_Speed.setType(TYPE_HBAR);
  
  osd_Cursor.setType(TYPE_CURSOR);
  osd_Cursor.setColor1(.7,.3,.3,0.2); //Color or "Triangle area"
  osd_Cursor.setColor2(.7,.3,.3,0.7); //Color of line and dots
  
  osd_Timer.setType(TYPE_TEXT);
  osd_Timer.setLabelColor(1,0,0,1);
  osd_Timer.setNoValue(true);
  
  osd_Mission.setType(TYPE_TEXT);
  osd_Mission.setLabelColor(1,0,0,1);
  osd_Mission.setNoValue(true);
  
  osd_Mask.setType(TYPE_MASK);
  osd_Mask.setFlasherOn();
  //osd_Mask.setColor1(.7,0,0,.5);
}

/****************** SWITCH TO ORTHO ************************/
void PoopWorld::setOrthProjSize(float w, float h){
  orthProjWidth = w;
  orthProjHeight = h; 
}

void PoopWorld::setOrthographicProjection() {

  // switch to projection mode
  glMatrixMode(GL_PROJECTION);
  // save previous matrix which contains the 
  // settings for the perspective projection
  glPushMatrix();
  // reset matrix
  glLoadIdentity();
  // set a 2D orthographic projection
  gluOrtho2D(0, orthProjWidth, 0, orthProjHeight);
  // invert the y axis, down is positive
  glScalef(1, -1, 1);
  // move the origin from the bottom left corner
  // to the upper left corner
  //cout << orthProjHeight<< endl;
  glTranslatef(0, -orthProjHeight/2, 0);
  glMatrixMode(GL_MODELVIEW);
}


/************** BACK TO PERSPECTIVE ***********************/

void PoopWorld::resetPerspectiveProjection() {
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
}


/********************* UPDATE *****************************/
void PoopWorld::Update(void)
{
  // Update all the entities in the world

  CollisionDetection();
  if (!isPaused){
    UpdatePedestrians();
    UpdateCars();
    UpdateFries();
    UpdateClouds();
  }
  bird.animate(); //ADDED BY SHERIF

  UpdateOnScreenDisplay();
  
}

void PoopWorld::loadTextures()
{
  block[currentblock].loadTexture();
  block[currentblock].loadTextureSky();
}

void PoopWorld::loadDisplayLists()
{
  // since each subwindow needs a seperate context, re-create any 
  // display lists we may need

  MAP_OBJ_LIST::iterator it = m_listMapObjects.begin();
  MapObject *pMapObj;
  while(it != m_listMapObjects.end())
    {
      pMapObj = (*it);
      pMapObj->SetupDisplayList();
      ++it;
    }

}

/********************* TILES *****************************/
/* x and y are the number of tiles in each direction, 
   so we get a x by y array of tiles
   size of the length of one side of the square
*/
void PoopWorld::SetTileArray(int x, int y, float size)
{
  m_TileX = x;
  m_TileY = y;
  m_TileSize = size;

  carPainter.loadTextureCar(); //TODO
  pedPainter.loadTexturePed();

  // use the world generator to make the world
  m_pTileArray = m_WorldGen.SetTileArray(x,y,size);
  m_WorldGen.GenerateRandomCity();
  m_WorldGen.MarkRoadDirections();
  m_WorldGen.GenerateMapObjectsFromTiles(m_listMapObjects, block[currentblock]);
  
  bird.setWorldSize(getWorldWidth(), getWorldDepth());
}

/********************* PEDESTRIANS *****************************/
void PoopWorld::CreatePedestrian()
{
  Pedestrian *pPed = new Pedestrian;
  
  // setup the position and direction of this ped
  int x, y = 0;
  int type = TILE_INVALID;
  
  pPed->ped_wc_headface			= pedPainter.texture[21];
  pPed->ped_wc_headsideRight	= pedPainter.texture[22];
  pPed->ped_wc_headsideLeft		= pedPainter.texture[23];
  pPed->ped_wc_headback			= pedPainter.texture[24];
  pPed->ped_wc_headtop			= pedPainter.texture[25];
  pPed->ped_wc_bodyfront		= pedPainter.texture[26];
  pPed->ped_wc_bodyback			= pedPainter.texture[27];
  pPed->ped_wc_bodymisc			= pedPainter.texture[27];
  
  while(!IS_ONE_OF(type,pPed->m_TilesCanWalkOn))
    {
      x = rand() % int((m_TileX * m_TileSize));
      y = rand() % int((m_TileY * m_TileSize));
      type = m_WorldGen.GetTileTypeFromWorldCoord(x,y);
    }
  
  // set general myobject stuff
  pPed->translateBy(x,y,1);
  
  // set ped specific stuff
  pPed->m_location.setXYZ(x,y,1);
  pPed->m_facingDir.setXYZ(1,1,0);
  
  m_listPedestrians.push_back(pPed);
}

void PoopWorld::DeleteAllPedestrians()
{
  MOVING_OBJ_LIST::iterator it = m_listPedestrians.begin();
  while(it != m_listPedestrians.end())
    {
      delete *it;
      it++;
    }
  m_listPedestrians.clear();
}

void PoopWorld::UpdatePedestrians()
{
  my3dPoint point;
  Pedestrian *pPed;
  
  MOVING_OBJ_LIST::iterator it = m_listPedestrians.begin();
  while(it != m_listPedestrians.end())
    {
       // the pedestrian wants to move in the direction given by m_facingDir
      // but if m_location + m_facingDir is not a sidewalk, then the ped has to pick a new direction

      pPed = (Pedestrian*)*it;

      point = pPed->m_location + pPed->m_facingDir*pPed->m_speed;
 // use bitwise and to check if the tile type is a legal one

      if(IS_TILE_TYPE(m_WorldGen.GetTileTypeFromWorldCoord(int(point.getX()), int(point.getY())), pPed->m_TilesCanWalkOn))
	{

	  pPed->Move();

	}
      else
	{

	    pPed->ChangeDirection();

	}

      it++;

    }
}

void PoopWorld::SoilPedestrian( MOVING_OBJ_LIST::iterator it){
  Pedestrian *pPed = (Pedestrian *) *it;
  if (pPed->getNumPooped() == 0)
    {
      bird.incPoints(pPed->getValue());
      m_objectives.PoopedPed();
    }

  pPed->poopedOn();
}

/********************* CARS *****************************/
void PoopWorld::CreateCar()
{
  Car *pCar = new Car;
  
  // setup the position and direction of this ped
  int x, y = 0;
  int type = TILE_INVALID;
  pCar->car_front = carPainter.texture[13];
  pCar->car_back = carPainter.texture[14];
  pCar->car_top	= carPainter.texture[15];
  pCar->car_windshield	= carPainter.texture[16];
  pCar->car_wheel = carPainter.texture[17];
  pCar->car_doorRight = carPainter.texture[18];
  pCar->car_doorLeft = carPainter.texture[19];
  pCar->car_sideWindow = carPainter.texture[20];
  
  while(!IS_ONE_OF(type,pCar->m_TilesCanWalkOn))
    {
      x = rand() % int((m_TileX * m_TileSize));
      y = rand() % int((m_TileY * m_TileSize));
      type = m_WorldGen.GetTileTypeFromWorldCoord(x,y);
      //cout << " type " << type << ", TilesCanWalkOn" << pCar->m_TilesCanWalkOn << endl;
    }
  
  // set general myobject stuff
  pCar->translateBy(x,y,1);
  
  // let the car choose a direction
  pCar->m_location.setXYZ(x,y,1);
  pCar->ChangeDirection();
  
  m_listCars.push_back(pCar);
}

void PoopWorld::DeleteAllCars()
{
  MOVING_OBJ_LIST::iterator it = m_listCars.begin();
  while(it != m_listCars.end())
    {
      delete *it;
      it++;
    }
  m_listCars.clear();
}

void PoopWorld::UpdateCars()
{
  my3dPoint point;
  Car *pCar;
  
  MOVING_OBJ_LIST::iterator it = m_listCars.begin();
  while(it != m_listCars.end())
    {
      // the pedestrian wants to move in the direction given by m_facingDir
      // but if m_location + m_facingDir*m_speed is not a sidewalk, then the ped has to pick a new direction
      pCar = (Car*)*it;
      
      point = pCar->m_location + pCar->m_facingDir*pCar->m_speed;
      // use bitwise and to check for the type
      if(IS_TILE_TYPE(m_WorldGen.GetTileTypeFromWorldCoord(int(point.getX()), int(point.getY())),pCar->m_TilesCanWalkOn))
	pCar->Move();
      else
	pCar->ChangeDirection();
      
      it++;
    }
}

void PoopWorld::SoilCar( MOVING_OBJ_LIST::iterator it){
  Car *pCar = (Car *) *it;
  if (pCar->getNumPooped() == 0){
    bird.incPoints(pCar->getValue());
    m_objectives.PoopedCar();
  }
  
  pCar->poopedOn();
}

/********************* FRIES *****************************/
void PoopWorld::CreateFries()
{
  myFries *pFry = new myFries;
  
  // setup the position and direction of this ped
  int x, y = 0;
  int type = TILE_INVALID;
  
  while(!IS_ONE_OF(type,TILE_ROAD)){
    x = rand() % int((m_TileX * m_TileSize));
    y = rand() % int((m_TileY * m_TileSize));
    type = m_WorldGen.GetTileTypeFromWorldCoord(x,y);
  }
  
  // set general myobject stuff
  float z = 50 + rand()%800;
  int randType = rand() % (FRY_TYPE_MAXVAL+1);
  float bonusStrength = FRY_BONUS_MINVAL + (rand() % (FRY_BONUS_MAXVAL-FRY_BONUS_MINVAL+1));
  pFry->setType(randType);
  pFry->setBonus(bonusStrength);
  pFry->translateBy(x,y,z);
  pFry->m_location.setXYZ(x,y,z);
  
  m_listFries.push_back(pFry);
}


void PoopWorld::DeleteAllFries()
{
  MOVING_OBJ_LIST::iterator it = m_listFries.begin();
  while(it != m_listFries.end())
    {
      delete *it;
      it++;
    }
  m_listFries.clear();
}

void PoopWorld::UpdateFries()
{
  if (FAST_COMPUTER){
    MOVING_OBJ_LIST::iterator it = m_listFries.begin();
    
    while(it != m_listFries.end()){
      (*it)->rotateAroundZBy(3);
      
      it++;
    }
  }
}

void PoopWorld::ConsumeFry( MOVING_OBJ_LIST::iterator it){
  myFries *pFry;
  pFry = (myFries *) *it;

  int fryType = pFry->getType();
  float bonus = pFry->getBonus();
  switch (fryType){
    //case FRY_TYPE_HEALTH:
    //bird.incHealthBy(bonus);
    //break;
  case FRY_TYPE_POOP:
    bird.incNumPoop((int)bonus);
    break;
  case FRY_TYPE_ACID:
    //setColor(0,.7,.7);
    bird.onAcid();
    break;
  case FRY_TYPE_PURIFRY:
    //setColor(1,1,1);
    bird.offAcid();
    break;
  }
  
  m_listFries.remove(pFry);
  //cout<<"Removed fry from list"<<endl;
  delete(pFry);
  //cout <<"Deleted object from Memory"<<endl;

  // mark that we ate the fry
  m_objectives.AteFry();

}

/********************* Clouds *****************************/
void PoopWorld::CreateClouds()
{
  myCloud *pCld = new myCloud;
  
  // setup the position and direction of this ped
  float x = rand() % (int) getWorldWidth();
  float y = rand() % (int) getWorldDepth();
  float z = 500 + rand() % 2000;

  pCld->translateBy(x,y,z);
  pCld->m_location.setXYZ(x,y,z);

  m_listClouds.push_back(pCld);
}

void PoopWorld::DeleteAllClouds()
{
  MOVING_OBJ_LIST::iterator it = m_listClouds.begin();
  while(it != m_listClouds.end())
    {
      delete *it;
      it++;
    }
  m_listClouds.clear();
}

void PoopWorld::UpdateClouds()
{
  //Todo: Make a check so that the clous "bounce" off the edjes
  MOVING_OBJ_LIST::iterator it = m_listClouds.begin();
  
  while(it != m_listClouds.end())
    {
      (*it)->Move();
      
      it++;
    }
}

/********************* DRAW CAMERA *****************************/
void PoopWorld::SetupCamera()
{
  bird.draw();
}

/********************* DRAW LIGHTS *****************************/
void PoopWorld::SetupLights()
{





}

/********************* DRAW ENTITIES ******************************/
void PoopWorld::DrawPedestrians()
{
  MOVING_OBJ_LIST::iterator it = m_listPedestrians.begin();
  
  while(it != m_listPedestrians.end())
    {
      (*it)->draw();
      
      it++;
    }
}

void PoopWorld::DrawCars()
{
  MOVING_OBJ_LIST::iterator it = m_listCars.begin();
  
  while(it != m_listCars.end())
    {
      (*it)->draw();
      
      it++;
    }
}

void PoopWorld::DrawFries()
{
  MOVING_OBJ_LIST::iterator it = m_listFries.begin();
  
  while(it != m_listFries.end())
    {
      (*it)->draw();
      
      it++;
    }
}

void PoopWorld::DrawClouds()
{
  MOVING_OBJ_LIST::iterator it = m_listClouds.begin();
  while(it != m_listClouds.end())
    {
      (*it)->draw();
      it++;
    }
}

void PoopWorld::DrawMapObjects()
{
  MAP_OBJ_LIST::iterator it = m_listMapObjects.begin();
  
  while(it != m_listMapObjects.end())
    {
      (*it)->draw();
      
      it++;
    }
}


void PoopWorld::StartLevel(int level /* = 1 */)
{
  // here is where we set up the objects we need for the level
  // for now, its fixed
  int i;

  currLevel = level;

  // remove everybody
  DeleteAllPedestrians();
  DeleteAllCars();
  DeleteAllFries();

  // add some pedestrians
  for(i = 0; i < 40; ++i)
    CreatePedestrian();		
  
  // add some cars
  for(i = 0; i < 40; ++i)
    CreateCar();
  
  // add some fries
  for(i = 0; i < 40; ++i)
    CreateFries();
  
  if (POOP_SHOW_CLOUDS){
    // add some Clouds
    for(i = 0; i < 10 ; ++i)
      CreateClouds();
  }
  m_objectives.setLevel(level);
}

int PoopWorld::StartNextLevel()
{
  StartLevel(++currLevel);
  return currLevel;
}




