/* Poopmup - An urban flight sim
 *
 * Copyright (C) 2002 Zaroubi, Fink, Leclerc, Stanek szaroubi@hotmail.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#ifndef POOP_EM_UP_WORLDGENERATOR_H
#define POOP_EM_UP_WORLDGENERATOR_H

#include "../mapObjects/MapObject.h"
#include "../mapObjects/Tile.h"
#include "../mapObjects/myBuilding.h"
#include "../mapObjects/myRoad.h"
#include "../mapObjects/myPark.h"
#include "../textureLoader.h"

class PoopWorldGenerator
{

 private:
  int num_roads;

 public:
  PoopWorldGenerator() {};
  ~PoopWorldGenerator() {};
  
  Tile* SetTileArray(int x, int y, float size);
  void GenerateRandomCity(int seed = -1);
  void PlaceRoads();
  void MarkSidewalks();
  void MarkRoadDirections();
  void MarkBuildings();
  void GenerateMapObjectsFromTiles(MAP_OBJ_LIST &listMapObjs, textureLoader &tex);
  void GrowBuilding(int x, int y, MapObject *pObj, int type);
  int GetTileType(int x, int y);
  int GetTileTypeFromWorldCoord(int x, int y);
  int InvalidIntersection(int i, int j);
  Tile GetTile(int x, int y);
  Tile GetTileFromWorldCoord(int x, int y);
  
  // helpers
  int GetTileIndex(int x, int y);
  
  
  // these are pointers to the real ones in PoopWorld
  int m_TileX, m_TileY;
  float m_TileSize;
  Tile *m_pTileArray;
  MAP_OBJ_LIST m_plistMapObjects;
  
};

Tile* PoopWorldGenerator::SetTileArray(int x, int y, float size)
{
  m_TileX = x;
  m_TileY = y;
  m_TileSize = size;
  int array_len = x*y;
  int i,j,idx;
  
  // clear the old one
  delete [] m_pTileArray;
  
  m_pTileArray = new Tile[array_len];
  
  // set up all the tiles size and position
  for(j = 0; j < y; ++j)
    for(i = 0; i < x; ++i)
      {
	idx = GetTileIndex(i,j);
	//m_pTileArray[idx].translateBy(i * size, j * size, 0);
	m_pTileArray[idx].m_size = size;
	m_pTileArray[idx].m_TileType = TILE_INVALID;
	//m_pTileArray[idx].m_TileType = (rand() % MAX_TILE_TYPE) + 1; // get 1 to MAX_TILE_TYPE
      }
  return m_pTileArray;
}

#define ROADS_STRAIGHT_ONLY		1 
#define ROADS_SNAKING_FROM_EDGE		2

void PoopWorldGenerator::GenerateRandomCity(int seed /* = -1 */)
{
  // set up the random number generator
  if(seed<0)
    seed = (int)time(NULL);
  srand(seed);
  
  int idx;
  
  // initialize the array
  for(int j = 0; j < m_TileY; ++j)
    for(int i = 0; i < m_TileX; ++i)
      {
	idx = GetTileIndex(i,j);
	m_pTileArray[idx].m_TileType = TILE_INVALID;
      }
  
  PlaceRoads();
  MarkSidewalks();
  MarkBuildings();
  
}

int PoopWorldGenerator::InvalidIntersection(int i, int j){
  int idx;
  int bRoadFound = 0;

  //How far from adjacent roads ?
  int distFromOtherRoad = 5;

  //cout << "=========="<<endl;
  for (int xOffset = 0;( (xOffset < m_TileX) && !bRoadFound); xOffset++){
    //For every row
    for (int dist = -distFromOtherRoad; dist < distFromOtherRoad ;dist++){
      if (xOffset != i){
	idx = GetTileIndex(xOffset, j+ dist);
	if(idx > 0 && IS_TILE_TYPE(m_pTileArray[idx].m_TileType,TILE_INTERSECTION)) bRoadFound = 1;
      }
    }
  }
  
  for (int yOffset = 0; ( (yOffset < m_TileY) ); yOffset++){
    //For every Col
    for (int dist = -distFromOtherRoad; dist < distFromOtherRoad;dist++){
      if (yOffset != j){
	idx = GetTileIndex(i + dist, yOffset);
	if(idx > 0 && IS_TILE_TYPE(m_pTileArray[idx].m_TileType,TILE_INTERSECTION)) bRoadFound = 1;
      }
    }
    
    
  }
  
  
  return bRoadFound;
  
}


void PoopWorldGenerator::PlaceRoads()
{
  // arrays for testing the 8 neighbours of the current tile
  // first 4 MUST be U,D,L,R
  // diagonals go in the last 4
  int deltax[8] = {-1, 1, 0, 0, 1, 1,-1,-1};
  int deltay[8] = { 0, 0, 1,-1,-1, 1, 1,-1};
  
  int num_roads = (int) ( sqrt((float) (m_TileX * m_TileY)) *  (5 +  rand() % 5)/100.0);
  //int num_roads = 5;
  int road_t_probability = 45;
  int i,j,idx;
  
  // for road generation
  int x[4], y[4];
  int valid[4];
  
  /* GENERATE ONLY STRAIGHT ROADS
     Choose random intersection points, and extend it to the end of the map
  */  
  while(num_roads)
    {
      i = rand() % m_TileX;
      j = rand() % m_TileY;
     
      // check for nearby roads. if there's another road too close to this interscetion, 
      // then just ignore this proposed intersection

      
      if( InvalidIntersection(i, j) )
	{
	  // road found that's too close. just stop everything
	  valid[0] = valid[1] = valid[2] = valid[3] = 0;
	}
      else
	{
	  // explore all the paths around this point
	  valid[0] = valid[1] = valid[2] = valid[3] = 1;
	  num_roads--;
	  idx = GetTileIndex(i, j);
	  m_pTileArray[idx].m_TileType = TILE_INTERSECTION;
	}
      
      
      x[0] = x[1] = x[2] = x[3] = i;
      y[0] = y[1] = y[2] = y[3] = j;
      
      // while at least one is valid
      while(valid[0] + valid[1] + valid[2] + valid[3])
	{
	  for(i = 0; i < 4; ++i)
	    {
	      if(valid[i])
		{
		  idx = GetTileIndex(x[i], y[i]);
		  if(idx >= 0)
		    {
		      // this index works. Set it as a road
		      if(IS_TILE_TYPE(m_pTileArray[idx].m_TileType,TILE_ROAD))
			{
			  // Intersection Test
			  // stop this road at the intersection if the probablilty ]
			  // says so
			  if(rand() % 100 < road_t_probability)
			    valid[i] = 0;
			}
		      else
			{
			  if (! (IS_TILE_TYPE(m_pTileArray[idx].m_TileType,TILE_INTERSECTION)))
			    m_pTileArray[idx].m_TileType = TILE_ROAD;
			}
		      
		      // push this point outward
		      x[i] += deltax[i];
		      y[i] += deltay[i];
		    } 
		  else
		    {
		      // index isn't valid, so kill the path
		      valid[i] = 0;
		    }
		} // end if valid[i]
	    } // end for all i
	} // end while at least one path is valid
      
      
    }
}

void PoopWorldGenerator::MarkSidewalks()
{
  int i, j, k, idx;
  Tile* pCurrTile; // pointer to current tile we're interested in
  int bRoadFound;

  int deltax[8] = {-1, 1, 0, 0, 1, 1,-1,-1};
  int deltay[8] = { 0, 0, 1,-1,-1, 1, 1,-1};

  // place the sidewalk and the buildings
  for(i = 0; i < m_TileX; ++i)
    for(j = 0; j < m_TileY; ++j)
      {
	pCurrTile = NULL;
	bRoadFound = 0;
	
	idx = GetTileIndex(i,j);
	if(idx >= 0)
	  pCurrTile = &m_pTileArray[idx];
	
	if(pCurrTile->m_TileType == TILE_INVALID)
	  {
				// look for neighbour that is a road
				// 8 neighbours to test
	    for(k = 0; k < 8 && !bRoadFound; ++k)
	      {
		idx = GetTileIndex(i + deltax[k], j + deltay[k]);
		if(idx >= 0)
		  {
		    if(IS_TILE_TYPE(m_pTileArray[idx].m_TileType,TILE_ROAD))
		      bRoadFound = 1;
		  }
	      }
	    
	    if(bRoadFound)
	      {
		pCurrTile->m_TileType = TILE_SIDEWALK;
	      }
	    
	  } // end if curr tile hasn't been set yet
	
      }// end for all i, j in the map
}

void PoopWorldGenerator::MarkBuildings()
{
  Tile *pCurrTile;
  int idx;

  // place the sidewalk and the buildings
  for(int i = 0; i < m_TileX; ++i)
    for(int j = 0; j < m_TileY; ++j)
      {
	pCurrTile = NULL;
	
	idx = GetTileIndex(i,j);
	if(idx >= 0)
	  pCurrTile = &m_pTileArray[idx];
	
	if(pCurrTile->m_TileType == TILE_INVALID)
	  {
	    pCurrTile->m_TileType = TILE_BUILDING;
	  } // end if curr tile hasn't been set yet
  
      }// end for all i, j in the map
}

void PoopWorldGenerator::MarkRoadDirections()
{
  // look through all the tiles and mark what directions the roads are going
  // this will also mark intersections
  int x, y, i;
  int n; // neighbour of the current tile.
  
  // directions are N S E W
  int type[4] = {TILE_ROAD_NS, TILE_ROAD_NS, TILE_ROAD_EW, TILE_ROAD_EW};
  int dx[4]   = {0, 0,1,-1};
  int dy[4]   = {1,-1,0, 0};
  
  int idx;
  
  for(x = 0; x < m_TileX; ++x)
    for(y = 0; y < m_TileY; ++y)
      {	
	idx  = GetTileIndex(x,y);
	if(IS_TILE_TYPE(m_pTileArray[idx].GetTileType(), TILE_ROAD))
	  {
				// only set direction for roads!
	    for(i = 0; i < 4; ++i)
	      {
		n = GetTileIndex(x + dx[i], y + dy[i]);
		if(   n >= 0 && IS_TILE_TYPE(m_pTileArray[n].m_TileType,TILE_ROAD))
		  m_pTileArray[idx].m_TileType |= type[i];
		
	      } // end for 4 directions
	  }
      } // end for all x,y
}

void PoopWorldGenerator::GenerateMapObjectsFromTiles(MAP_OBJ_LIST &listMapObjects, textureLoader &block)
{
  
  // read from the TILE array, and create appropriate MapObjects from them
  // this makes collision detection with buildings easier, as well as 
  // reducing the # of polygons in the scene
  
  MapObject *pObj;
  
  int i,x,y;
  
  for(x = 0; x < m_TileX; ++x)
    for(y = 0; y < m_TileY; ++y)
      {
	i = GetTileIndex(x,y);	
	if(i>=0 && m_pTileArray[i].m_pMapObject == NULL)
	  {
	    // set up the map object
	    
	    if(IS_TILE_TYPE(m_pTileArray[i].m_TileType, TILE_BUILDING))
	      {
		// 20% of buildings will be parks
		if(rand() % 100 < 20)
		  {
		    pObj = new myPark;
		    pObj->setColor(0,0.8,0);
		    pObj->m_Top = 100;
		    pObj->m_texTop = block.texture[12];
		  }
		else
		  {
		    // make it an actual building 
		    pObj = new myBuilding;
		    pObj->m_Top = 100 + 50 *(rand() %20);
		    pObj->m_texTop = block.texture[5];
		    pObj->m_texWest = block.texture[2];
		    pObj->m_texEast = block.texture[3];
		    pObj->m_texNorth = block.texture[4];
		    pObj->m_texSouth = block.texture[0];
		  }
	      }
	    else if (IS_TILE_TYPE(m_pTileArray[i].m_TileType, TILE_INTERSECTION))
	      {
		pObj = new myBuilding;
		pObj->setColor(1,1,1);
		pObj->m_texTop = block.texture[9];
	      }
	    else if (IS_TILE_TYPE(m_pTileArray[i].m_TileType, TILE_ROAD_NS))
	      {
		pObj = new myRoad;
		pObj->setColor(1,1,1);
		pObj->m_texTop = block.texture[6];
	      }
	    else if(IS_TILE_TYPE(m_pTileArray[i].m_TileType, TILE_ROAD_EW))
	      {
		pObj = new myRoad;
		pObj->setColor(1,1,1);
		pObj->m_texTop = block.texture[7];
	      }
	    else if(IS_TILE_TYPE(m_pTileArray[i].m_TileType, TILE_SIDEWALK))
	      {
		pObj = new myRoad;
		pObj->m_texTop = block.texture[8];
	      }
	    else
	      {
		// this is a unknown type
		pObj->setColor(1,0,1);		
	      }
	    
	    GrowBuilding(x,y,pObj, m_pTileArray[i].m_TileType);
	    
	    pObj->m_DisplayList = displayLists.CreateNewList();
	    
	    // let this object set up its own drawing method
	    pObj->SetupDisplayList();
	    
	    // add it to the list
	    listMapObjects.push_back(pObj);
	  }
      }
  
  
  // _DEBUG verify the things
  for(int ll=0; ll < m_TileX * m_TileY; ++ll)
    {
      if(m_pTileArray[ll].m_pMapObject == NULL)
	cout << "Moo!";
    }
  
  
}

void PoopWorldGenerator::GrowBuilding(int x, int y, MapObject *pObj, int type)
{
  int N,S,E,W;
  int good;
  int keep_going = 1;
  int j,k;
  int max_width;
  
  // try to grow this one square 
  N = S = y;
  E = W = x;
  
  if(IS_TILE_TYPE(type, TILE_PARK))
    {
      max_width = 2;
    }
  else if(IS_ONE_OF(type, TILE_ROAD))
    {
      max_width = 1;
    }
  else
    {
      max_width = 4;
    }
  
  // set up the current square
  m_pTileArray[GetTileIndex(x,y)].m_pMapObject = pObj;
  
  // try to grow the boundaries of the square
  while(keep_going)
    {
      keep_going = 0;
      if(E-W <= max_width)
	{
	  good = 1;
	  // grow E
	  for(j = S; j <= N && good; ++j)
	    {
	      k = GetTileIndex(E+1, j);
	      
	      if(k < 0) 
		{
		  // index is off the map, then we can't grow in that direction
		  good = 0;
		}
	      else if(m_pTileArray[k].m_TileType != type)
		{
		  // wrong type of tile, can't grow in that direction
		  good = 0;
		}
	      else if(m_pTileArray[k].m_pMapObject != NULL)
		{
		  // that tile is already in a 
		  good = 0;
		}
	    } // end for all E
	  if(good)
	    {
				// all tiles to the E are ok, so grow to the east
	      ++E;
	      keep_going = 1;
	      for(j = S; j <= N; ++j)
		{
		  m_pTileArray[GetTileIndex(E,j)].m_pMapObject = pObj;
		}
	    }
	}
      // grow W
      
      // grow N
      if(N-S <= max_width)
	{
	  good = 1;
	  for(j = W; j <= E && good; ++j)
	    {
	      k = GetTileIndex(j, N+1);
	      
	      if(k < 0) 
		{
		  // index is off the map, then we can't grow in that direction
		  good = 0;
		}
	      else if(m_pTileArray[k].m_TileType != type)
		{
		  // wrong type of tile, can't grow in that direction
		  good = 0;
		}
	      else if(m_pTileArray[k].m_pMapObject != NULL)
		{
		  // that tile is already in a 
		  good = 0;
		}
	    } // end for all N
	  if(good)
	    {
				// all tiles to the N are ok, so grow to the north
	      ++N;
	      keep_going = 1;
	      for(j = W; j <= E; ++j)
		{
		  m_pTileArray[GetTileIndex(j,N)].m_pMapObject = pObj;
		}
	    }
	}
      // grow S
      
    } 
  
  pObj->m_West = W * m_TileSize - .001;
  pObj->m_East = (E+1) * m_TileSize + .001;
  pObj->m_South = S * m_TileSize - .001;
  pObj->m_North = (N+1) * m_TileSize + .001;
}

int PoopWorldGenerator::GetTileType(int x, int y)
{
  // x and y and the INDEX from the array
  
  int idx = GetTileIndex(x,y);
  if (idx > -1){
    return  m_pTileArray[idx].GetTileType();
  }
  return TILE_INVALID;
}
int PoopWorldGenerator::GetTileTypeFromWorldCoord(int x, int y)
{
  // x and y are COORDINATES in the world. we need to scale them by the size of
  // a tile to get the array index
  return GetTileType(int(x/m_TileSize), int(y/m_TileSize));
}

Tile PoopWorldGenerator::GetTile(int x, int y)
{
  // x and y and the INDEX from the array
  int idx = GetTileIndex(x,y);
  if (idx > -1){
    return  m_pTileArray[idx];
  }
  return Tile(); //Return invalid Tile
}

Tile PoopWorldGenerator::GetTileFromWorldCoord(int x, int y)
{
  // x and y are COORDINATES in the world. we need to scale them by the size of
  // a tile to get the array index
  return GetTile(int(x/m_TileSize), int(y/m_TileSize));
}

/********************* HELPERS ******************************/
int PoopWorldGenerator::GetTileIndex(int x, int y)
{
  // tile references start go from 0 to m_TileX-1 in the x direction and 
  // and 0 to m_TileY-1 in the y direction
  
  //TODO: maybe add validation later
  if ( 
      (x >= m_TileX) || ( y >= m_TileY) 
      ||
      (x < 0)       || ( y < 0 )
      ){
    return -1;
  }
  return (x * m_TileX) + y;
  
}

#endif


