/* Poopmup - An urban flight sim
 *
 * Copyright (C) 2002 Zaroubi, Fink, Leclerc, Stanek szaroubi@hotmail.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* class for maintaining statistics on run-time performance
   hopefully this will help us decide where to optimize things
*/

#ifndef POOP_EM_UP_TIMESTATS_H
#define POOP_EM_UP_TIMESTATS_H

#include "common.h"
#include <fstream>
#include <time.h>

using namespace std;

// if COLLECT_TIMER_STATS is defined, 
// then the macros BEGIN_TIMESTAT(n) and END_TIMESTAT(n)
// will actually do something, otherwise they will be empty
#define COLLECT_TIMER_STATS

#define MAX_TIMESTAT_TIMERS 40
#define TIMESTAT_REPORT_FILE "poopmup_ts.txt"

// timer IDs - for every new event you want to check the time on, 
// add a new ID, and bracket the code with BEGIN_ and END_TIMESTAT(id)
// calls
#define TIMESTAT_OnDisplay				0
#define TIMESTAT_OnIdle					1
#define TIMESTAT_World_Atmozpherizer	2
#define TIMESTAT_World_SetupCamera		3
#define TIMESTAT_World_DrawMapObjects	4
#define TIMESTAT_World_DrawPedestrians	5
#define TIMESTAT_World_DrawCars			6
#define TIMESTAT_World_DrawFries		7
#define TIMESTAT_World_DrawClouds		8
#define TIMESTAT_World_BirdDrawGeometry	9
#define TIMESTAT_World_DrawOSD			10

class myTimeStats
{
public:
  long m_timers[MAX_TIMESTAT_TIMERS];

  // construction
  myTimeStats()
  {
	  // just reset all the timers
	  for(int i = 0; i < MAX_TIMESTAT_TIMERS; ++i)
		  m_timers[i] = 0;
  }
  
  // Functions
  bool IsValidTimerID(int n) {
    return (n >=0 && n < MAX_TIMESTAT_TIMERS);
  }

  void Begin(int n) {
     if(IsValidTimerID(n)) 
        m_timers[n] -= clock();
  }

  void End(int n) {
    if(IsValidTimerID(n))
       m_timers[n] += clock();
  }
  
  void WriteReport(void)
  {
	// dump everything to a file
	  ofstream r;

	r.open (TIMESTAT_REPORT_FILE, ofstream::out );

	r << "PoopmUp TimeStats report:" << endl << endl;
	r << "CLOCKS_PER_SEC defined as: " << CLOCKS_PER_SEC << endl << endl;
	// Top level timers
	r << "Global OnDisplay: " << m_timers[TIMESTAT_OnDisplay] << endl;
	r << "Global OnIdle: " << m_timers[TIMESTAT_OnIdle] << endl;

	r << "World Draw Atmosphere: "		<< m_timers[TIMESTAT_World_Atmozpherizer] << endl;
	r << "World SetupCamera: "			<< m_timers[TIMESTAT_World_SetupCamera] << endl;
	r << "World Draw MapObjs: "			<< m_timers[TIMESTAT_World_DrawMapObjects] << endl;
	r << "World Draw Peds: "			<< m_timers[TIMESTAT_World_DrawPedestrians] << endl;
	r << "World Draw Cars: "			<< m_timers[TIMESTAT_World_DrawCars] << endl;
	r << "World Draw Fries: "			<< m_timers[TIMESTAT_World_DrawFries] << endl;
	r << "World Draw Clouds: "			<< m_timers[TIMESTAT_World_DrawClouds] << endl;
	r << "World Draw bird geometry: "	<< m_timers[TIMESTAT_World_BirdDrawGeometry] << endl;
	r << "World Draw OSD: "				<< m_timers[TIMESTAT_World_DrawOSD] << endl;

	//more timers can go here...

	r.close();

  }
};

#ifdef COLLECT_TIMER_STATS
// the macros for updating these things
#define BEGIN_TIMESTAT(n) (timeStats.Begin(n))
#define END_TIMESTAT(n)   (timeStats.End(n))
#define TIMESTAT_REPORT()   (timeStats.WriteReport())

#else
// empty macros for COLLECT_TIMER_STATS !defined
#define BEGIN_TIMESTAT(n) 
#define END_TIMESTAT(n)   
#define TIMESTAT_REPORT() 

#endif

#endif






