/* Poopmup - An urban flight sim
 *
 * Copyright (C) 2002 Zaroubi, Fink, Leclerc, Stanek szaroubi@hotmail.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* Poop'm'Up world model test
  Stephen Fink
  Oct 26, 2002
*/

#include "common.h"
#include "includes/theWorld/PoopWorld.h"
#include "includes/base/myVector.h"
#include <time.h>
#include "includes/base/myPlane.h"
#include "includes/osd/menu.h"


/* GLOBAL OBJECTS */
PoopWorld theWorld;
myVector speed, gravity;
myDisplayLists displayLists;
myTimeStats timeStats;    

float breakEffect = -PI/40;
float winWidth;
float winHeight;

int mainWnd;
int subWnd = -1;

int currLevel = 1;

/* Optimization options */
unsigned short FAST_COMPUTER = 1; 
int POOP_SHOW_CLOUDS = 1;
int POOP_SHOW_TREES = 1;
int POOP_TEXTURE_2D = GL_TEXTURE_2D;

unsigned short POOP_WINDOW_MODE = 0;
unsigned short POOP_LIGHTING = 1;
unsigned short POOP_GAME_SPEED = 60;


GLuint POOP_SHADING_TYPE = GL_SMOOTH;

/* MOUSE MOVEMENTS */
float mouseX = 0;
float mouseY = 0;
int mouseDown = 0;
int mouseUp = 0;
int mouseButton = 0;

float hVect;
float vVect;
float upXDisp;
float upYDisp;
float upX;
float upY;
float upZ;

float theta;
float phi;
float R = 100;
float pi = 3.1416;
int prevMouseX = 0;
int prevMouseY = 0;
int dX = 0;
int dY = 0;
int mflag = 0;
float birdSpeedInc = 0; 

/* ARROW KEYS */
int idleCounter = 0;

int inMenu = 1;

unsigned long frameCount = 0;
unsigned long currTime;
unsigned long goTime = 0;
unsigned long displayTimer;

//Is the world moving, no... but the bird keeps on living.
int theWORLDisPAUSED = 0; 

/* menu stuff */
//Menu stuff must be included after all global vars
//Because it will use some of them within the menu.
#include "myConfig.h" //Include before menufuncs.h
#include "menufuncs.h"



/* CALLBACKS */
void deletePoopOCam();
void createPoopOCam();

void OnDisplay(void)
{	
  BEGIN_TIMESTAT(TIMESTAT_OnDisplay);
	
	displayTimer = clock();
  //glutSetWindow(mainWnd);
  displayLists.SetCurrWindow(0);
  frameCount++;

  //Don't need the clear since we have a box aroung ourWorld.
  //And we are shure that the bird will not fly out
  glClear(GL_COLOR_BUFFER_BIT |  GL_DEPTH_BUFFER_BIT );	

  theWorld.Draw();	

  theWorld.setOrthographicProjection();
  //DRAW THE MENUS INSIDE THE ORTHOGRAPHIC
  if (mainmenu.active())   mainmenu.draw();
  if (configMenu.active()) configMenu.draw();
  theWorld.resetPerspectiveProjection();

  glutSwapBuffers();

  END_TIMESTAT(TIMESTAT_OnDisplay);	
}

void requestRedisplay()
{
  glutSetWindow(mainWnd);
  glutPostRedisplay();
  if(subWnd != -1){
    glutSetWindow(subWnd);
    glutPostRedisplay();
  }
}


void OnSpecialKeyboard(int key, int x, int y)
{
  switch(key)		
    {
      // UP and DOWN control the bird's speed		
      // in the lookAt direction
    case GLUT_KEY_UP:
      birdSpeedInc = 2;
      break;		
    case GLUT_KEY_DOWN:
      birdSpeedInc = -3;
      break;						
    case GLUT_KEY_LEFT:
      break;
    case GLUT_KEY_RIGHT:		
      break;		
    }
  requestRedisplay();	
}

void OnMouseClick(int button, int state, int x, int y){	
  mouseButton = button; 
  if (state == GLUT_DOWN){
    mouseDown = 1;
    mouseUp = 0;
  }
  else{
    //OnButtonUp
    mouseUp = 1;
    mouseDown = 0;
    if (button == GLUT_LEFT_BUTTON){
      if (!inMenu){
	theWorld.bird.dropPoop();
      }
    }	
    else if (button == GLUT_MIDDLE_BUTTON){
    }	
    else if (button == GLUT_RIGHT_BUTTON){		
      
    }
    mouseDown = 0;
  }
  
  mouseX = x;
  mouseY = y;
}

void OnKeyboard(unsigned char key, int x, int y)
{	
  switch(key)		
    {		
    case 27:	
      // ESC KEY
      mainmenu.setLabel("MAIN MENU");
      mainmenu.enable();
      break;
    case 13:	
      // ENTER KEY
      mainmenu.disable();
      break;
    case 32:
      // SPACEBAR
      theWorld.bird.dropPoop();
      break;
    case '+':
      theWorld.osd_Radar.incViewDist(0.1);
      theWorld.osd_Radar.incRadius(0.1);
      break;		
    case '-':		
      theWorld.osd_Radar.decViewDist(0.1);
      theWorld.osd_Radar.decRadius(0.1);
      break;	
    case 'w':
    case 'W':
      birdSpeedInc = 2;
      break;
    case 's':
    case 'S':
      birdSpeedInc = -3;
      break;
    case 'm':
    case 'M':
      mainmenu.enable();
      break;
    case 'g':
      theWORLDisPAUSED = 1 - theWORLDisPAUSED;
      theWorld.isPaused = theWORLDisPAUSED;
      break;
    case '1':
      deletePoopOCam();
      break;
    case '2':
      createPoopOCam();
      break;
    }		
  requestRedisplay();	
}

/* MOUSE INPUT */
void OnMouseMove(int x, int y)
{	
  mouseX = x;
  mouseY = y;

  hVect = (mouseX-winWidth/2)/(winWidth/2);
  vVect = (mouseY-winHeight/2)/(winHeight/2); 
  
  theWorld.mousePos.setXYZ(
			   mouseX,
			   mouseY-winHeight/2,
			   0
			   );
  
  if (mainmenu.active())
    mainmenu.cursorPos.setXYZ(x,y-winHeight/2,0);
  if (configMenu.active())
    configMenu.cursorPos.setXYZ(x,y-winHeight/2,0);
} 

void reshapeWindow(int w, int h)
{	
  winWidth = w;	
  winHeight = h;	
  glViewport(0, 0, w, h);	
  glMatrixMode(GL_PROJECTION);	
  glLoadIdentity();	
  gluPerspective(70, GLfloat(w) / GLfloat(w), 0.1, 10000);	

  theWorld.setOrthProjSize(w,h);

  mainmenu.setX((int) winWidth/5);
  mainmenu.setY((int) winHeight/4);
  configMenu.setX((int) winWidth/5);
  configMenu.setY((int) winHeight/4);
  if (subWnd != -1){
    glutSetWindow(subWnd);
    glutPositionWindow((int) (winWidth * 12.5/15),(int) (winHeight * 1/30));
    glutReshapeWindow((int) (winWidth * 2/15),(int) (winWidth*2/15));
    glutSetWindow(mainWnd);
  }

}



void OnIdle()
{	
	
	currTime = clock();
  inMenu = mainmenu.active() + configMenu.active();

  if(currTime > goTime)
  {
	  // set the next time slice
	  goTime = currTime + (CLOCKS_PER_SEC/POOP_GAME_SPEED);
  }
  else
  {
	  // its too early
	  return;
  }


  if (!theWorld.bird.isAlive()){
    mainmenu.setLabel("Ohhh you died!");
    mainmenu.enable();
  }

  if (theWorld.m_objectives.timeout()){
    mainmenu.setLabel( "Ohhh you took too long!");
    mainmenu.enable();
  }
  // update the time
  if (!inMenu){
	BEGIN_TIMESTAT(TIMESTAT_OnIdle);

	glutSetCursor(GLUT_CURSOR_NONE);
    
    
    // decrease bird's flying speed	
    ++idleCounter;      
    
    //Move the Bird
    theWorld.bird.incSpeed(birdSpeedInc);
    birdSpeedInc = 0;
    
    
    theWorld.bird.turnBy(hVect, vVect);
    //if the mouse button is down

    //animate the world
    theWorld.Update();	
  
	END_TIMESTAT(TIMESTAT_OnIdle);

    // check for objective completion
  // check if we've met objectives - of so, go to next level
  if(theWorld.m_objectives.IsObjectiveMet(ALL_OBJECTIVES))
    {
      char buf[60];
      currLevel = theWorld.StartNextLevel();
      sprintf(
	      buf, 
	     "New mission: Hit %d Peds, %d Cars and eat %d fries",
	      theWorld.m_objectives.getPedsLeft(),
	      theWorld.m_objectives.getCarsLeft(),
	      theWorld.m_objectives.getFriesLeft()
	      ); 

      mainmenu.setLabel("Mission Complete");
      mainmenu.setMenuDescription(buf);
      mainmenu.enable();
    }

  }
  else{
    //ANIMATION OF THE MENU
    //glutSetCursor(GLUT_CURSOR_LEFT_ARROW);

    if (mainmenu.active()){
      mainmenu.animate();
      mainmenu.mouseButton = mouseButton;
      mainmenu.mouseDown =  mouseDown;
      mainmenu.mouseUp = mouseUp;
    }
    if (configMenu.active()){
      configMenu.mouseButton = mouseButton;
      configMenu.mouseDown =  mouseDown;
      configMenu.mouseUp = mouseUp; 
      configMenu.animate();
    }
  }
  mouseDown = mouseUp = 0;
  //Tell glut that the world needs to be redisplayed
  requestRedisplay();	
	
}

/* Poopocam subwindow handler */
void OnDisplayPoopoCam()
{

  //glutSetWindow(subWnd);
  displayLists.SetCurrWindow(1);

  glClear(GL_COLOR_BUFFER_BIT | GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );	
  glEnable( GL_DEPTH_TEST );
  glEnable( GL_NORMALIZE);
  
  

  theWorld.DrawPoopoCam();	

  glutSwapBuffers(); 
}



/* INITIALIZATION */
void InitializeGL(int argc, char **argv)
{	
  // Initialize everything for OpenGL	
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH);	
  glutInitWindowSize(800,800);	
  mainWnd = glutCreateWindow("::Poopmup::");	
  if (!POOP_WINDOW_MODE)
    glutFullScreen();

  glutSetCursor(GLUT_CURSOR_NONE); //New
  glutDisplayFunc(OnDisplay);	
  glutKeyboardFunc(OnKeyboard);	
  glutSpecialFunc(OnSpecialKeyboard);	
  glutPassiveMotionFunc(OnMouseMove);	
  glutMotionFunc(OnMouseMove);
  glutReshapeFunc(reshapeWindow);
  glutMouseFunc(OnMouseClick);
  glutIdleFunc(OnIdle);	

  glEnable( GL_DEPTH_TEST );
  glEnable( GL_NORMALIZE);

  if ( POOP_LIGHTING ) glEnable(GL_LIGHTING);
  if ( POOP_LIGHTING ) glEnable(GL_LIGHT0);
  
  theWorld.currentblock = 0;
  theWorld.loadTextures();


  if ( POOP_LIGHTING ) glEnable(GL_LIGHTING);
  if ( POOP_LIGHTING ) glEnable(GL_LIGHT0);
  
  theWorld.currentblock = 1;
  theWorld.loadTextures();

  //createPoopOCam();
  displayLists.setWindow(0, mainWnd);

  displayLists.CreateAllLists();

  // gop back to the main window
  displayLists.SetCurrWindow(0);
  //glutSetWindow(mainWnd);

}

void deletePoopOCam(){
  if (subWnd != -1){
    glutDestroyWindow(subWnd);
    displayLists.setWindow(1, -1);
    subWnd = -1;
    theWorld.bird.poopOCamEnabled = 0;
  }

}
void createPoopOCam(){
  // now set up the sub window - the poopocam
  if (subWnd == -1){
    subWnd = glutCreateSubWindow(
				 mainWnd, 
				 (int) winWidth * 7/9,
				 (int) winHeight * 1/9,
				 200,
				 200
				 );

    theWorld.bird.poopOCamEnabled = 1;
    glutDisplayFunc(OnDisplayPoopoCam);
    displayLists.setWindow(1, subWnd);
    displayLists.SetCurrWindow(1);
    theWorld.loadTextures();
    theWorld.loadDisplayLists();
    if ( POOP_LIGHTING )glEnable(GL_LIGHTING);
    if ( POOP_LIGHTING )glEnable(GL_LIGHT0);

  }
}

void InitializeWorld(void)
{	
  // Initialize the non-GL world objects	
  srand(time(NULL));	
  
  theWorld.currentblock = 0;

  // generate display lists for main window
  displayLists.SetCurrWindow(0);

  // Set the size of the world
  theWorld.SetTileArray(100,100,100);	
  
  // set up the first set of objectives
  theWorld.StartLevel(1);
  
  // Set to perspective mode
  theWorld.bird.setModePerspective();	
  
  // Initialize the on screen Displays
  theWorld.initOnScreenDisplay();
  
  // Initialize the forces
  speed.setXYZ(0,0,0);	
  gravity.setXYZ(0,0,0);	
  
  // Place the bird correctly in the world
  theWorld.bird.moveTo(5000,5000, 1500);
  theWorld.bird.setLookAt(0,0,0);

  // Init the menu
  mainmenu.enable();

}

void ParseCommandLine(int argc, char**argv)
{
  /* check for command line switches. Currently supported switches are:
     --fast-machine[=n] (default is 1)
  */
  
  char *pstr;

  for(int i = 1; i < argc; ++i)
    {
      if(0 == strncmp(argv[i], "--fast-computer", 15))
	{
	  pstr = argv[i] + 15;
	  if(pstr[0] == 0)
	    {
				// use default
	      FAST_COMPUTER = 1;
	    }
	  else
	    {
				// subtract '0' to get numerical value
	      FAST_COMPUTER = pstr[1] - '0';
	    }
	  
	}
      if(0 == (strcmp(argv[i], "--no-textures")))
	{
	  POOP_TEXTURE_2D = 0;
	}
      if(0 == (strcmp(argv[i], "--flat-shading")))
	{
	  POOP_SHADING_TYPE = GL_FLAT;
	}
      if ( 0 == (strcmp(argv[i], "--no-clouds")))
	{
	  POOP_SHOW_CLOUDS = 0;
	}

      if ( 0 == (strcmp(argv[i], "--no-trees")))
	{
	  POOP_SHOW_TREES = 0;
	}

      if ( 0 == (strcmp(argv[i], "--window-mode")))
	{
	  POOP_WINDOW_MODE = 1;
	}

      if ( 0 == (strcmp(argv[i], "--no-lighting")))
	{
	  POOP_LIGHTING = 0;
	}
    }
}


/* MAIN */
int main(int argc, char **argv)
{	
  
  //Get config commands from file
  getConfiguration();
  //Will overwrite the conf file params.
  ParseCommandLine(argc, argv); 
  InitializeGL(argc, argv);	
  InitializeWorld();	
  InitMenu();
  glutMainLoop();	
  return 0;
}

















